
c Program FASRATIO

c Computes the ratio of Fourier amplitude spectra for specified time series.  
* The program reads a list of time series, and an output 
* file containing the spectral ratios is written.
* A summary file is also created.  The intended use is to provide input to
* CoPlot on the PC.

* The time series to be used in the ratio are given in sequential lines.
* Here is a sample input file:

*summary file name:
* watr.sum
*output file name:
* watr.out
*df_intrp, f_intrp_low, f_intrp_high:
*   0.1  0.2 7.
*column heading, numerator, denominator files, blank delimitor:
* ssz_ssn 
*  watr_ssz.smc 13.5 25.0 10 2.0
*  watr_ssn.smc 13.5 25.0 10 2.0
*
* ssz_sse 
*  watr_ssz.smc 13.5 25.0 10 2.0
*  watr_sse.smc 13.5 25.0 10 2.0
*
* rsz_rsr 
*  watr_rsz.smc 13.5 25.0 10 2.0
*  watr_rsr.smc 13.5 25.0 10 2.0
*
* rsz_rst 
*  watr_rsz.smc 13.5 25.0 10 2.0
*  watr_rst.smc 13.5 25.0 10 2.0
* stop

* Assumptions:

*    - Number of frequency output points is less than 5000.

*    - The number of spectral ratios is 22 or less (22 ratios is the max 
*      so that recl <= 255)

*    - With one exception, no assumption is made about equality of dt or npts
*      for each time series when computing spectra.  
*      The exception for frequency is if df_intrp = 0, in which
*      case the output frequencies are the same as the fft frequencies 
*      (within the bounds specified by the low and high interpolation 
*      parameters), and these must be the same for each spectrum written to
*      a file, since I assume a common frequency axis (although I could
*      write separate frequency columns for each spectrum).

*    - time series less than 8200 points

* Dates: 12/16/97 - Written by D. M. Boore.
*                   This is a revision of an earlier version, using
*                   TS2FAS and the old FASRATIO as a guide.  
*        12/04/00 - added ibgn, iend to rcf, rci, rcc calling arguments

c
c Dimension and declaration statements:

      real fas1(5000), fas2(5000), ratio(5000,22),
     :     freq_out(5000), tsdum(1)

      integer  iprcntfrtaper1, iprcntfrtaper2

      character date_c*10, time_c*11, buffer*80,
     :        ts_name1*80, ts_name2*80, 
     :        bigbuf*380, f_in*40, f_sum*40, f_out*40,
     :        col_head(22)*8

      logical f_in_exist

c Initializations:

       nu_in  = 3
       nu_out= 20
       nu_sum = 7

* Get name of input file:

      f_in_exist = .false.
      do while (.not. f_in_exist)
        f_in = ' '
        write(*, '(a\)') 
     :    ' Enter name of file with parameters (cr to quit): '
        read(*, '(a)') f_in
        if (f_in(1:4) .eq. '    ') stop
        inquire(file=f_in, exist=f_in_exist)
        if (.not. f_in_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      open(unit=nu_in, file=f_in, status='unknown')

      read(nu_in, *)
      f_sum = ' '
      read(nu_in,'(a)') f_sum
      open(unit=nu_sum, file=f_sum, status='unknown')

      read(nu_in, *)
      f_out = ' '
      read(nu_in,'(a)') f_out

* Read interpolation parameters:

      read(nu_in, *)
      read(nu_in, *) df_intrp, f_intrp_low, f_intrp_high
      read(nu_in, *)

* Set up the frequency array:

      if (df_intrp .eq. 0.0) then
         mspct = 0
         freq_out(1) = f_intrp_low
         freq_out(2) = f_intrp_high
      else
         mspct = (f_intrp_high-f_intrp_low)/df_intrp + 1
         do i = 1, mspct
           freq_out(i) = float(i-1) * df_intrp + f_intrp_low
         end do
      end if

c Date and time stamp:

      call get_date(date_c)
      call get_time(time_c)

* Write header info in summary file:

      write(nu_sum,*)
      write(nu_sum, '(1x,a,1x,a,1x,a,1x,a)')
     :  ' Date: ', date_c, '; Time: ', time_c
      write(nu_sum,*)
      write(nu_sum, '(1x,a,1x,a)') ' Input file  : ', f_in
      write(nu_sum, '(1x,a,1x,a)') ' Summary file: ', f_sum
      write(nu_sum, '(1x,a,1x,a)') ' Output file : ', f_out
      write(nu_sum,*)
      write(nu_sum, '(1x,a,f5.3,a,f6.2,a,i4)' )
     :       ' df_intrp= '     , df_intrp, 
     :       ' f_intrp_high= ' , f_intrp_high,
     :       ' mspct= '        , mspct
       

c
c----------------------------------------------------------------
c LOOP OVER RATIOS. 
c

      nrat = 0

1000  continue
      

* Read 3 lines, the first with the column heading, and the next two
* with the filename, start times, window length, 
* percent taper at front (the back taper is assumed to be the same length),
* and the smoothing (in terms of the width of the smoothing window
* in Hz).
* The ratio pairs are separated by a blank line.
* Mark the end of the list with a line having "stop" starting on column 2.

      buffer = ' '
      read (nu_in, '(a)', end=9999) buffer
      call upstr(buffer)
      if(buffer(2:5) .eq. 'STOP') go to 9999

      nrat = nrat+ 1

*      write(col_head(nrat),'(a8)') buffer
      col_head(nrat) = buffer(1:8)

      write(nu_sum, *)
      write(nu_sum, '(1x,a,1x,a)') ' col_head: ', col_head(nrat)

* Read numerator line:
      buffer = ' '
      read (nu_in, '(a)', end=9999) buffer
* Now extract the filename, start time, window length, etc:
      call RCC(1, buffer, 80, ts_name1      , nc, ibgn,iend)
      call RCF(2, buffer, 80, tstartsignal1, ibgn, iend, istat)
      call RCF(3, buffer, 80, tlengthsignal1, ibgn, iend, istat)
      call RCI(4, buffer, 80, iprcntfrtaper1 , ibgn,iend, istat)
      call RCF(5, buffer, 80, smoo1         , ibgn, iend, istat)

* Read denominator and extract fields:
      buffer = ' '
      read (nu_in, '(a)', end=9999) buffer
      call RCC(1, buffer, 80, ts_name2      , nc, ibgn, iend)
      call RCF(2, buffer, 80, tstartsignal2 , ibgn, iend, istat)
      call RCF(3, buffer, 80, tlengthsignal2, ibgn, iend, istat)
      call RCI(4, buffer, 80, iprcntfrtaper2 , ibgn,iend, istat)
      call RCF(5, buffer, 80, smoo2         , ibgn, iend, istat)
      
* Get the smoothed spectrum of file 1:

      call Get_Spectrum(ts_name1,
     :     tstartsignal1, tlengthsignal1, iprcntfrtaper1, smoo1, 
     :     dt, npts, npw2, df, fas1, tsdum,
     :     freq_out, mspct, nu_sum)
 
 
* Get the smoothed spectrum of file 2:

      call Get_Spectrum(ts_name2,
     :     tstartsignal2, tlengthsignal2, iprcntfrtaper2, smoo2, 
     :     dt, npts, npw2, df, fas2, tsdum,
     :     freq_out, mspct, nu_sum)

* Compute the spectral ratio 
 
      do ifreq = 1, mspct
        if (fas2(ifreq) .eq. 0.) then
            denom = 1.0e-20
        else
            denom = fas2(ifreq)
        end if
        ratio(ifreq, nrat) = fas1(ifreq)/denom
      end do
  
c Loop back for another station

* First skip past the blank line (and check for a STOP)

      buffer = ' '
      read (nu_in, '(a)', end=9999) buffer
      call upstr(buffer)
      if(buffer(2:5) .eq. 'STOP') go to 9999

      go to 1000

9999  continue

* Close input file

      close(unit=nu_in)

* Write the spectra  (note: see the block of commented statements at the end
* for writing the time series).

* Figure out record length and open the file:

      irecord_length_rat = 11*nrat + 8

      open(unit=nu_out, file=f_out, recl = irecord_length_rat,
     :     status='unknown')

*   First write the header:
      
      bigbuf = ' '
      bigbuf(5:8) = 'FREQ'

      do irat = 1, nrat
           iseg = 11*(irat-1)
           bigbuf(iseg+12:iseg+19) = 
     :       col_head(irat)
     :       //'_____________'     ! Add this in case col_head < 8 characters
      end do

      write(nu_out, '(a)') bigbuf(1:irecord_length_rat)

*   Now write the Fourier spectral ratios:

      do i = 1, mspct
        bigbuf = ' '
        write(bigbuf(2:8),'(f7.3)') 
     :          freq_out(i)
        do irat = 1, nrat
          iseg = 11*(irat-1)
          write(bigbuf(iseg+10:iseg+19),'(1pe10.3)') 
     :          ratio(i,irat)
        enddo
        write(nu_out, '(a)') bigbuf(1:irecord_length_rat)
      end do

* Close file containing the Fourier spectral ratios
      close(unit=nu_out)

* Close summary file
      close(unit=nu_sum)

      stop
      end

* ------------------------ BEGIN Get_Spectrum ----------------------      
      subroutine Get_Spectrum(fname,
     :     tstartsignal, tlengthsignal,iprcntfrtaper, smoo,
     :     dt, npts, npw2, df, spect_out, ts,
     :     freq_out, mspct, nu_sum)
 
* Note that this version of Get_Spectrum includes smoo rather than nhits
* in the argument list (nhits is an argument of the version of
* Get_Spectrum in BH_SPECT).  In all cases Get_Spectrum is bundled 
* with the main program, so changes in the arguments should not lead to
* confusion.

      real spect_out(*), freq_out(*), ts(*)
      real data(8200), freq(4200), spect(8200)

      integer  iprcntfrtaper

      character fname*80

      character char_head(11)*80
      integer int_head(48)
      real real_head(50)

c Initializations:

      iudata = 4

      write(*,'(1x,a)') 'Processing file:'//fname

      write(*,'(2(1x,f7.3), 1x,i2,1x,f5.2)') tstartsignal, 
     :               tlengthsignal, iprcntfrtaper, smoo

      write (nu_sum, 30) fname,
     *                       tstartsignal,tlengthsignal,
     :                       iprcntfrtaper, smoo

30    format(2x,'data file= ',a/
     : 3x,'signal: start=',f7.3,' length=',f7.3,
     : ' iprcnttaper= ', i2, ' smoo= ',f5.2)

* Read the data:

      call ReadSMC(iudata, fname, tstartsignal, tlengthsignal,
     :     data, npts, sps, char_head, int_head, real_head)

      dt = 1.0/sps
     
*      do i = 1, npts
*        ts(i) = data(i)
*      end do

      write(nu_sum,4210) dt, npts
4210  format(3x,'signal: dt=',f8.6,' npts=',i5,30('.'))

* Compute the Fourier spectra:

      call Abs_Spectra(data, dt, npts, iprcntfrtaper, 
     :                 spect, npw2, df)

*      do i = 1, npw2
*        ts(i) = data(i)
*      end do

      write(nu_sum,4211) df, npw2
4211  format(3x,'spectra: df=',f7.4,' npw2=',i5,30('.'))

* Smooth the spectra

      ihwid = int( smoo/(2*df) )
      if (ihwid .lt. 1) ihwid = 1
      call SMTHS (spect, npw2, df, ihwid, 1)

* Interpolate to specified frequencies:

* First set up the frequency array for the spectra:

      nfreq = npw2/2 
      do i = 1, nfreq
        freq(i) = float(i-1)*df
      end do

* Now interpolate

* If mspct = 0, use fft frequencies:

      if (mspct .eq. 0) then
        fstart = freq_out(1)
        fstop = freq_out(2)
        call locate(freq, nfreq, fstart, m_start)
         m_start = m_start + 1
        call locate(freq, nfreq, fstop, m_stop)
         m_stop = m_stop - 1
        mspct = m_stop - m_start + 1
        do i = 1, mspct
         freq_out(i) = freq(i + m_start - 1)
        end do
      end if       

      do i = 1, mspct
         spect_out(i) = yintrf(freq_out(i), freq, spect, nfreq)
      end do
 
      return
      end
* ------------------------ END Get_Spectrum ----------------------      

* ------------------------ BEGIN Abs_Spectra ---------------------------
      subroutine Abs_Spectra( datain, dt, npts, 
     :    iprcntfrtaper, spect, npw2, df_fft)

c Returns the absolute value of the spectrum.
c The program applies a tapered window to the
c front and back of the time series, pads with zeros,  and computes the
c spectrum.

c Written by D. M. Boore

* Dates:  3/22/89 - Modified from SqrSpectFT, by removing
*                   the smoothing and interpolation.
*        11/08/89 - Pass df_fft out through the argument list.
*         4/07/95 - Declared prcntfrtaper as an integer.  I wonder
*                   how many bad results were obtained as a result
*                   of this error?!

      real DATAIN(*), SPECT(*)
      real DATA(8200)
      complex CX(8200)
      integer iprcntfrtaper

c Fill working array with input data:

      do 1 i = 1, npts
      data(i) = datain(i)
1     continue

c
c     remove dc, apply taper, pad with zeros
c

      call dcdt(data, dt, npts, 1, npts, .true., .false.)

      ifront = iprcntfrtaper
      iback = iprcntfrtaper
      call fbctpr (ifront, iback, data, npts)

      call zeropad2( data, npts, npts, npw2)

c sample rate and frequency spacing:

      sr = 1.0 / dt
      df = sr/ float(npw2)
      df_fft = df

      npw2d2 = npw2 / 2

c Fill working complex array

      do 90 j = 1, npw2
        cx(j)=cmplx(data(j), 0.0)
        datain(j)=data(j)  ! To check if tapering, etc was OK
90    continue

c     FFT to get spectrum

      call fork(npw2,cx,-1.)

      do 100 j =1 ,npw2d2
        cx(j)=cx(j)*dt*sqrt(float(npw2))
        spect(j)=cabs( cx(j) )
100   continue

      return
      end
* ------------------------ END Abs_Spectra ---------------------------

* ------------------------ BEGIN SMTHS ---------------------------
      subroutine SMTHS (Z,IPW2,DF,IHWID,IMODE)

c SMTHS - smooth amplitude spectrum Z with a triangle window.
c         Assume Z(1) corresponds to DC; smooth up to Nyquist.
c         If IMODE = 1, smooth amplitudes.
c         If IMODE = 2, smooth log amplitudes.
c Smooth index range 2...IPW2/2;
c shift results to IPW2/2+2...IPW2 and shift back to 2...IPW2/2 at the end.
c Smoothing triangle base has odd integer number of points (2*IHWID+1);
c the two outside points get zero weight, so IHWID<=1 implies no smooth.
C Notice that beginning and end of spectrum are only partially smoothed.
c Written by Chuck Mueller, USGS.

* Dates:  xx/xx/xx - Written by Chuck Mueller
*          2/ 6/90 - IHWID set to '1' if it is less than unity.
*                    This was done because in PROCESS IHWID is 
*                    determined by IHWID = nint(DF_SMOOTH/DF_FFT),
*                    and this evaluates to 0 if DF_SMOOTH is very
*                    small (as is the case if no smoothing is desired).


      real Z(IPW2)

      if (ihwid .lt. 1) ihwid = 1

      ISHIFT = IPW2/2
10    do 13 I=1,IHWID
         FACT = 1./I
         WSUM = 0.
         ZSUM = 0.
         if (IMODE.eq.1) then
           do 11 J=1,2*I-1
              W = (I-IABS(I-J))*FACT
              WSUM = WSUM+W
11            ZSUM = ZSUM+W*Z(J+1)
           Z(I+1+ISHIFT) = ZSUM/WSUM
         elseif (IMODE.eq.2) then
           do 12 J=1,2*I-1
              W = (I-IABS(I-J))*FACT
              WSUM = WSUM+W
12            ZSUM = ZSUM+W*ALOG10(Z(J+1))
           Z(I+1+ISHIFT) = 10.**(ZSUM/WSUM)
         endif
13       continue
20    FACT = 1./IHWID
      do 23 I=IHWID+1,IPW2/2-(IHWID+1)
         WSUM = 0.
         ZSUM = 0.
         if (IMODE.eq.1) then
           do 21 J=I-IHWID+1,I+IHWID-1
              W = (IHWID-IABS(I-J))*FACT
              WSUM = WSUM+W
21            ZSUM = ZSUM+W*Z(J+1)
           Z(I+1+ISHIFT) = ZSUM/WSUM
         elseif (IMODE.eq.2) then
           do 22 J=I-IHWID+1,I+IHWID-1
              W = (IHWID-IABS(I-J))*FACT
              WSUM = WSUM+W
22            ZSUM = ZSUM+W*ALOG10(Z(J+1))
           Z(I+1+ISHIFT) = 10.**(ZSUM/WSUM)
         endif
23       continue
30    do 33 I=IPW2/2-IHWID,IPW2/2-1
         FACT = 1./(IPW2/2-I)
         WSUM = 0.
         ZSUM = 0.
         if (IMODE.eq.1) then
           do 31 J=IPW2/2-2*(IPW2/2-I)+1,IPW2/2-1
              W = (IHWID-IABS(I-J))*FACT
              WSUM = WSUM+W
31            ZSUM = ZSUM+W*Z(J+1)
           Z(I+1+ISHIFT) = ZSUM/WSUM
         elseif (IMODE.eq.2) then
           do 32 J=IPW2/2-2*(IPW2/2-I)+1,IPW2/2-1
              W = (IHWID-IABS(I-J))*FACT
              WSUM = WSUM+W
32            ZSUM = ZSUM+W*ALOG10(Z(J+1))
           Z(I+1+ISHIFT) = 10.**(ZSUM/WSUM)
         endif
33       continue
40    do 41 I=2,IPW2/2
41       Z(I) = Z(I+ISHIFT)
      return

      end
* ------------------------ END SMTHS ---------------------------

* ------------------------ BEGIN DCDT -------------------
      SUBROUTINE DCDT (Y,DT,NPTS,INDX1,INDX2,LDC,LDT)
c+
c  DCDT - Fits DC or trend between indices INDX1 and INDX2.
C         Then removes DC or detrends whole trace.
c         Y is real, DT = delta t.
c         If remove DC, LDC = .TRUE.
c         IF detrend, LDT = .TRUE.
c-
      real Y(1)
      logical LDC,LDT
c
c...Fit DC and trend between indices INDX1 and INDX2.
100   NSUM = INDX2-INDX1+1
      SUMX = 0.0
      SUMX2 = 0.0
      SUMY = 0.0
      SUMXY = 0.0
      DO 200 I=INDX1,INDX2
         XSUBI = (I-1)*DT
         SUMXY = SUMXY+XSUBI*Y(I)
         SUMX = SUMX+XSUBI
         SUMX2 = SUMX2+XSUBI*XSUBI
200      SUMY = SUMY+Y(I)
C
C... Remove DC.
300   IF (LDC) THEN
        AVY = SUMY/NSUM
        DO 360 I=1,NPTS
360        Y(I) = Y(I)-AVY
        RETURN
      ENDIF
C
C... Detrend. See Draper and Smith, p. 10.
400   IF (LDT) THEN
        BXY = (SUMXY-SUMX*SUMY/NSUM)/(SUMX2-SUMX*SUMX/NSUM)
        AXY = (SUMY-BXY*SUMX)/NSUM
        QXY = DT*BXY
        DO 450 I=1,NPTS
450        Y(I) = Y(I)-(AXY+(I-1)*QXY)
        RETURN
      ENDIF
C
      STOP
      END
* ------------------------ END DCDT -------------------

* ------------------------ BEGIN FBCTPR -------------------
      subroutine FBCTPR (IFRONT,IBACK,Z,NZ)

c  Apply IFRONT% and IBACK% cosine tapers to ends of time series array Z.
c  Written by Chuck Mueller, USGS.
c  Modified by D. M. Boore on 8/31/88 to eliminate the use of ZNULL;
c  see FBCTPR_CSM for the original version.

*  Dates:  2/13/90 - if ifront or iback is zero, do not apply a taper.

      real Z(*)

      PI = 4.0*ATAN(1.0)
      LZ = NZ*(IFRONT/100.)

      if (lz .lt. 1) lz = 1

      SF = PI/LZ
      do 1 I=1,LZ
c         if (Z(I).eq.ZNULL) goto 1
         F = 0.5*(1.0-COS(SF*(I-1)))
         Z(I) = Z(I)*F
1        continue
      LZ = NZ*(IBACK/100.)

      if (lz .lt. 1) lz = 1


      SF = PI/LZ
      do 2 I=NZ,NZ-LZ+1,-1
c         if (Z(I).eq.ZNULL) goto 2
         F = 0.5*(1.0-COS(SF*(NZ-I)))
         Z(I) = Z(I)*F
2        continue

      return
      end
* ------------------------ END FBCTPR -------------------

* ------------------------ BEGIN ZEROPAD2 -------------------
      subroutine ZEROPAD2 (Y,NIN,NTOTIN,NPW2)

c Calculates NPW2, the next power-of-2 greater than NTOTIN.
c Pads time-series array Y with (NPW2-NIN) zeroes.  With this program 
c the window of the data, which determines NIN, can be different 
c for different time series, yet the overall length of 
c the time series used in the FFT can 
c be the same (determined by NTOTIN), thus guaranteeing 
c that the frequencies for which FFT
c values are computed are the same.
c I assume that the user makes sure that NTOTIN .ge. NIN.
c Written by Chuck MUeller, USGS; modified by Dave Boore,
c 5/1/87.

      real Y(*)
      
      npw2 = 2
1     if ( npw2 .lt. ntotin) then
           npw2 = 2 * npw2
           go to 1
      endif

      do 20 I=NIN+1,NPW2
20       Y(I) = 0.0

      return
      end
* ------------------------ END ZEROPAD2 -------------------

* ------------------------ BEGIN FORK -------------------
      SUBROUTINE FORK(LX,CX,SIGNI)
C FAST FOURIER                                  2/15/69
C                          LX
C    CX(K) = SQRT(1.0/LX)* SUM (CX(J)*EXP(2*PI*SIGNI*I*(J-1)*(K-1)/LX))
C                          J=1                        FOR K=1,2,...,LX
C
C  THE SCALING BETWEEN FFT AND EQUIVALENT CONTINUUM OUTPUTS
C  IS AS FOLLOWS.
C
C
C     GOING FROM TIME TO FREQUENCY:
C             F(W)=DT*SQRT(LX)*CX(K)
C
C                  WHERE W(K)=2.0*PI*(K-1)/(LX*DT)
C
C
C     GOING FROM FREQUENCY TO TIME, WHERE THE FREQUENCY
C     SPECTRUM IS GIVEN BY THE DIGITIZED CONTINUUM SPECTRUM:
C
C             F(T)=SQRT(LX)/(LX*DT)*CX(K)
C                  WHERE T(K)=(K-1)*DT
C
C
C  THE RESULT OF THE SEQUENCE...TIME TO FREQUENCY,POSSIBLE MODIFICATIONS
C  OF THE SPECTRUM (FOR FILTERING,ETC.), BACK TO TIME...
C  REQUIRES NO SCALING.
C
C
C  THIS VERSION HAS A SLIGHT MODIFICATION TO SAVE SOME TIME...
C  IT TAKES THE FACTOR 3.1415926*SIGNI/L OUTSIDE A DO LOOP (D.BOORE 12/8
C  FOLLOWING A SUGGESTION BY HENRY SWANGER).
C
      COMPLEX CX,CARG,CEXP,CW,CTEMP
      DIMENSION CX(LX)
      J=1
      SC=SQRT(1./LX)
      DO 5 I=1,LX
      IF(I.GT.J) GO TO 2
      CTEMP=CX(J)*SC
      CX(J)=CX(I)*SC
      CX(I)=CTEMP
 2    M=LX/2
 3    IF(J.LE.M) GO TO 5
      J=J-M
      M=M/2
      IF(M.GE.1) GO TO 3
  5   J=J+M
      L=1
  6   ISTEP=2*L
      TEMP=3.14159265*SIGNI/L
      DO 8 M=1,L
      CARG=(0.,1.)*TEMP*(M-1)
      CW=CEXP(CARG)
      DO 8 I=M,LX,ISTEP
      CTEMP=CW*CX(I+L)
      CX(I+L)=CX(I)-CTEMP
  8   CX(I)=CX(I)+CTEMP
      L=ISTEP
      IF(L.LT.LX) GO TO 6
 9    RETURN
      END
* ------------------------ END FORK -------------------

*--------------------- START GET_DATE  ---------------------
      subroutine get_date(date_c)

      character date_c*(*)
      integer*2 iyr, imon, iday

* Microsoft compiler version:
*      call GETDAT (iyr, imon, iday)

*      date_c = ' '         ! character date_c*10

*      date_c(3:3) = '/'
*      date_c(6:6) = '/'

*      write(date_c(1:2),'(i2.2)') imon      
*      write(date_c(4:5),'(i2.2)') iday      
*      write(date_c(7:10),'(i4.4)') iyr      

* Lahey compiler version:
      call DATE(date_c)   ! character date_c*8, but *10 is OK

      return
      end
*--------------------- END GET_DATE  ---------------------

*--------------------- START GET_TIME  ---------------------
      subroutine get_time(time_c)

      character time_c*(*)
      integer*2 ihr, imin, isec, i100th

* Microsoft compiler version:
*      call GETTIM (ihr, imin, isec, i100th)

*      time_c = ' '             ! character time_c*11 for both compilers

*      time_c(3:3) = ':'
*      time_c(6:6) = ':'
*      time_c(9:9) = '.'

*      write(time_c(1:2),'(i2.2)') ihr      
*      write(time_c(4:5),'(i2.2)') imin      
*      write(time_c(7:8),'(i2.2)') isec      
*      write(time_c(10:11),'(i2.2)') i100th

* Lahey compiler version:
      call TIME(time_c)

      return
      end
*--------------------- END GET_TIME  ---------------------

* --------------------- BEGIN UPSTR ----------------------------------
      Subroutine UPSTR ( text )
* Converts character string in TEXT to uppercase
* Dates: 03/12/96 - Written by Larry Baker

C
      Implicit   None
C
      Character  text*(*)
C
      Integer    j
      Character  ch
C
      Do 1000 j = 1,LEN(text)
         ch = text(j:j)
         If ( LGE(ch,'a') .and. LLE(ch,'z') ) Then
            text(j:j) = CHAR ( ICHAR(ch) - ICHAR('a') + ICHAR('A') )
         End If
 1000    Continue
C
      Return
      End
* --------------------- END UPSTR ----------------------------------

*------------------   Begin ReadSMC   ----------------------------------
      
      subroutine ReadSMC(unit, file, tskip, tlength,
     :                   y, npts, sps, char_head, int_head, real_head)

* The output arrays char_head, int_head, real_head should be dimensioned
* as follows in the calling program:

*      character char_head(11)*80
*      integer int_head(48)
*      real real_head(50)

*  Dates: 07/17/97 - Modified order of input parameters, to put input
*                    first (unit, file, tskip), followed by output.
*         12/18/97 - Read tlength

      real real_head(*)
      integer int_head(*)
      character*80 char_head(*)
      character file*(*)
      real y(*)
      integer unit

      open(unit= unit, file=file,
     :   status='old')

      do i = 1, 11
        read(unit, '(a)') char_head(i)
      end do

      read(unit, '(8I10)') (int_head(i), i=1, 48)

      read(unit, '(5e15.7)') (real_head(i), i = 1, 50)

      sps = real_head(2)

      do i = 1, int_head(16)
        read(unit,*)
      end do

      read(unit, '(8(e10.4e1))') (y(i), i = 1, int_head(17))

* Skip into the trace:

      nskip = tskip * sps
      npts = tlength * sps
      if (nskip + npts .gt. int_head(17)) then
        npts = int_head(17) - nskip
      end if
      int_head(17) = npts
            
      do i = 1, npts
        y(i) = y( i + nskip)
      end do

      close(unit=unit)
      return
      end

*------------------   End ReadSMC   ----------------------------------

* --------------------- BEGIN LOCATE --------------------------------
      SUBROUTINE locate(xx,n,x,j)

* finds j such that x is between xx(j) & xx(j+1)
* j = 0 if x <= xx(1) (note problem if x = xx(1)!)
* j = n if x > xx(n)
* xx can be increasing or decreasing

      INTEGER j,n
      REAL x,xx(n)
      INTEGER jl,jm,ju
      jl=0
      ju=n+1
10    if(ju-jl.gt.1)then
        jm=(ju+jl)/2
        if((xx(n).gt.xx(1)).eqv.(x.gt.xx(jm)))then
          jl=jm
        else
          ju=jm
        endif
      goto 10
      endif
      j=jl
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software $!6)$-"11j.
* --------------------- END LOCATE ----------------------------------

* --------------------- BEGIN YINTRF ------------------------------------
      function yintrf( x, xin, yin, n)
c
c returns an interpolated value (yintrf) based on straight line
c interpolation of the data in xin and yin.
c
c dates:  3/14/85 - written
*        11/30/95 - substituted LOCATE instead of starting from beginning
*                   each time
*        03/13/96 - added code to deal with xin increasing or decreasing

      dimension xin(1), yin(1)
      logical incrs

* Is xin increasing or decreasing?
      incrs = .true.
      if (xin(n) .lt. xin(1)) incrs = .false.

* Set value if x is outside the range of xin:
      if (incrs) then
        if ( x .le. xin(1) ) then
            yintrf = yin(1)
            return
        end if
        if ( x .ge. xin(n) ) then
            yintrf = yin(n)
            return
        end if  
      else
        if ( x .ge. xin(1) ) then
            yintrf = yin(1)
            return
        end if
        if ( x .le. xin(n) ) then
            yintrf = yin(n)
            return
        end if  
      end if

* Locate the proper cell and interpolate:
      call locate(xin, n, x, j)
      yintrf = yin(j) + (x-xin(j))*(yin(j+1) - yin(j))/
     * (xin(j+1)-xin(j))

      return
      end
* --------------------- END YINTRF ------------------------------------

      include '\forprogs\rc_subs'
