		      Program Filt_Plot_Gen

* Filter a time series with a series of filter corners and
* plot results on a single page.  This program is intended to
* help in deciding what high-pass filter corner to choose.

*!Control file for program Filt_Plot_Gen
*!
*! Note on use of "!" to denote a comment line.  This is only used
*! for lines in the group of input lines containing processing
*! parameters.  Once "pp:" is encountered, the program assumes that
*! all following lines until the next "pp:" or "stop" contain
*! the names of files to be processed (note that this is consistent
*! with file name restrictions:  starting a file name with "!" is legal).
*!
*! Revision of program involving a change in the control file on this date:
*   02/14/18
*!Summary file name:
* filt_plot_gen.sum
*!
*! NOTE: Those parameters most likely to stay fixed are immediately below,
*! before the list of files.  Those likely to change for various groups
*! of files are set below the "pp:" line.
*!
*!PARAMETERS FOR OBTAINING THE MEAN:
*!
*! mean info (contained after "pp:" line below):
*! type of mean (0=no correction, 1=standard average; -1=specify mean)
*! t4mean_strt, t4mean_stop, mean (use if type = -1)
*! NOTE: need entries even if the parameter is not used (because read
*! info use list-directed [unformatted] input) 
*!
*!PARAMETER FOR DETRENDING AFTER MEAN REMOVAL STEP:
*!
*! 0 = NO; 1 = YES  (the detrending removes a straight line fit to the first and
*!                   last points)
*!
*!PARAMETERS FOR ZERO PADDING:
*!
*! Add pads if apply acausal filter (Y/N)?  This should almost always be
*! set to "Y".   Even if it is set to "N", note that the subroutine that
*! does the filtering adds pads (but it does not go to the first zero crossing 
*! and it does not save the padded time series)
* Y
*! Factor by which the number of zeros is increased beyond that determined by
*! the standard (BAP) equation.
* 1.0
*! Replace discarded portions of original data with zeros (Y/N)?  By doing this,
*! two horizontal components with the same number of time points before
*! padding (the usual case) will have the same number after padding and can
*! be rotated.
* Y
*!
*! Go to first zero crossing?
* Y
*! Use first point if smaller than first zero crossing?
* Y
*! Go to last zero crossing?
* Y
*! Use last point if smaller than last zero crossing?
* Y
*!
*! IMPORTANT NOTE: Care must be taken if the zero pads start at a point 
*! determined by going inward to the first or last zero crossings
*! ("Go to first/last zero crossing = T").    
*! Look at the headers of the smc files to see how many data points were 
*! skipped. Sometimes the beginning or end of the unfiltered time series 
*! has a drift, in which case too many points will be skipped before the 
*! first zero crossing is reached.
*!
*!   11/03/06 - If zcross_l or zcross_t = F and use_first or use_last = T, 
*!              pad with values equal to the last points.  Allow this option
*!              to deal with cases such as filtering a displacement
*!              time series with a low-frequency drift, which will have
*!              a step offset if pad with zeros at the end.  Be careful in
*!              in using this option for the first points, however (in
*!              a future version I may eliminate the option for the beginning,
*!              as generally the leading pad should be zero).  The need for
*!              this option arose when I tried to filter a displacement
*!              obtained by integrating a zeroth-order corrected acceleration.
*!              The results were different than filtering the acceleration
*!              before integration.
*!              IF WANT TO PAD WITH ZEROS, STARTING AT 
*!              BEGINNING AND END OF DATA, SET ZCROSS_L, ZCROSS_T = F AND
*!              USE_FIRST, USE_LAST = F (This is potentially confusing, and
*!              I may remove the option of padding with the first and last
*!              values in a later version of the program)
*!
*! Remove mean before determining segment to be padded?
*! If Yes, the mean will be determined from the whole record.  If this
*! is not desireable, remove the mean outside of this program
* N
*! Remove mean after determining segment to be padded (the segment
*! will probably be different than the segment originally used to determine the
*! mean, and therefore the mean will no longer be 0.0. But then after removing
*! this mean, the segment to be padded will not be at 0.0 values, but 
*! I hope this is a small problem.  I probably should iterate):
* N
*! Save padded time series?  (".pad" will be appended to file name)
* Y
*!
*!PARAMETERS FOR FILTERING:
*!
*! flc, nslope_lc, fhc (0 = no filter), nslope_hc, icaus(1=causal)
*!  (nslope >= 2,4,6, etc for causal and 4,8,12, etc for acausal; this is
*!   enforced in the program)
*! note that nslope is the asymptotic power of frequency, so that the filter
*! goes as f^{nslope} for a low-cut filter and f^{-nslope} for a
*! high-cut filter. In previous versions I used "nroll" as the parameter 
*! controlling the asymptotic behavior of the filter, the reason being that 
*! it is the parameter used by the filter routines in BAP.   This was always
*! confusing, so I switched to nslope on 27 Oct. 2005, with a translation to
*! nroll in blpadflt (nroll = 0.5*nslope for causal and 
*! nroll = 0.25*nslope for acausal). (Note more confusing terminology:
*! nroll = 0.5*npoles for a causal Butterworth filter.) 
*! The response of an acausal filter
*! is not that of a Butterworth, because time domain filters are used.
*! Bazzuro et al. (2004 COSMOS paper) use the terminology npnp, where
*! "n" is the number of poles of a standard Butterworth filter.  The "npnp"
*! notation is used to indicate that two passes of a Butterworth filter are
*! used.   So, for example, "2p2p" means that two passes of a two-pole
*! Butterworth filter are used, with an asymptotic dependence as f^4 (for a
*! low-cut filter). This notation is used to distinguish this case
*! from that in which an acausal filter is simulated in the frequency domain, in
*! which case the filter can have a true Butterworth response function. 
*! For that case, a "4p" acausal filter constructed in the frequency 
*! domain and using the standard Butterworth response would have the same
*! asymptotic dependence (f^4).
*!
*! I added a notch filter on 03/20/10.  The notch filter is invoked if
*! (flc = fhc) and flc /= 0.0, and the width parameter "d" of the filter is given by 
*! d = real(nslope_lc)*10.0**real(-abs(nslope_hc)).  The filter will be causal if 
*! both icaus_lc = 1 and icaus_hc = 1. Adding this new filter did
*! not require a change in the input parameters, but I changed the date of the
*! control file to make the user aware of a change in the capability of the
*! program. Here is a sample specification:
*!   25.0 1 1 25.0 2  0
*! which corresponds to a 25 Hz causal notch filter with filter width of 
*! 1*10**(-2)= 0.01, which is the starting trial value recommended by 
*! Bill Joyner, who wrote the notch filter subroutine.
*!
*! On 07/11/17 I allowed for the low-cut and high-cut to be causal or acausal
*! separately; previously, both were either causal or acausal.  I made the change
*! because causal low-cut filters can affect high frequency response spectra, whereas
*! there might be some advantages to using a causal high-cut filter because the filter
*! is down by 1/sqrt 2 at the corner frequency rather than 1/2 (thus giving a sharper transition
*! between the unfiltered and filtered portions of the spectra).  Previously I need to run blpadflt
*! twice to achieve this, the first time with an acausal low-cut filter, and then using
*! the resulting padded time series in a second run of blpadflt for a causal high-cut
*! filter.  
*!
*! In the previous version, the filter routine "band" was called if both flc and fhc /= 0.0 and
*! flc /= fhc.  The possibility of different types of filters for flc and fhc means that I need to call
*! locut and hicut separately, but I want to retain the possibility of a band filter.  For that reason I
*! added a Y/N parameter to indicate if a band filter is desired. If Yes, then it will be
*! a causal filter only if both icaus_lc and icaus_hc = 1.
*!
*!PLOT PARAMETERS
*!Precede filtered traces with plots of unfiltered a(1), a&v(2), a,v,&d(3)?
* 1
*!Plot filtered a, v, or d?
*! A
* V
*! D
*!start time, duration to plot (<=0 to start at 1st pt; >>= 0 to plot all):
*0.0 900.0
*!peak-to-peak distance (cm) (ntraces*(p2p+0.5)<= 23.5, prog will reset if 
*! needed)
* 5.0
*!name to be added before the smc file name in creating the output files  (a final period is not needed).
*! Logically, this would be in the "pp: new set of parameters" group, but I often forget
*! to change the name if I've copied the group to create a new plot, in which case the
*! same file name is used for both groups.  I decided instead to add an index number after the
*! string below
* filt_plot_gen
*!
*!PROCESSING PARAMETERS AND FILES, IN GROUPS
*!
*pp: new set of parameters
*!mean: type (-1=specify,0=none,1=standard), t4mean_strt, t4mean_stop, 
*!mean (use if type = -1):
* 1  0.0 2000.0  -999.
*!detrend after mean removal (0=no, 1=yes, in which case a line fit to
*!the first and last points is removed; note that the mean need not be removed
*!before detrending if type = 0 above):
* 0
*!number of filters (<=20) 
*  9
*!Filter information (see earlier text for information):
*! flc, nslope_lc, icaus_lc, fhc, nslope_hc, icaus_hc
*! (flc, fhc = 0 for no filter; icaus = 0 for acausal, 1 for causal)
*! (nslope >= 2: nslope >= 2,4,6, etc for causal and 4,8,12, etc for acausal)
*! (The number of entries is the same as the number of filters)
* 0.02 8  0  0.04  8  0
* 0.04 8  0  0.08  8  0
* 0.08 8  0  0.16  8  0
* 0.16 8  0  0.32  8  0
* 0.32 8  0  0.64  8  0
* 0.64 8  0  1.28  8  0
* 1.28 8  0  2.56  8  0
* 2.56 8  0  5.12  8  0
* 5.12 8  0 10.24  8  0
*!Are these band filters (Y/N)? (the entry is not used if either flc or fhc = 0 or if
*! flc == fhc) (for most generality, the answer should be "N"; also, because a band
*! filter will be unusual, I assume that all filters will be band filters, rather 
*! than allowing each of nfilt filters to be or not to be band filters.  But note that
*! even if band = Y, a band filter is used only if flc and fhc /= 0.0 and
*! flc /= fhc
* N
*!
*!Length of taper at beginning and end of time series, before adding zeros
*! (Not used if no filter or a causal filter, but still need this parameter
*! (in the input file):
* 2.0 10.0
*!Names of files to be processed with the parameters above:
*HER19401.L.dat.smc
*stop
  
  
* Dates: 06/04/09 - Written by D. Boore, based on blpadflt and smctsplt
*        06/25/09 - Minor change to names of output files.
!        03/17/10 - Updated call to construct_filename_extension
!        06/03/10 - Calculate mean square differences of consecutive
!                   filtered displacements ("eps_disp", Aspa Zerva)
!                 - Increased max number of filters to 20
!        07/13/11 - Add tskip and tlength to call to construct_filename_extension
!                   (in two places)
!                 - Increased length of string_to_append from 40 to 100
!        02/02/12 - Renamed from filt_plot to filt_plot_lc, because I will also
!                   write a version that uses a list of flc, fhc (filt_plot_gen)
!        02/03/12 - Remove option to compute and save response spectra, to
!                   write plots of displacement and velocity, and 
!                   to compute eps_disp and eps_vel
!        02/14/18 - Revise because the inputs to construct_filename_extension and filter
!                   have changed.
!        02/24/18 - Move setting of nslope_bp, nroll_bp before checks of acausal, causal for a band filter

!      implicit none
      
      parameter (small = 0.01) 

      character char_head(11)*80, comments(500)*80, 
     :                            ctl_cmmnts(100)*200
     
      character date_ctl_correct*8, date_ctl_in*30
 
      integer int_head(48), option
      integer iplot_unfiltered_a_av_avd
      
      real real_head(50)

      real a_in(:), a_out(:), v_out(:), d_out(:), v_dum(:), d_dum(:),
     :     ts2plot(:,:) 
      allocatable :: a_in, a_out, v_out, d_out, v_dum, d_dum,
     :               ts2plot
     
      double precision avg
      character f_ctl*60, f_sum*60, f_in*120, buf*80, buf_c*79
      character f_stem*100, f_a*120, f_v*120, f_d*120, f_rs*120
      character string_to_append*100, buf3*3, buf4*4, buf6*6, 
     :          f_pad*120, filename_string*100
      character compute_v_d*10, precision*8
      
      character filter_type*10, plotnumber*12, what2plot*10

      real sps_ts(23)
      integer npts_ts(23)
      
      character plot_name*400, ts_name(23)*120, 
     :   type_c(23)*3

      logical f_ctl_exist, f_in_exist, rmv_trnd, add_impulse
      logical zcross_l, zcross_t, use_first, use_last
      logical remove_mean_before, remove_mean_after, detrend
      logical apply_pads, save_padded_time_series, 
     :        replace_discarded_data_with_zeros
      
      character date_c*8, time_c*11
      
      real flc_in(20), fhc_in(20) 
      integer  nslope_lc_in(20), nslope_hc_in(20), 
     :         icaus_lc_in(20), icaus_hc_in(20)
     
      logical band_filter

      pi =  4.0*atan(1.)
      twopi = 2.0*pi

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file ("Enter" = filt_plot_gen.ctl): '
        read(*, '(a)') f_ctl
        if (f_ctl == ' ') f_ctl = 'filt_plot_gen.ctl'
        call trim_c(f_ctl,nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

! Check date of control file:
      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      date_ctl_in = ' '
      read(nu_ctl,'(a)') date_ctl_in
      call trim_c(date_ctl_in,nc_date_ctl_in)
      
      date_ctl_correct = ' '
      date_ctl_correct = '02/19/18'
      call trim_c(date_ctl_correct,nc_date_ctl_correct)


      if (date_ctl_correct(1:nc_date_ctl_correct) /= 
     :    date_ctl_in(1:nc_date_ctl_in)) then
        write(*,'(a)') 
     :     ' The control file has the wrong date; update your '//
     :       'control file and rerun the program!'
        write(*,'(a)') 
     :     ' The date for the control file you used is '
     :       //date_ctl_in(1:nc_date_ctl_in)
        write(*,'(a)') 
     :     ' The correct date is '
     :       //date_ctl_correct(1:nc_date_ctl_correct)
        close(nu_ctl)
        stop
      end if
! Check date of control file:

      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call trim_c(f_sum,nc_f_sum)
      call get_lun(nu_sum)
      open(unit=nu_sum,file=f_sum(1:nc_f_sum),status='unknown')
      write(nu_sum,'(a)') ' Summary file for program filt_plot_gen'
      write(nu_sum,'(a)') 
     :     '   Using control file: '//f_ctl(1:nc_f_ctl)


* PARAMETERS FOR ZERO PADDING:

      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) == 'Y') then
        apply_pads = .true.
      else
        apply_pads = .false.
      end if

      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      read(nu_ctl, *) xfactor_zpads

      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) == 'Y') then
        replace_discarded_data_with_zeros = .true.
      else
        replace_discarded_data_with_zeros = .false.
      end if

      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) == 'Y') then
        zcross_l = .true.
      else
        zcross_l = .false.
      end if

      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) == 'Y') then
        use_first = .true.
      else
        use_first = .false.
      end if

      call skipcmnt(nu_ctl, ctl_cmmnts, nctl_cmmnts)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) == 'Y') then
        zcross_t = .true.
      else
        zcross_t = .false.
      end if

      call skipcmnt(nu_ctl, ctl_cmmnts, nctl_cmmnts)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) == 'Y') then
        use_last = .true.
      else
        use_last = .false.
      end if

      write(*,'(a)') ' zcross_l, zcross_t = '
      write(*, *)      zcross_l, zcross_t

      write(*,'(a)') ' use_first, use_last = '
      write(*, *)      use_first, use_last

      call skipcmnt(nu_ctl, ctl_cmmnts, nctl_cmmnts)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) == 'Y') then
        remove_mean_before = .true.
      else
        remove_mean_before = .false.
      end if
      write(*,'(a,1x,l1)') 
     :      ' remove_mean_before = ', remove_mean_before

      call skipcmnt(nu_ctl, ctl_cmmnts, nctl_cmmnts)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) == 'Y') then
        remove_mean_after = .true.
      else
        remove_mean_after = .false.
      end if
      write(*,'(a,1x,l1)') 
     :      ' remove_mean_after = ', remove_mean_after

      call skipcmnt(nu_ctl, ctl_cmmnts, nctl_cmmnts)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) == 'Y') then
        save_padded_time_series = .true.
      else
        save_padded_time_series = .false.
      end if

* PARAMETERS FOR PLOT
      
      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      do ihc = 1, nc_ctl_cmmnts
        write(nu_sum,'(1x,a)') ctl_cmmnts(ihc)
      end do
!Precede filtered traces with plots of unfiltered a(1), a&v(2), a,v,&d(3)?
      read(nu_ctl,*) iplot_unfiltered_a_av_avd
      write(nu_sum,*) iplot_unfiltered_a_av_avd 
      if (iplot_unfiltered_a_av_avd /= 1 .and. 
     :    iplot_unfiltered_a_av_avd /= 2 .and. 
     :    iplot_unfiltered_a_av_avd /= 3) then
       print *,' ERROR: iplot_unfiltered_a_av_avd = ', 
     :    iplot_unfiltered_a_av_avd, 
     :         ' QUITTING'
       close(nu_ctl)
       stop
      end if

      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      do ihc = 1, nc_ctl_cmmnts
        write(nu_sum,'(1x,a)') ctl_cmmnts(ihc)
      end do
!Plot filtered a, v, or d?
      what2plot = ' '
      read(nu_ctl,'(a)') what2plot
      call trim_c(what2plot, nc_what2plot)
      call upstr(what2plot)
      write(nu_sum,'(a)') what2plot(1:nc_what2plot)

      if (nc_what2plot /= 1 ) then
        print *,' ERROR: nc_what2plot = ', 
     :                   nc_what2plot, 
     :         ' which is illegal; QUITTING'
        close(nu_ctl)
        stop
      end if
      if (nc_what2plot == 1) then
        if (what2plot(1:1) /= 'A' .and.
     :      what2plot(1:1) /= 'V' .and.   
     :      what2plot(1:1) /= 'D') then   
          print *,' ERROR: what2plot = '// 
     :      what2plot(1:nc_what2plot)// 
     :         ' which is illegal; QUITTING'
          close(nu_ctl)
          stop
        end if
      end if

      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      do ihc = 1, nc_ctl_cmmnts
        write(nu_sum,'(1x,a)') ctl_cmmnts(ihc)
      end do
      read(nu_ctl,*) t_strt_in, t_plot_in
      write(nu_sum,*) t_strt_in, t_plot_in 
      
      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      do ihc = 1, nc_ctl_cmmnts
        write(nu_sum,'(1x,a)') ctl_cmmnts(ihc)
      end do
      read(nu_ctl,*) pp_cm
      write(nu_sum,*) pp_cm
      
      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      do ihc = 1, nc_ctl_cmmnts
        write(nu_sum,'(1x,a)') ctl_cmmnts(ihc)
      end do
      filename_string = ' '
      read(nu_ctl,'(a)') filename_string
      call trim_c(filename_string, nc_filename_string)
      write(nu_sum,'(1x,a)') filename_string(1:nc_filename_string)        
          
      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)

* CHECK FOR ANOTHER FILE TO PROCESS

      iplot = 0

1000  continue

        write(nu_sum,*)  
        
        buf = ' '
        read(nu_ctl,'(a)') buf
        write(nu_sum,'(1x,a)') buf
        call trim_c(buf,nc_buf)
        buf3 = ' '
        buf3 = buf(1:3)
        call upstr(buf3)
        buf4 = ' '
        buf4 = buf(1:4)
        call upstr(buf4)
        if (buf4 == 'STOP') go to 9999
        if (buf3 == 'PP:') then
* New processing parameters

          iplot = iplot + 1

* PARAMETERS FOR OBTAINING THE MEAN:

          call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
          do ihc = 1, nc_ctl_cmmnts
            write(nu_sum,'(1x,a)') ctl_cmmnts(ihc)
          end do
          read(nu_ctl,*) itype_mean, 
     :                 t4mean_strt_in, t4mean_stop_in, spec_mean
          write(nu_sum,*)  itype_mean, 
     :                   t4mean_strt_in, t4mean_stop_in, spec_mean

* PARAMETER FOR DETRENDING:

          call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
          do ihc = 1, nc_ctl_cmmnts
            write(nu_sum,'(1x,a)') ctl_cmmnts(ihc)
          end do
          read(nu_ctl,*) i_detrend
          write(nu_sum,*) i_detrend
          if (i_detrend == 0) then
            detrend = .false.
          else
            detrend = .true.
          end if
 
* PARAMETERS FOR FILTERING:

!number of filters (<=20) 
!  9
!Filter information (see earlier text for information):
! flc, nslope_lc, icaus_lc, fhc, nslope_hc, icaus_hc
! (flc, fhc = 0 for no filter; icaus = 0 for acausal, 1 for causal)
! (nslope >= 2: nslope >= 2,4,6, etc for causal and 4,8,12, etc for acausal)
! (The number of entries is the same as the number of filters)
! 0.02 8  0  0.04  8  0
! 0.04 8  0  0.08  8  0
! 0.08 8  0  0.16  8  0
! 0.16 8  0  0.32  8  0
! 0.32 8  0  0.64  8  0
! 0.64 8  0  1.28  8  0
! 1.28 8  0  2.56  8  0
! 2.56 8  0  5.12  8  0
! 5.12 8  0 10.24  8  0

          call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
          do ihc = 1, nc_ctl_cmmnts
            write(nu_sum,'(1x,a)') ctl_cmmnts(ihc)
          end do
          read(nu_ctl,*) nfilt
          write(nu_sum,*) nfilt

          call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
          do ihc = 1, nc_ctl_cmmnts
            write(nu_sum,'(1x,a)') ctl_cmmnts(ihc)
          end do
   
          do i = 1, nfilt
            read(nu_ctl,*) 
     :          flc_in(i), nslope_lc_in(i), icaus_lc_in(i),
     :          fhc_in(i), nslope_hc_in(i), icaus_hc_in(i)

            write(nu_sum,*) flc_in(i), nslope_lc_in(i), icaus_lc_in(i),
     :                      fhc_in(i), nslope_hc_in(i), icaus_hc_in(i)
          end do

          call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
          buf = ' '
          read(nu_ctl,'(a)') buf
          call upstr(buf)
          call trim_c(buf, nc_buf)
          do ihc = 1, nc_ctl_cmmnts
            write(nu_sum,'(1x,a)') ctl_cmmnts(ihc)
          end do
          write(nu_sum,'(a)') buf(1:nc_buf)
          if (buf(1:1) == 'Y') then
            band_filter = .true.
          else
            band_filter = .false.
          end if
          

* OPTION TO TAPER END OF TIME SERIES BEFORE ADDING LEADING AND
* TRAILING ZERO PADS

          call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
          do ihc = 1, nc_ctl_cmmnts
            write(nu_sum,'(1x,a)') ctl_cmmnts(ihc)
          end do
          read(nu_ctl,*) taper_beginning, taper_end
          write(nu_sum,'(a)') 
     :                 ' taper_beginning, taper_end upon input:'
          write(nu_sum,*) taper_beginning, taper_end
 

          go to 1000

        end if ! finished reading a set of processing parameters

* Read the input file:

        f_in = ' '
        f_in = buf(1:nc_buf)
        call trim_c(f_in, nc_f_in)

        inquire(file=f_in(1:nc_f_in), exist=f_in_exist)
        if (.not. f_in_exist) then
          write(*,'(a)') ' ** Input file '//f_in(1:nc_f_in)//
     :    ' does not exist, moving to next file **'
          go to 1000
        end if

        write(*,'(a)') ' Processing file: '//f_in(1:nc_f_in)
        write(nu_sum,*)  
        write(nu_sum,'(a)') ' Processing file: '//f_in(1:nc_f_in)

        call smc_npts(f_in(1:nc_f_in), npts_max, sps)
        
        npts_4alloc = npts_max+10
        allocate (a_in(npts_max), 
     :     ts2plot(npts_4alloc,nfilt+iplot_unfiltered_a_av_avd), 
     :            v_dum(npts_max), d_dum(npts_max))
     
        ts2plot = 0.0
        
        tskip = 0.0
        tlength = -1.0

        call SMCRead(f_in(1:nc_f_in), tskip, tlength,
     :               a_in, char_head, int_head, real_head, comments)
        sps = real_head(2)
        dt = 1.0/sps
        ncomments = int_head(16)
        npts = int_head(17)
        tend = float(npts-1)/sps
        
        write(nu_sum,'(a)') ' sps, ncomments, npts = '
        write(nu_sum, *) sps, ncomments, npts

* Write comment headers (NOTE: I am distributing these throughout
* rather than in one place as in Bl_Fltr because smcpadf writes
* comment within the subroutine, and I wanted the comments in proper order.)

        call date(date_c)
        call time(time_c)


* Step 1: REMOVE MEAN FROM WHOLE RECORD

*  Determine the appropriate mean:

        t4mean_strt = t4mean_strt_in
        t4mean_stop = t4mean_stop_in
        
        if (itype_mean == 0) then
          n_t4mean = 0
          avg_all = 0.0
          ncomments = ncomments + 1
          comments(ncomments) = 
     :    '|   no mean removed.'
        else if (itype_mean == -1) then   ! use specified mean
          avg_all = spec_mean
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,1x,1pe12.5,a)') 
     :    '|   remove the specified mean ', avg_all,
     :      ' from whole record.'
        else if (itype_mean == 1) then
          if (t4mean_strt < 0.0) t4mean_strt = 0.0
          if (t4mean_stop > tend) t4mean_stop = tend
          n_t4mean_strt = nint(sps * t4mean_strt + 1.1)
          n_t4mean_stop = nint(sps * t4mean_stop + 1.1)
          n_t4mean = n_t4mean_stop - n_t4mean_strt + 1
          t4mean = float(n_t4mean -1)/sps
          avg = 0.0d0
          do i = n_t4mean_strt, n_t4mean_stop
            avg = avg + dble(a_in(i))
          end do
          avg_all = sngl(avg/dble(n_t4mean))
        else            ! shouldn't be here!
          write(nu_sum, '(a,1x,i3,1x,a)') '  Should not be here, '//'
     :         improper value of itype_mean.  itype_mean = ', 
     :         itype_mean, ' QUITTING!!'
          write(*) '  Should not be here, '//'
     :         improper value of itype_mean.  itype_mean = ', 
     :         itype_mean, ' QUITTING!!'
          go to 9999
        end if

*  Remove the mean from the whole record:
        write(nu_sum,'(a,1x,i6)') ' n_t4mean = ', n_t4mean
        write(nu_sum,'(a,1x,1pe12.5,1x,a)') 
     :   ' Removing mean = ', avg_all, ' from the whole record'

        do i = 1, npts
          a_in(i) = a_in(i) - avg_all
        end do

        
* Step 2: DETREND

        if (detrend) then
          call rmvtrend(a_in, npts)
          ncomments = ncomments + 1
          comments(ncomments) = 
     :    '|   detrend the time series by subtracting '//
     :    'a line fit to the first & last points' 
        end if	 

* Set initial velocity, displacement

        d0bl = 0.0d0
        v0bl = 0.0d0

        d0 = - d0bl
        v0 = - v0bl ! minus to account for subtraction of baseline

* Step 3: Compute zeroth-order velocity (for plotting)

        v04vd = 0.0
        d04vd = 0.0

        rmv_trnd = .false.
        call acc2vd(a_in, npts, dt, rmv_trnd, v04vd, d04vd, 
     :              v_dum, d_dum)

!Store unprocessed a, av, or avd in ts2plot:        
        do i = 1, npts
          ts2plot(i,1) = a_in(i)
        end do
        ts_name(1) = ' '
        ts_name(1) = f_in(1:nc_f_in)
        type_c(1) = 'ACC'
        sps_ts(1) = sps
        npts_ts(1) = npts

        option = 10
        flc = 0.0
        nslope_lc = 8
        fhc = 0.0
        nslope_hc = 8
        icaus_lc = 0
        filter_type = 'NF'
        nc_filter_type = 2
        icaus_hc = 0
        icaus_bn = 0
        call construct_filename_extension
     :      (tskip, tlength, option, 
     :       flc, nslope_lc, icaus_lc,
     :       fhc, nslope_hc, icaus_hc, icaus_bn,
     :       filter_type, nc_filter_type,
     :       taper_beginning, taper_end,     
     :       string_to_append, nc_s2appnd)

        if (iplot_unfiltered_a_av_avd == 2 
     :     .or. iplot_unfiltered_a_av_avd == 3) then  ! second time series is velocity

          do i = 1, npts
            ts2plot(i,2) = v_dum(i)
          end do
          ts_name(2) = ' '
          ts_name(2) = f_in(1:nc_f_in)//
     :          string_to_append(1:nc_s2appnd)//
     :                 '.v'
          type_c(2) = 'VEL'
          sps_ts(2) = sps
          npts_ts(2) = npts
        end if

        if (iplot_unfiltered_a_av_avd == 3) then  ! third time series is displacement

          do i = 1, npts
            ts2plot(i,3) = d_dum(i)
          end do
          ts_name(3) = ' '
          ts_name(3) = f_in(1:nc_f_in)//
     :          string_to_append(1:nc_s2appnd)//
     :                 '.d'
          type_c(3) = 'DIS'
          sps_ts(3) = sps
          npts_ts(3) = npts
        end if
 
        deallocate(v_dum, d_dum)
!Store unprocessed a, av, or avd in ts2plot:        
        
* Step 4: Loop over nfilt

        loop over filters: DO ifilt = 1, nfilt

          flc = flc_in(ifilt)
          fhc = fhc_in(ifilt)
          nslope_lc = nslope_lc_in(ifilt)
          nslope_hc = nslope_hc_in(ifilt)
          icaus_lc = icaus_lc_in(ifilt)
          icaus_hc = icaus_hc_in(ifilt)

! Determine filter type and rest parameters if needed, including
! enforcing a factor of two and four for causal and acausal filters, respectively

! First assign filter type:
          filter_type = ' '
          if (flc == 0.0 .and. fhc == 0.0) then
            filter_type = 'NF'
          else if (fhc == 0.0) then
            filter_type = 'LC'
          else if (flc == 0.0) then
            filter_type = 'HC'
          else if (flc == fhc ) then  ! notch
            filter_type = 'NOTCH'
          else if (band_filter) then
            filter_type = 'BAND'
          else
            filter_type = 'LCHC'
          end if
            
          call trim_c(filter_type,nc_filter_type)
          
          
! Set icaus_bn and reset icaus_lc, icaus_hc for icaus_bn = 0:

          if (filter_type(1:nc_filter_type) == 'NOTCH' .or.
     :        filter_type(1:nc_filter_type) == 'BAND') then 
            if(icaus_lc == 1 .and. icaus_hc == 1) then
              icaus_bn = 1
            else
              icaus_bn = 0
              icaus_lc = 0  
              icaus_hc = 0
! for a notch or bandpass filter, it needs to be either causal or acausal; 
! the proper way to specify this for an acausal filter is to set icaus_lc = icaus_hc = 0
! in the control file, but sometimes one is set to 0 and the other to 1.  For that
! reason I have reset them to 0 for an acausal filter
            end if
          else
            icaus_bn = 0
          end if

! Reset nslope if needed:
          if (icaus_lc == 0) then   ! acausal lc  
            if (nslope_lc < 4) nslope_lc = 4
            nslope_lc = 4 * int(float(nslope_lc)/4.0)
            nroll_lc = nslope_lc/4
          else                      ! causal lc
            if (nslope_lc < 2) nslope_lc = 2
            nslope_lc = 2 * int(float(nslope_lc)/2.0)
            nroll_lc = nslope_lc/2
          end if
          if (icaus_hc == 0) then   ! acausal hc  
            if (nslope_hc < 4) nslope_hc = 4
            nslope_hc = 4 * int(float(nslope_hc)/4.0)
            nroll_hc = nslope_hc/4
          else                      ! causal hc
            if (nslope_hc < 2) nslope_hc = 2
            nslope_hc = 2 * int(float(nslope_hc)/2.0)
            nroll_hc = nslope_hc/2
          end if

! Set notch parameter:
          if (filter_type(1:nc_filter_type) == 'NOTCH') then
            d4notch = real(nslope_lc)*10.0**(-abs(real(nslope_hc)))
          else
            d4notch = 0.0
          end if
          
        option = 10
        call construct_filename_extension
     :      (tskip, tlength, option, 
     :       flc, nslope_lc, icaus_lc,
     :       fhc, nslope_hc, icaus_hc, icaus_bn,
     :       filter_type, nc_filter_type,
     :       taper_beginning, taper_end,     
     :       string_to_append, nc_s2appnd)
 
          ts_name(ifilt+iplot_unfiltered_a_av_avd) = ' '
          ts_name(ifilt+iplot_unfiltered_a_av_avd) = f_in(1:nc_f_in)//
     :            string_to_append(1:nc_s2appnd)//
     :            '.taper'
          call trim_c(ts_name(ifilt+iplot_unfiltered_a_av_avd), 
     :                nc_ts_name)
          if (taper_beginning >= 10.0) then
            write(ts_name(ifilt+iplot_unfiltered_a_av_avd)
     :             (nc_ts_name+1:nc_ts_name+4),
     :             '(f4.1)') taper_beginning
          else
            ts_name(ifilt+iplot_unfiltered_a_av_avd)
     :              (nc_ts_name+1:nc_ts_name+1) = '_'
            write(ts_name(ifilt+iplot_unfiltered_a_av_avd)
     :             (nc_ts_name+2:nc_ts_name+4),
     :             '(f3.1)') taper_beginning
          end if
        
          ts_name(ifilt+iplot_unfiltered_a_av_avd)
     :             (nc_ts_name+5:nc_ts_name+5) = '_'
          call trim_c(ts_name(ifilt+iplot_unfiltered_a_av_avd), 
     :                nc_ts_name)
          if (taper_end >= 10.0) then
            write(ts_name(ifilt+iplot_unfiltered_a_av_avd)
     :             (nc_ts_name+1:nc_ts_name+4),
     :             '(f4.1)') taper_end
          else
            ts_name(ifilt+iplot_unfiltered_a_av_avd)
     :             (nc_ts_name+1:nc_ts_name+1) = '_'
            write(ts_name(ifilt+iplot_unfiltered_a_av_avd)
     :             (nc_ts_name+2:nc_ts_name+4),
     :             '(f3.1)') taper_end
          end if

        if (what2plot(1:1) == 'A') then ! plot filtered acc        
          ts_name(ifilt+iplot_unfiltered_a_av_avd)
     :       (nc_ts_name+5:nc_ts_name+6) = '.a'
          type_c(ifilt+iplot_unfiltered_a_av_avd) = 'ACC'
        else if (what2plot(1:1) == 'V') then ! plot filtered vel        
          ts_name(ifilt+iplot_unfiltered_a_av_avd)
     :       (nc_ts_name+5:nc_ts_name+6) = '.v'
          type_c(ifilt+iplot_unfiltered_a_av_avd) = 'VEL'
        else if (what2plot(1:1) == 'D') then ! plot filtered dis        
          ts_name(ifilt+iplot_unfiltered_a_av_avd)
     :       (nc_ts_name+5:nc_ts_name+6) = '.d'
          type_c(ifilt+iplot_unfiltered_a_av_avd) = 'DIS'
        else
          print *, what2plot(1:1)//' /= A, V, or D; QUIT!'
          stop
        end if
        
        call trim_c(ts_name(ifilt+iplot_unfiltered_a_av_avd),
     :        nc_ts_name)
         
 
 
* Need to allocate a_out, v_out, d_out differently because
* more than npts_max might be used in acausal filter routines (those
* routines pad the array beyond npts).  Must do this even if choose not to
* add pads when do an acausal filter.

* NOTE: the pad computation here is done because of the need to allocate the
* time series; nlz and ntz are passed into subroutine smcpadf.

! FROM BLPADFLT
!
! Need to allocate a_out, v_out, d_out differently because
! more than npts_max might be used in acausal filter routines (those
! routines pad the array beyond npts).  Must do this even if choose not to
! add pads when do an acausal filter.

! NOTE: the pad computation here is done because of the need to allocate the
! time series; nlz and ntz are passed into subroutine smcpadf.

! It is more efficient, but not by much, to use a series of if then else statements.  This is
! more readable, even though each statement must be evaluated.


!In a previous version there is a problem for bandpass filter
! with different npads for 
! acausal when icaus_lc or hc = 1; check to see if this is still true.

! NOTE: npad = 0 for all causal filters (careful for locut followed by hicut)

        npad = 0  ! only change if acausal
        npad_lc = 0  ! only change if acausal
        npad_hc = 0  ! only change if acausal
        npad4alloc = 0   ! only change if acausal! only change if acausal   
        if (filter_type(1:nc_filter_type) /= 'NF') then 
          if (filter_type(1:nc_filter_type) == 'LC') then
            if (icaus_lc == 0) then ! acausal locut only
              npad = 3.0*float(nroll_lc)/(flc*dt)
            else
              npad = 0
            end if
          else if (filter_type(1:nc_filter_type) == 'HC') then
            if (icaus_hc == 0) then ! acausal hicut only
              npad = 3.0*float(nroll_hc)/(fhc*dt)
            else
              npad = 0
            end if
          else if (filter_type(1:nc_filter_type) == 'NOTCH') then 
            if (icaus_bn == 0) then ! acausal notch
              npad = 3.0*pi/d4notch
            else
              npad = 0
            end if
          else if (filter_type(1:nc_filter_type) == 'BAND') then 
            nroll_bp = max0(nroll_lc, nroll_hc)                ! make sure that these were defined earlier as if they are acausal
            nslope_bp = max0(nslope_lc, nslope_hc)
            if (icaus_bn == 0) then ! acausal bandpass
              npad1=3.0*float(nroll_lc)/(flc*dt)
              npad2=6.0*float(nroll_bp)/((fhc-flc)*dt)
              npad = max0(npad1, npad2)
            else
              npad = 0
            end if
          else if (filter_type(1:nc_filter_type) == 'LCHC') then
              if (icaus_lc == 0) then
                npad_lc = 3.0*float(nroll_lc)/(flc*dt)
              else
                npad_lc = 0
              end if
              if (icaus_hc == 0) then
                npad_hc = 3.0*float(nroll_hc)/(fhc*dt)
              else
                npad_hc = 0
              end if
              npad = npad_lc + npad_hc
          else
             write(*,'(a)') 
     :           ' In npad part of blpadflt, unknown filter type ('//
     :             filter_type(1:nc_filter_type)//'); quitting!!! '
             stop
           end if
         end if

         write(nu_sum,'(a)') 
     :  ' For filter type = '//filter_type(1:nc_filter_type)//
     :  ', flc, icaus_lc, fhc, icaus_hc, icaus_bn, npad_lc,'//
     :  '  npad_hc, npad:' 
        write(nu_sum,*) 
     :    flc, icaus_lc, fhc, icaus_hc, icaus_bn, npad_lc, npad_hc, 
     :    npad
        write(nu_sum,*) 

! moved this out of IF loop
        if (xfactor_zpads < 1.0) then
          npad4alloc =  npad  ! need this because the allocations in
                              ! subroutine locut assume that npad is at
                              ! least as large as the standard value.
        else
          npad4alloc = int(xfactor_zpads * float(npad))
        end if
        npad = int(xfactor_zpads * float(npad))

        nlz = npad/2
        ntz = npad/2
        izcl = nlz
        izct = ntz
        npts4alloc = npts_max + 2*npad4alloc + 2
 
        write(nu_sum,'(a)') ' After determination of npad: '//
     :     'npts, npts4alloc, npts_max, nlz, ntz, npad, npad4alloc ='
        write(nu_sum,*) 
     :      npts, npts4alloc, npts_max, nlz, ntz, npad, npad4alloc

* extra trailing pad of npad because the filter subroutines add their own 
* zero pad equal to npad to the end of the time series.  If no integration was
* being done, then that would be OK, but I want to add npad/2 to the front so 
* that the integration accounts properly for the filter transient. 
* I first added only an extra npad/2, but this led to a hard-to-find run time
* error.

        allocate (a_out(npts4alloc))
        allocate (v_out(npts4alloc))
        allocate (d_out(npts4alloc))
 
        a_out = 0.0
        v_out = 0.0
        d_out = 0.0
  
        do i = 1, npts
          a_out(i) = a_in(i)
        end do
        
* Step 4a: PAD TIME SERIES IF USE ACAUSAL FILTER

        
        if (npad /= 0) then ! acausal filter, apply pads


          write(nu_sum,'(a)') 
     :          ' Inside pad time series if acausal filter:'
          write(nu_sum,'(a)') ' Working on file '//f_in(1:nc_f_in)
          write(nu_sum,'(a)') 
     :                 ' taper_beginning, taper_end upon input:'
          write(nu_sum,*) taper_beginning, taper_end
 
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,f6.2)') 
     :            '| Acausal filter, apply pads, '//
     :            'increasing the number of zeros by '
  
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,f6.2)') 
     :            '|   the multiplicative factor ', xfactor_zpads
 
          call smcpadf(a_out, npts, sps, nlz, ntz, 
     :      replace_discarded_data_with_zeros, 
     :      taper_beginning, taper_end, 
     :      npts_out, izcl, izct,  
     :      remove_mean_before, remove_mean_after, 
     :      zcross_l, use_first, zcross_t, use_last,
     :      comments, ncomments)    

* Store new ncomments, npts:

          int_head(16) = ncomments            

        else

          nlz = 0
          ntz = 0
          izcl = 1
          izct = npts
          npts_out = npts
 
          ncomments = ncomments + 1
          if (flc == 0.0 .and. fhc == 0.0) then 
             comments(ncomments) = '| No filter, so no pads'
          else if (icaus_lc == 1 .and. icaus_hc == 1) then
             comments(ncomments) = '| Causal filter, no pads'
          else if (.not.(icaus_lc == 1 .and. icaus_hc == 1 .and. 
     :                                apply_pads)) then 
             comments(ncomments) = '| Acausal filter, but choose'//
     :                              ' not to apply pads'
          end if
          int_head(16) = ncomments

        end if

* Step 4b: FILTER

          write(nu_sum,'(a)') ' Filter parameters after enforcing'//
     :       ' power of two and four for causal and acausal filters:'
          write(nu_sum,'(a)') 
     :     ' flc, nslope_lc, icaus_lc, fhc, nslope_hc, icaus_hc'
          write(nu_sum,*) 
     :       flc, nslope_lc, icaus_lc, fhc, nslope_hc, icaus_hc
          write(nu_sum,'(a)') ' and nroll_lc, nroll_hc'
          write(nu_sum,*) nroll_lc, nroll_hc

        if (flc /= 0.0 .or. fhc /= 0.0) then

          write(nu_sum, '(a,1x,i7)') 
     :          ' Before call to filter, npts_out = ', npts_out
     
          call filter(a_out, npts_out, dt, flc, nroll_lc, icaus_lc,
     :              fhc, nroll_hc, icaus_hc, icaus_bn, 
     :              d4notch, filter_type, nc_filter_type)
        end if

* Step 4c: INTEGRATE TO VELOCITY AND DISPLACEMENT

   
          v04vd = 0.0
          d04vd = 0.0

          rmv_trnd = .false.
          call acc2vd(a_out, npts_out, dt, rmv_trnd, v04vd, d04vd, 
     :              v_out, d_out)
         
* Save pad-stripped displacement for plotting

          npts_out_no_pads = npts_out - nlz - ntz
          do i = 1, npts_out_no_pads
            if (what2plot(1:1) == 'A') then
              ts2plot(i,ifilt+iplot_unfiltered_a_av_avd) = 
     :             a_out(i+nlz)
            else if (what2plot(1:1) == 'V') then
              ts2plot(i,ifilt+iplot_unfiltered_a_av_avd) = 
     :             v_out(i+nlz)
            else if (what2plot(1:1) == 'D') then
              ts2plot(i,ifilt+iplot_unfiltered_a_av_avd) = 
     :             d_out(i+nlz)
            else
              print *, what2plot(1:1)//' /= A, V, or D; QUIT!'
              stop
            end if
          end do
          sps_ts(ifilt+iplot_unfiltered_a_av_avd) = sps
          npts_ts(ifilt+iplot_unfiltered_a_av_avd) = npts_out_no_pads
            
          write(nu_sum,'(a,1x,i6, 1x,i6, 1x,i6)') 
     :      ' npts, npts_out, npts_out_no_pads = ',
     :        npts, npts_out, npts_out_no_pads
          

        deallocate (a_out, v_out, d_out)

        END DO loop over filters
        
* Step 5: PLOT:

* Create name of output file

        plotnumber = '.pp_group_00'
        if (iplot < 10) then
          write(plotnumber(12:12),'(i1)') iplot
        else
          write(plotnumber(11:12),'(i2)') iplot
        end if
        
        plot_name = ' '        
        plot_name = filename_string(1:nc_filename_string)//
     :              plotnumber//'.'//f_in(1:nc_f_in)
     :                      
        call trim_c(plot_name, nc_plot_name)

        if (what2plot(1:1) == 'A') then
          plot_name = plot_name(1: nc_plot_name)//'.a.ps'
        else if (what2plot(1:1) == 'V') then
          plot_name = plot_name(1: nc_plot_name)//'.v.ps'
        else if (what2plot(1:1) == 'D') then
          plot_name = plot_name(1: nc_plot_name)//'.d.ps'
        else
          print *, what2plot(1:1)//' /= A, V, or D; QUIT!'
          stop
        end if

        call trim_c(plot_name, nc_plot_name)
        write(nu_sum,*)
        write(nu_sum, '(a)') ' plot_name: '//plot_name(1:nc_plot_name) 
        write(nu_sum,'(a)') ' i, ts_name:'
        do i = 1, nfilt+iplot_unfiltered_a_av_avd
          write(nu_sum, '(1x,i2, 1x,a)') i, ts_name(i)(1:nc_ts_name) 
        end do
        write(nu_sum,*)
        
        ntraces = nfilt + iplot_unfiltered_a_av_avd
        ndimen1 = npts_4alloc
        ndimen2 = nfilt + iplot_unfiltered_a_av_avd



        call tsplot_sub(ts2plot(1,1), ndimen1, ndimen2, 
     :          sps_ts, npts_ts, ntraces, 
     :          t_strt_in, t_plot_in, pp_cm,
     :          plot_name, ts_name, type_c)
 
        deallocate(a_in)
        deallocate (ts2plot)

      goto 1000  ! loop over input time series

9999  continue

      close(nu_ctl)
      close(nu_sum)

      stop
      end


      include '\forprogs\tsplot_sub.for'
 
      include '\forprogs\filt_plot_util_subs.for'
      
!      include '\forprogs\rdrvaa.for'
!      
!      include '\forprogs\get_lun.for'
!      include '\forprogs\upstr.for'
!      include '\forprogs\trim_c.for'
!      include '\forprogs\skip.for'
!      include '\forprogs\mnmax.for'
!      include '\forprogs\smc_nsps.for'
!      include '\forprogs\smcread.for'
!      include '\forprogs\datetime.for'
!      include '\forprogs\skipcmnt.for'
!      include '\forprogs\get_path_from_system_call.for'
!       
!      include '\forprogs\filter.for'
!      include '\forprogs\smcpadf.for'
!      include '\forprogs\construct_filename_extension.for'
!      include '\forprogs\acc2vd.for'
!      include '\forprogs\rmvtrend.for'
!      include '\forprogs\smc_npts.for'
!      
!      include '\forprogs\band.for'
!      include '\forprogs\hicut.for'
!      include '\forprogs\locut.for'                                   
!      include '\forprogs\mnmaxidx.for'
!      include '\forprogs\get_avg.for'
 