		      Program Filt_Plot_LC

* Filter a time series with a series of filter corners and
* plot results on a single page.  This program is intended to
* help in deciding what high-pass filter corner to choose.

*!Control file for program Filt_Plot_LC
*!
*! Note on use of "!" to denote a comment line.  This is only used
*! for lines in the group of input lines containing processing
*! parameters.  Once "pp:" is encountered, the program assumes that
*! all following lines until the next "pp:" or "stop" contain
*! the names of files to be processed (note that this is consistent
*! with file name restrictions:  starting a file name with "!" is legal).
*!
*! Revision of program involving a change in the control file on this date:
*   06/13/09
*!Summary file name:
* filt_plot_lc.sum
*!
*! NOTE: Those parameters most likely to stay fixed are immediately below,
*! before the list of files.  Those likely to change for various groups
*! of files are set below the "pp:" line.
*!
*!PARAMETERS FOR OBTAINING THE MEAN:
*!
*! mean info (contained after "pp:" line below):
*! type of mean (0=no correction, 1=standard average; -1=specify mean)
*! t4mean_strt, t4mean_stop, mean (use if type = -1)
*! NOTE: need entries even if the parameter is not used (because read
*! info use list-directed [unformatted] input) 
*!
*!PARAMETER FOR DETRENDING AFTER MEAN REMOVAL STEP:
*!
*! 0 = NO; 1 = YES  (the detrending removes a straight line fit to the first and last points)
*!
*!PARAMETERS FOR ZERO PADDING:
*!
*! Add pads if apply acausal filter (Y/N)?  This should almost always be
*! set to "Y".   Even if it is set to "N", note that the subroutine that
*! does the filtering adds pads (but it does not go to the first zero crossing 
*! and it does not save the padded time series)
* Y
*! Factor by which the number of zeros is increased beyond that determined by
*! the standard (BAP) equation.
* 1.0
*! Replace discarded portions of original data with zeros (Y/N)?  By doing this,
*! two horizontal components with the same number of time points before
*! padding (the usual case) will have the same number after padding and can
*! be rotated.
* Y
*!
*! Go to first zero crossing?
* Y
*! Use first point if smaller than first zero crossing?
* Y
*! Go to last zero crossing?
* Y
*! Use last point if smaller than last zero crossing?
* Y
*!
*! IMPORTANT NOTE: Care must be taken if the zero pads start at a point determined
*! by going inward to the first or last zero crossings
*! ("Go to first/last zero crossing = T").    
*! Look at the headers of the smc files to see how many data points were skipped.
*! Sometimes the beginning or end of the unfiltered time series has a drift, in
*! which case too many points will be skipped before the first zero crossing is reached.
*!
*!        11/03/06 - If zcross_l or zcross_t = F and use_first or use_last = T, 
*!                   pad with values equal to the last points.  Allow this option
*!                   to deal with cases such as filtering a displacement
*!                   time series with a low-frequency drift, which will have
*!                   a step offset if pad with zeros at the end.  Be careful in
*!                   in using this option for the first points, however (in
*!                   a future version I may eliminate the option for the beginning,
*!                   as generally the leading pad should be zero).  The need for
*!                   this option arose when I tried to filter a displacement
*!                   obtained by integrating a zeroth-order corrected acceleration.
*!                   The results were different than filtering the acceleration
*!                   before integration.
*!                   IF WANT TO PAD WITH ZEROS, STARTING AT 
*!                   BEGINNING AND END OF DATA, SET ZCROSS_L, ZCROSS_T = F AND
*!                   USE_FIRST, USE_LAST = F (This is potentially confusing, and
*!                   I may remove the option of padding with the first and last
*!                   values in a later version of the program)
*!
*! Remove mean before determining segment to be padded?
*! If Yes, the mean will be determined from the whole record.  If this
*! is not desireable, remove the mean outside of this program
* N
*! Remove mean after determining segment to be padded (the segment
*! will probably be different than the segment originally used to determine the
*! mean, and therefore the mean will no longer be 0.0. But then after removing
*! this mean, the segment to be padded will not be at 0.0 values, but I hope this
*! is a small problem.  I probably should iterate):
* N
*! Save padded time series?  (".pad" will be appended to file name)
* Y
*!
*!PARAMETERS FOR FILTERING:
*!
*! flc, nslope_lc, fhc (0 = no filter), nslope_hc, icaus(1=causal)
*!  (nslope >= 2,4,6, etc for causal and 4,8,12, etc for acausal; this is
*!   enforced in the program)
*! note that nslope is the asymptotic power of frequency, so that the filter
*! goes as f^{nslope} for a low-cut filter and f^{-nslope} for a high-cut filter.
*! In previous versions I used "nroll" as the parameter controlling the
*! asymptotic behaviro of the filter, the reason being that it is the parameter
*! used by the filter routines in BAP.   This was always
*! confusing, so I switched to nslope on 27 Oct. 2005, with a translation to
*! nroll in blpadflt (nroll = 0.5*nslope for causal and 
*! nroll = 0.25*nslope for acausal). (Note more confusing terminology:
*! nroll = 0.5*npoles for a causal Butterworth filter.) 
*! The response of an acausal filter
*! is not that of a Butterworth, because time domain filters are used.
*! Bazzuro et al. (2004 COSMOS paper) use the terminology npnp, where
*! "n" is the number of poles of a standard Butterworth filter.  The "npnp"
*! notation is used to indicate that two passes of a Butterworth filter are
*! used.   So, for example, "2p2p" means that two passes of a two-pole
*! Butterworth filter are used, with an asymptotic dependence as f^4 (for a
*! low-cut filter). This notation is used to distinguish this case
*! from that in which an acausal filter is simulated in the frequency domain, in
*! which case the filter can have a true Butterworth response function. For that 
*! case, a "4p" acausal filter constructed in the frequency 
*! domain and using the standard Butterworth response would have the same
*! asymptotic dependence (f^4).
*!
*!PARAMETERS FOR RESPONSE SPECTRA:
*!
*!Periods for response spectra (nperiods log spaced from per_low to per_high):
* 200 0.01 90
*!Damping value
*  0.05  
*!PLOT PARAMETERS
*!Plot v(1), d(2), or both(3)?
* 2
*!start time, duration to plot (<=0 to start at 1st pt; >>= 0 to plot all):
*0.0 900.0
*!peak-to-peak distance (cm) (ntraces*(p2p+0.5)<= 23.5, prog will reset if needed)
* 5.0
*!PROCESSING PARAMETERS AND FILES, IN GROUPS
*!
*pp: new set of parameters
*!mean: type (-1=specify,0=none,1=standard), t4mean_strt, t4mean_stop, 
*!mean (use if type = -1):
* 1  0.0 20.0  -999.
*!detrend after mean removal (0=no, 1=yes, in which case a line fit to
*!the first and last points is removed; note that the mean need not be removed
*!before detrending if type = 0 above):
* 0
*!flow, fhigh, number of filters (<=20) (frequencies will be evenly spaced in log f)
*  0.2, 2.0, 10
*!nslope_lc, fhc (0 = no filter), nslope_hc, icaus(1=causal) (nslope >= 2)
*!  (nslope >= 2,4,6, etc for causal and 4,8,12, etc for acausal; this is
*!   enforced in the program).  These are the same for each flc
* 8  0.0  4  0
*!Length of taper at beginning and end of time series, before adding zeros
*! (Not used if no filter or a causal filter, but still need this parameter
*! (in the input file):
* 0.0 0.0
*!Names of files to be processed with the parameters above:
*file1.smc
*file2.smc
*file3.smc
*pp: new set of parameters
*!mean: type (-1=specify,0=none,1=standard), t4mean_strt, t4mean_stop, 
*!mean (use if type = -1):
* 1  0.0 20.0  -999.
*!detrend after mean removal (0=no, 1=yes, in which case a line fit to
*!the first and last points is removed; note that the mean need not be removed
*!before detrending if type = 0 above):
* 0
*!flow, fhigh, number of filters (<=10) (frequencies will be evenly spaced in log f)
*  0.02, 0.8, 8
*!nslope_lc, fhc (0 = no filter), nslope_hc, icaus(1=causal) (nslope >= 2)
*!  (nslope >= 2,4,6, etc for causal and 4,8,12, etc for acausal; this is
*!   enforced in the program).  These are the same for each flc
* 8  0.0  4  0
*!Length of taper at beginning and end of time series, before adding zeros
*! (Not used if no filter or a causal filter, but still need this parameter
*! (in the input file):
* 0.0 0.0
*!Names of files to be processed with the parameters above:
*file10.smc
*file20.smc
*file30.smc
*stop
 
  
* Dates: 06/04/09 - Written by D. Boore, based on blpadflt and smctsplt
*        06/25/09 - Minor change to names of output files.
!        03/17/10 - Updated call to construct_filename_extension
!        06/03/10 - Calculate mean square differences of consecutive
!                   filtered displacements ("eps_disp", Aspa Zerva)
!                 - Increased max number of filters to 20
!        07/13/11 - Add tskip and tlength to call to construct_filename_extension
!                   (in two places)
!                 - Increased length of string_to_append from 40 to 100
!        02/02/12 - Renamed from filt_plot to filt_plot_lc, because I will also
!                   write a version that uses a list of flc, fhc (filt_plot_gen)
!        08/02/12 - Include lc filter frequency in the response spectrum column headers
!        05/16/14 - Modernize boolean operators, work on file names.
!        02/14/18 - Revise because the inputs to construct_filename_extension and filter
!                   have changed.
!        07/03/19 - Correct bug that first displacement plotted is rm, not the low value of flc.
!                 - The problem was that filter_type was set to "LC" after the call to construct_filename_extension 
!                   in the first pass through the loop over nfilt.

      parameter (small = 0.01) 

      character char_head(11)*80, comments(500)*80, 
     :                            ctl_cmmnts(100)*79
     
      character date_ctl_correct*8, date_ctl_in*30
 
      integer int_head(48), option, iplot_v_d_both
      
      real real_head(50)

      real a_in(:), a_out(:), v_out(:), d_out(:), dum(:)
      real ts2plot(:,:,:), 
     :     period(:), rd(:,:), rv(:,:), aa(:,:)
      allocatable :: a_in, a_out, v_out, d_out, dum, 
     :               ts2plot,  
     :               period, rd, rv, aa
     
      double precision avg
      character f_ctl*60, f_sum*60, f_in*120, buf*80, buf_c*79
      character f_stem*100, f_a*120, f_v*120, f_d*120, f_rs*120
      character string_to_append*100, buf3*3, buf4*4, buf6*6, 
     :          f_pad*120
      character rs_colhead_string*13, freq_c*6

      character filter_type*10

      character compute_v_d*10, precision*8
      
      real sps_ts(22)
      integer npts_ts(22)
      
      real eps_disp(20), eps_vel(20)
      character f_eps*200
      
      character plot_name(2)*120, ts_name(22,2)*120, 
     :   type_c(22,2)*3

      logical f_ctl_exist, f_in_exist, rmv_trnd, add_impulse
      logical zcross_l, zcross_t, use_first, use_last
      logical remove_mean_before, remove_mean_after, detrend
      logical apply_pads, save_padded_time_series, 
     :        replace_discarded_data_with_zeros
      
      real omega, omega2
      character rsheader(50)*15, damp_c*2
      character date_c*8, time_c*11
      
      real flc_in(20), fhc_in(20) 
      integer  nslope_lc_in(20), nslope_hc_in(20), icaus_in(20)

      twopi = 2.0*4.0*atan(1.)

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr = filt_plot_lc.ctl): '
        read(*, '(a)') f_ctl
        if (f_ctl == ' ') f_ctl = 'filt_plot_lc.ctl'
        call trim_c(f_ctl,nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

! Check date of control file:
      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      date_ctl_in = ' '
      read(nu_ctl,'(a)') date_ctl_in
      call trim_c(date_ctl_in,nc_date_ctl_in)
      
      date_ctl_correct = ' '
      date_ctl_correct = '06/13/09'
      call trim_c(date_ctl_correct,nc_date_ctl_correct)


      if (date_ctl_correct(1:nc_date_ctl_correct) /= 
     :    date_ctl_in(1:nc_date_ctl_in)) then
        write(*,'(a)') 
     :     ' The control file has the wrong date; update your '//
     :       'control file and rerun the program!'
        write(*,'(a)') 
     :     ' The date for the control file you used is '
     :       //date_ctl_in(1:nc_date_ctl_in)
        write(*,'(a)') 
     :     ' The correct date is '
     :       //date_ctl_correct(1:nc_date_ctl_correct)
        close(nu_ctl)
        stop
      end if
! Check date of control file:

      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call trim_c(f_sum,nc_f_sum)
      call get_lun(nu_sum)
      open(unit=nu_sum,file=f_sum(1:nc_f_sum),status='unknown')
      write(nu_sum,'(a)') ' Summary file for program Filt_Plot_LC'
      write(nu_sum,'(a)') 
     :     '   Using control file: '//f_ctl(1:nc_f_ctl)


* PARAMETERS FOR ZERO PADDING:

      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) == 'Y') then
        apply_pads = .true.
      else
        apply_pads = .false.
      end if

      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      read(nu_ctl, *) xfactor_zpads

      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) == 'Y') then
        replace_discarded_data_with_zeros = .true.
      else
        replace_discarded_data_with_zeros = .false.
      end if

      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) == 'Y') then
        zcross_l = .true.
      else
        zcross_l = .false.
      end if

      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) == 'Y') then
        use_first = .true.
      else
        use_first = .false.
      end if

      call skipcmnt(nu_ctl, ctl_cmmnts, nctl_cmmnts)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) == 'Y') then
        zcross_t = .true.
      else
        zcross_t = .false.
      end if

      call skipcmnt(nu_ctl, ctl_cmmnts, nctl_cmmnts)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) == 'Y') then
        use_last = .true.
      else
        use_last = .false.
      end if

      write(*,'(a)') ' zcross_l, zcross_t = '
      write(*, *)      zcross_l, zcross_t

      write(*,'(a)') ' use_first, use_last = '
      write(*, *)      use_first, use_last

      call skipcmnt(nu_ctl, ctl_cmmnts, nctl_cmmnts)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) == 'Y') then
        remove_mean_before = .true.
      else
        remove_mean_before = .false.
      end if
      write(*,'(a,1x,l1)') 
     :      ' remove_mean_before = ', remove_mean_before

      call skipcmnt(nu_ctl, ctl_cmmnts, nctl_cmmnts)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) == 'Y') then
        remove_mean_after = .true.
      else
        remove_mean_after = .false.
      end if
      write(*,'(a,1x,l1)') 
     :      ' remove_mean_after = ', remove_mean_after

      call skipcmnt(nu_ctl, ctl_cmmnts, nctl_cmmnts)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      if (buf(1:1) == 'Y') then
        save_padded_time_series = .true.
      else
        save_padded_time_series = .false.
      end if

* PARAMETERS FOR RESPONSE SPECTRA:

      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      do ihc = 1, nc_ctl_cmmnts
        write(nu_sum,'(1x,a)') ctl_cmmnts(ihc)
      end do

      read(nu_ctl,*) nperiods, per_low, per_high
      write(nu_sum,*) nperiods, per_low, per_high

* Setup response spectra stuff
      allocate(period(nperiods)) 
     
* Set up periods:
        
          if (nperiods .gt. 1) then
            dlogt = alog10(per_high/per_low)/float(nperiods-1)
          else
            dlogt = 0.0
          end if

          do i = 1, nperiods
            period(i) = per_low*10.0**(float(i-1) * dlogt)
          end do

      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      do ihc = 1, nc_ctl_cmmnts
        write(nu_sum,'(1x,a)') ctl_cmmnts(ihc)
      end do
      read(nu_ctl,*)  damp 
      write(nu_sum,*) damp 

* PARAMETERS FOR PLOT
      
      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      do ihc = 1, nc_ctl_cmmnts
        write(nu_sum,'(1x,a)') ctl_cmmnts(ihc)
      end do
      read(nu_ctl,*) iplot_v_d_both
      write(nu_sum,*) iplot_v_d_both 
      if (iplot_v_d_both /= 1 .and. 
     :    iplot_v_d_both /= 2 .and. 
     :    iplot_v_d_both /= 3) then
       print *,' ERROR: iplot_v_d_both = ', iplot_v_d_both, 
     :         ' QUITTING'
       close(nu_ctl)
       stop
      end if

      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      do ihc = 1, nc_ctl_cmmnts
        write(nu_sum,'(1x,a)') ctl_cmmnts(ihc)
      end do
      read(nu_ctl,*) t_strt_in, t_plot_in
      write(nu_sum,*) t_strt_in, t_plot_in 
      
      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      do ihc = 1, nc_ctl_cmmnts
        write(nu_sum,'(1x,a)') ctl_cmmnts(ihc)
      end do
      read(nu_ctl,*) pp_cm
      write(nu_sum,*) pp_cm
      
      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)

* CHECK FOR ANOTHER FILE TO PROCESS

1000  continue

        write(nu_sum,*)  
        
        buf = ' '
        read(nu_ctl,'(a)') buf
        write(nu_sum,'(1x,a)') buf
        call trim_c(buf,nc_buf)
        buf3 = ' '
        buf3 = buf(1:3)
        call upstr(buf3)
        buf4 = ' '
        buf4 = buf(1:4)
        call upstr(buf4)
        if (buf4 == 'STOP') go to 9999
        if (buf3 == 'PP:') then
* New processing parameters

* PARAMETERS FOR OBTAINING THE MEAN:

          call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
          do ihc = 1, nc_ctl_cmmnts
            write(nu_sum,'(1x,a)') ctl_cmmnts(ihc)
          end do
          read(nu_ctl,*) itype_mean, 
     :                 t4mean_strt_in, t4mean_stop_in, spec_mean
          write(nu_sum,*)  itype_mean, 
     :                   t4mean_strt_in, t4mean_stop_in, spec_mean

* PARAMETER FOR DETRENDING:

          call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
          do ihc = 1, nc_ctl_cmmnts
            write(nu_sum,'(1x,a)') ctl_cmmnts(ihc)
          end do
          read(nu_ctl,*) i_detrend
          write(nu_sum,*) i_detrend
          if (i_detrend == 0) then
            detrend = .false.
          else
            detrend = .true.
          end if
 
* PARAMETERS FOR FILTERING:

          call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
          do ihc = 1, nc_ctl_cmmnts
            write(nu_sum,'(1x,a)') ctl_cmmnts(ihc)
          end do
          read(nu_ctl,*) flow, fhigh, nfilt
          write(nu_sum,*) flow, fhigh, nfilt

          call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
          do ihc = 1, nc_ctl_cmmnts
            write(nu_sum,'(1x,a)') ctl_cmmnts(ihc)
          end do
          write(nu_sum,'(1x,a)') ' First entry is flc (line above '//
     :                           'is an image from the control file; '
          write(nu_sum,'(1x,a)') ' flc is not in that line, but is '//
     :                 'derived from the previous control input line).'
          read(nu_ctl,*) nslope_lc_dum, fhc_dum, 
     :                     nslope_hc_dum, icaus_dum

          filt_ratio = (fhigh/flow)**(1.0/float(nfilt-1))
          flc_in(1) = flow          
          do i= 2, nfilt
            flc_in(i) = flc_in(i-1)*filt_ratio
          end do
          
          do i= 1, nfilt
            nslope_lc_in(i) = nslope_lc_dum
            fhc_in(i) = fhc_dum 
            nslope_hc_in(i) = nslope_hc_dum
            icaus_in(i) = icaus_dum
            write(nu_sum,*) flc_in(i), nslope_lc_in(i), fhc_in(i), 
     :                     nslope_hc_in(i), icaus_in(i)
          end do
                 
          
* OPTION TO TAPER END OF TIME SERIES BEFORE ADDING LEADING AND
* TRAILING ZERO PADS

          call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
          do ihc = 1, nc_ctl_cmmnts
            write(nu_sum,'(1x,a)') ctl_cmmnts(ihc)
          end do
          read(nu_ctl,*) taper_beginning, taper_end
          write(nu_sum,'(a)') 
     :                 ' taper_beginning, taper_end upon input:'
          write(nu_sum,*) taper_beginning, taper_end
 
          call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)

          go to 1000

        end if ! finished reading a set of processing parameters

* Read the input file:

        f_in = ' '
        f_in = buf(1:nc_buf)
        call trim_c(f_in, nc_f_in)

        inquire(file=f_in(1:nc_f_in), exist=f_in_exist)
        if (.not. f_in_exist) then
          write(*,'(a)') ' ** Input file '//f_in(1:nc_f_in)//
     :    ' does not exist, moving to next file **'
          go to 1000
        end if

        write(*,'(a)') ' Processing file: '//f_in(1:nc_f_in)
        write(nu_sum,*)  
        write(nu_sum,'(a)') ' Processing file: '//f_in(1:nc_f_in)

        call smc_npts(f_in(1:nc_f_in), npts_max, sps)
        
        npts_4alloc = npts_max+10
        allocate (a_in(npts_max), ts2plot(npts_4alloc,nfilt+2,2), 
     :            dum(npts_max))
     
        ts2plot = 0.0
        
        tskip = 0.0
        tlength = -1.0

        call SMCRead(f_in(1:nc_f_in), tskip, tlength,
     :               a_in, char_head, int_head, real_head, comments)
        sps = real_head(2)
        dt = 1.0/sps
        ncomments = int_head(16)
        npts = int_head(17)
        tend = float(npts-1)/sps
        
        write(nu_sum,'(a)') ' sps, ncomments, npts = '
        write(nu_sum, *) sps, ncomments, npts

* Write comment headers (NOTE: I am distributing these throughout
* rather than in one place as in Bl_Fltr because smcpadf writes
* comment within the subroutine, and I wanted the comments in proper order.)

        call date(date_c)
        call time(time_c)


* Step 1: REMOVE MEAN FROM WHOLE RECORD

*  Determine the appropriate mean:

        t4mean_strt = t4mean_strt_in
        t4mean_stop = t4mean_stop_in
        
        if (itype_mean == 0) then
          n_t4mean = 0
          avg_all = 0.0
          ncomments = ncomments + 1
          comments(ncomments) = 
     :    '|   no mean removed.'
        else if (itype_mean == -1) then   ! use specified mean
          avg_all = spec_mean
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,1x,1pe12.5,a)') 
     :    '|   remove the specified mean ', avg_all,
     :      ' from whole record.'
        else if (itype_mean == 1) then
          if (t4mean_strt .lt. 0.0) t4mean_strt = 0.0
          if (t4mean_stop .gt. tend) t4mean_stop = tend
          n_t4mean_strt = nint(sps * t4mean_strt + 1.1)
          n_t4mean_stop = nint(sps * t4mean_stop + 1.1)
          n_t4mean = n_t4mean_stop - n_t4mean_strt + 1
          t4mean = float(n_t4mean -1)/sps
          avg = 0.0d0
          do i = n_t4mean_strt, n_t4mean_stop
            avg = avg + dble(a_in(i))
          end do
          avg_all = sngl(avg/dble(n_t4mean))
        else            ! shouldn't be here!
          write(nu_sum, '(a,1x,i3,1x,a)') '  Should not be here, '//'
     :         improper value of itype_mean.  itype_mean = ', 
     :         itype_mean, ' QUITTING!!'
          write(*) '  Should not be here, '//'
     :         improper value of itype_mean.  itype_mean = ', 
     :         itype_mean, ' QUITTING!!'
          go to 9999
        end if

*  Remove the mean from the whole record:
        write(nu_sum,'(a,1x,i6)') ' n_t4mean = ', n_t4mean
        write(nu_sum,'(a,1x,1pe12.5,1x,a)') 
     :   ' Removing mean = ', avg_all, ' from the whole record'

        do i = 1, npts
          a_in(i) = a_in(i) - avg_all
        end do

        
* Step 2: DETREND

        if (detrend) then
          call rmvtrend(a_in, npts)
          ncomments = ncomments + 1
          comments(ncomments) = 
     :    '|   detrend the time series by subtracting '//
     :    'a line fit to the first & last points' 
        end if	 

        do j = 1, 2
          do i = 1, npts
            ts2plot(i,1,j) = a_in(i)
          end do
          ts_name(1,j) = ' '
          ts_name(1,j) = f_in(1:nc_f_in)
          type_c(1,j) = 'ACC'
        end do
        sps_ts(1) = sps
        npts_ts(1) = npts

* Set initial velocity, displacement

        d0bl = 0.0d0
        v0bl = 0.0d0

        d0 = - d0bl
        v0 = - v0bl ! minus to account for subtraction of baseline

* Step 3: Compute zeroth-order velocity (for plotting)

        v04vd = 0.0
        d04vd = 0.0

        rmv_trnd = .false.
        call acc2vd(ts2plot(1,1,1), npts, dt, rmv_trnd, v04vd, d04vd, 
     :              ts2plot(1,2,1), dum)
        deallocate(dum)
        
        do i = 1, npts
          ts2plot(i,2,2) = ts2plot(i,2,1)
        end do
     
        sps_ts(2) = sps
        npts_ts(2) = npts
     
        if (itype_mean == 0) then
          option = 11 ! "xrm" in ts_name
        else
          option = 10
        end if
        flc = 0.0
        nslope_lc = 8
        fhc = 0.0
        nslope_hc = 8
        icaus_lc = 0
        filter_type = 'NF'
        nc_filter_type = 2
        icaus_hc = 0
        icaus_bn = 0
        call construct_filename_extension
     :      (tskip, tlength, option, 
     :       flc, nslope_lc, icaus_lc,
     :       fhc, nslope_hc, icaus_hc, icaus_bn,
     :       filter_type, nc_filter_type,
     :       taper_beginning, taper_end,     
     :       string_to_append, nc_s2appnd)

        do j = 1, 2
          ts_name(2,j) = ' '
          ts_name(2,j) = f_in(1:nc_f_in)//
     :          string_to_append(1:nc_s2appnd)//
     :               '.v'
          type_c(2,j) = 'VEL'
        end do
 
* Allocate arrays for response spectra

        rsheader = ' '
        if (nperiods /= 0) then
          allocate( 
     :     rd(nperiods,nfilt), rv(nperiods,nfilt), aa(nperiods,nfilt))     
        end if
     
* Step 4: Loop over nfilt

        eps_disp = 0.0
        eps_vel = 0.0
        
        loop over filters: DO ifilt = 1, nfilt

          flc = flc_in(ifilt)
          fhc = fhc_in(ifilt)
          nslope_lc = nslope_lc_in(ifilt)
          nslope_hc = nslope_hc_in(ifilt)
          icaus_lc = icaus_in(ifilt)

* Force power of two and four for causal and acausal filters, respectively:

          if (icaus_lc == 0) then   ! acausal
            if (nslope_lc .lt. 4) nslope_lc = 4
            if (nslope_hc .lt. 4) nslope_hc = 4
            nslope_lc = 4 * int(float(nslope_lc)/4.0)
            nslope_hc = 4 * int(float(nslope_hc)/4.0)
            nroll_lc = nslope_lc/4
            nroll_hc = nslope_hc/4
          else                     ! causal
            if (nslope_lc .lt. 2) nslope_lc = 2
            if (nslope_hc .lt. 2) nslope_hc = 2
            nslope_lc = 2 * int(float(nslope_lc)/2.0)
            nslope_hc = 2 * int(float(nslope_hc)/2.0)
            nroll_lc = nslope_lc/2
            nroll_hc = nslope_hc/2
          end if
          
        option = 10
! assume low-cut only. Use filt_plot_gen for both low and high cut
        filter_type = 'LC'
        nc_filter_type = 2
        call construct_filename_extension
     :      (tskip, tlength, option, 
     :       flc, nslope_lc, icaus_lc,
     :       fhc, nslope_hc, icaus_hc, icaus_bn,
     :       filter_type, nc_filter_type,
     :       taper_beginning, taper_end,     
     :       string_to_append, nc_s2appnd)
 
        do j = 1, 2
          ts_name(ifilt+2,j) = ' '
          if (itype_mean == 0) then
            ts_name(ifilt+2,j) = f_in(1:nc_f_in)//'.xrm'    
          else
            ts_name(ifilt+2,j) = f_in(1:nc_f_in)//'.rm'  
          end if
          call trim_c(ts_name(ifilt+2,j), nc_ts_name)
          ts_name(ifilt+2,j) = ts_name(ifilt+2,j)(1:nc_ts_name)//
     :            string_to_append(1:nc_s2appnd)
          call trim_c(ts_name(ifilt+2,j), nc_ts_name)
        end do
        
        ts_name(ifilt+2,1)(nc_ts_name+1:nc_ts_name+2) = '.v'
        
        ts_name(ifilt+2,2)(nc_ts_name+1:nc_ts_name+2) = '.d'

        do j = 1, 2        
          call trim_c(ts_name(ifilt+2,j), nc_ts_name)
        end do
        
        if (nperiods /= 0) then
* Create headers for response spectral output
          rs_colhead_string = ' '
          if (icaus_lc == 0) then
            rs_colhead_string(1:3) = 
     :       'alc'
          else
            rs_colhead_string(1:3) = 
     :       'clc'
          end if
          call freq2char(flc_in(ifilt), freq_c)
          rs_colhead_string(4:11) = freq_c(1:6)//'ns'
          write(rs_colhead_string(12:13), '(i2.2)') nslope_lc
          
          rsheader(2*(ifilt-1)+1) = 'sd'//rs_colhead_string(1:13) 
          rsheader(2*(ifilt-1)+2) = 'pa'//rs_colhead_string(1:13)  
        end if
        

        type_c(ifilt+2,1) = 'VEL'
        type_c(ifilt+2,2) = 'DIS'
 
* Need to allocate a_out, v_out, d_out differently because
* more than npts_max might be used in acausal filter routines (those
* routines pad the array beyond npts).  Must do this even if choose not to
* add pads when do an acausal filter.

* NOTE: the pad computation here is done because of the need to allocate the
* time series; nlz and ntz are passed into subroutine smcpadf.

        if (flc == 0.0 .and. fhc == 0.0) then   ! no filtering
          npad = 0      
          npad4alloc = 0      
        else                                        ! filtering
          if (flc /= 0.0 .and. fhc /= 0.0) then ! bandpass implied
              nroll_bp = max0(nroll_lc, nroll_hc)
              nslope_bp = max0(nslope_lc, nslope_hc)
          end if
          if (icaus_lc == 1) then
            npad = 0                                ! causal, no pads
            npad4alloc = 0                          ! causal, no pads
          else
            if (flc == 0.0) then                  ! hicut implied
              npad = 3.0*float(nroll_hc)/(fhc*dt)
            else if (fhc == 0.0) then             ! locut implied
              npad = 3.0*float(nroll_lc)/(flc*dt)
            else                                    ! band implied
              npad1=3.0*float(nroll_lc)/(flc*dt)
              npad2=6.0*float(nroll_bp)/((fhc-flc)*dt)
              npad = max0(npad1, npad2)
            end if
            if (xfactor_zpads .lt. 1.0) then
              npad4alloc =  npad  ! need this because the allocations in
                                  ! subroutine locut assume that npad is at
                                  ! least as large as the standard value.
            else
              npad4alloc = int(xfactor_zpads * float(npad))
            end if
            npad = int(xfactor_zpads * float(npad))
          end if
        end if

        nlz = npad/2
        ntz = npad/2
        izcl = nlz
        izct = ntz
        npts4alloc = npts_max + 2*npad4alloc + 2
 
        write(nu_sum,'(a)') ' After determination of npad: '//
     :     'npts, npts4alloc, npts_max, nlz, ntz, npad, npad4alloc ='
        write(nu_sum,*) 
     :      npts, npts4alloc, npts_max, nlz, ntz, npad, npad4alloc

* extra trailing pad of npad because the filter subroutines add their own 
* zero pad equal to npad to the end of the time series.  If no integration was
* being done, then that would be OK, but I want to add npad/2 to the front so 
* that the integration accounts properly for the filter transient. 
* I first added only an extra npad/2, but this led to a hard-to-find run time
* error.

        allocate (a_out(npts4alloc))
        allocate (v_out(npts4alloc))
        allocate (d_out(npts4alloc))
 
        a_out = 0.0
        v_out = 0.0
        d_out = 0.0
  
        do i = 1, npts
          a_out(i) = a_in(i)
        end do
        
* Step 4a: PAD TIME SERIES IF USE ACAUSAL FILTER

        
        if ((flc /= 0.0 .or. fhc /= 0.0) 
     :         .and. icaus_lc == 0
     :         .and. apply_pads) then  ! acausal filter, apply pads

          write(nu_sum,'(a)') 
     :          ' Inside pad time series if acausal filter:'
          write(nu_sum,'(a)') ' Working on file '//f_in(1:nc_f_in)
          write(nu_sum,'(a)') 
     :                 ' taper_beginning, taper_end upon input:'
          write(nu_sum,*) taper_beginning, taper_end
 
          call smcpadf(a_out, npts, sps, nlz, ntz, 
     :      replace_discarded_data_with_zeros, 
     :      taper_beginning, taper_end, 
     :      npts_out, izcl, izct,  
     :      remove_mean_before, remove_mean_after, 
     :      zcross_l, use_first, zcross_t, use_last,
     :      comments, ncomments)    

* Store new ncomments, npts:

          int_head(16) = ncomments            

        else

          nlz = 0
          ntz = 0
          izcl = 1
          izct = npts
          npts_out = npts
 
        end if

* Step 4b: FILTER

* Force power of two and four for causal and acausal filters, respectively:

          if (icaus_lc == 0) then   ! acausal
            if (nslope_lc .lt. 4) nslope_lc = 4
            if (nslope_hc .lt. 4) nslope_hc = 4
            nslope_lc = 4 * int(float(nslope_lc)/4.0)
            nslope_hc = 4 * int(float(nslope_hc)/4.0)
            nroll_lc = nslope_lc/4
            nroll_hc = nslope_hc/4
          else                     ! causal
            if (nslope_lc .lt. 2) nslope_lc = 2
            if (nslope_hc .lt. 2) nslope_hc = 2
            nslope_lc = 2 * int(float(nslope_lc)/2.0)
            nslope_hc = 2 * int(float(nslope_hc)/2.0)
            nroll_lc = nslope_lc/2
            nroll_hc = nslope_hc/2
          end if
          
          write(nu_sum,'(a)') ' Filter parameters after enforcing'//
     :       ' power of two and four for causal and acausal filters:'
          write(nu_sum,'(a)')' flc, nslope_lc, fhc, nslope_hc, icaus_lc'
          write(nu_sum,*) flc, nslope_lc, fhc, nslope_hc, icaus_lc
          write(nu_sum,'(a)') ' and nroll_lc, nroll_hc'
          write(nu_sum,*) nroll_lc, nroll_hc

!        if (flc /= 0.0 .or. fhc /= 0.0) then
! assume low-cut only. Use filt_plot_gen for both low and high cut
         if (flc /= 0.0) then

          write(nu_sum, '(a,1x,i7)') 
     :          ' Before call to filter, npts_out = ', npts_out
     
          d4notch = 0.01
          filter_type = 'LC'
          nc_filter_type = 2
          icaus_hc = 0
          icaus_bn = 0
          call filter(a_out, npts_out, dt, flc, nroll_lc, icaus_lc,
     :              fhc, nroll_hc, icaus_hc, icaus_bn, 
     :              d4notch, filter_type, nc_filter_type)
        else
          write(*,'(a)') ' ERROR: flc == 0.0; QUITTING'
          stop
        end if

* Step 4c: INTEGRATE TO VELOCITY AND DISPLACEMENT

   
          v04vd = 0.0
          d04vd = 0.0

          rmv_trnd = .false.
          call acc2vd(a_out, npts_out, dt, rmv_trnd, v04vd, d04vd, 
     :              v_out, d_out)
         
* Save pad-stripped displacement for plotting

          npts_out_no_pads = npts_out - nlz - ntz
          do i = 1, npts_out_no_pads
            ts2plot(i,ifilt+2,1) = v_out(i+nlz)
            ts2plot(i,ifilt+2,2) = d_out(i+nlz)
          end do
          sps_ts(ifilt+2) = sps
          npts_ts(ifilt+2) = npts_out_no_pads
          
! compute eps_disp and eps_vel
          if (ifilt > 1) then
            dum4eps_disp = 0.0
            dum4eps_vel = 0.0
            do i = 1, npts_out_no_pads
              dum4eps_vel = dum4eps_vel + 
     :           (ts2plot(i,ifilt+2,1) - ts2plot(i,ifilt-1+2,1))**2
              dum4eps_disp = dum4eps_disp + 
     :           (ts2plot(i,ifilt+2,2) - ts2plot(i,ifilt-1+2,2))**2
            end do
!            call abs_mnmaxidx(ts2plot(1,ifilt+2,1),
!     :                1,npts_out_no_pads,1,
!     :                amin,amax,indx_min,indx_max, abs_max_eps_vel) 
!            call abs_mnmaxidx(ts2plot(1,ifilt+2,2),
!     :                1,npts_out_no_pads,1,
!     :                amin,amax,indx_min,indx_max, abs_max_eps_disp) 

! It does not make sense to normalize by the max amp of the recent filtered 
! time series, because it can decrease the large changes in eps for low cutoff
! filters.

            abs_max_eps_vel = 1.0
            abs_max_eps_disp = 1.0
            eps_vel(ifilt) = (dum4eps_vel/real(npts_out_no_pads))/
     :                       abs_max_eps_vel**2
            eps_disp(ifilt) = (dum4eps_disp/real(npts_out_no_pads))/
     :                       abs_max_eps_disp**2
          end if
          
          write(nu_sum,'(a,1x,i6, 1x,i6, 1x,i6)') 
     :      ' npts, npts_out, npts_out_no_pads = ',
     :        npts, npts_out, npts_out_no_pads
          
* Step 8: COMPUTE RESPONSE SPECTRA
        IF (nperiods /= 0) THEN


* Find time of acceleration maximum:
          call mnmaxidx(a_out, 1, npts_out, 1,
     :                 amin,amax,indx_min,indx_max) 
          imax = indx_min
          if (abs(amax) .gt. abs(amin)) then
            imax = indx_max
          end if
          tmax = float(imax - 1)*dt
          write(nu_sum,'(a)') 
     :     ' amin,amax,indx_min,indx_max,imax,tmax = '
          write(nu_sum,'(1p2(1x,e10.3),0p3(1x,i6),1p1x,e10.3)') 
     :       amin,amax,indx_min,indx_max,imax,tmax

* Loop over periods and damping, computing and writing response spectra:

          write(*,'(a)') 
     :     ' Patience! Computing rd for ts '//
     :             ts_name(ifilt+2,1)(1:nc_ts_name)
          write(*,*)
          
          do iper = 1, nperiods
            write(*,'(a,1x,f9.3)') 
     :    '+ and period ', period(iper)

            omega = twopi/period(iper)
            omega2 = omega**2
            d04rs = 0.0
            v04rs = 0.0
            call rdrvaa(a_out, npts_out, omega, 
     :            damp, dt, 
     :            rd(iper, ifilt), rv(iper, ifilt), aa(iper, ifilt), 
     :            d04rs, v04rs)
!            write(nu_sum,'(a)')
!     :           npts_out,omega,damp,dt,d04rs,v04rs,iper,ifilt,rd:'
!            write(nu_sum,*)
!     :     npts_out,omega,damp,dt,d04rs,v04rs,iper,ifilt,rd(iper,ifilt)
          end do
          
        END IF

        deallocate (a_out, v_out, d_out)

        END DO loop over filters
        
! Write eps_vel and eps_disp

        call get_lun(nu_eps)
        f_eps = ' '        
        f_eps = f_in(1:nc_f_in)//'.flc_'        
        call trim_c(f_eps, nc_f_eps)
        write(f_eps(nc_f_eps+1:nc_f_eps+4),'(f4.2)') 
     :                   flow
        call trim_c(f_eps, nc_f_eps)
        f_eps = f_eps(1:nc_f_eps)//'_'
        call trim_c(f_eps, nc_f_eps)
        write(f_eps(nc_f_eps+1:nc_f_eps+4),'(f4.2)')
     :                   fhigh
        call trim_c(f_eps, nc_f_eps)
        f_eps = f_eps(1:nc_f_eps)//'.taper'
      
        call trim_c(f_eps, nc_f_eps)
        if (taper_beginning .ge. 10.0) then
          write(f_eps(nc_f_eps+1:nc_f_eps+4),'(f4.1)') 
     :             taper_beginning
        else
          f_eps(nc_f_eps+1:nc_f_eps+1) = '_'
          write(f_eps(nc_f_eps+2:nc_f_eps+4),'(f3.1)') 
     :             taper_beginning
        end if
      
        f_eps(nc_f_eps+5:nc_f_eps+5) = '_'
      
        call trim_c(f_eps, nc_f_eps)
        if (taper_end .ge. 10.0) then
          write(f_eps(nc_f_eps+1:nc_f_eps+4),'(f4.1)') 
     :             taper_end
        else
          f_eps(nc_f_eps+1:nc_f_eps+1) = '_'
          write(f_eps(nc_f_eps+2:nc_f_eps+4),'(f3.1)') 
     :             taper_end
        end if
      
        call trim_c(f_eps, nc_f_eps)
        
        f_eps = f_eps(1:nc_f_eps)//'.eps_vel_disp.out'
        call trim_c(f_eps, nc_f_eps)
        
        open(nu_eps,file=f_eps(1:nc_f_eps),status='unknown')
        write(nu_eps, '(1x,a, 5x,a, 4x,a, 3x,a)') 
     :         'ifilt', 'flc', 'eps_vel', 'eps_disp'
        do i = 2, nfilt
          write(nu_eps,'(4x,i2, 1x,f7.4, 1x,es10.3, 1x,es10.3)') 
     :         i, flc_in(i), eps_vel(i), eps_disp(i)
        end do
        close(nu_eps)
 
* Step 5: PLOT:

* Create name of output file

        do i = 1, 2

          plot_name(i) = ' '        
          plot_name(i) = f_in(1:nc_f_in)//'.flc_'        
          call trim_c(plot_name(i), nc_plot_name)
          write(plot_name(i)(nc_plot_name+1:nc_plot_name+4),'(f4.2)') 
     :                   flow
          call trim_c(plot_name(i), nc_plot_name)
          plot_name(i) = plot_name(i)(1:nc_plot_name)//'_'
          call trim_c(plot_name(i), nc_plot_name)
          write(plot_name(i)(nc_plot_name+1:nc_plot_name+4),'(f4.2)')
     :                   fhigh
          call trim_c(plot_name(i), nc_plot_name)
          plot_name(i) = plot_name(i)(1:nc_plot_name)//'.taper'
        
          call trim_c(plot_name(i), nc_plot_name)
          if (taper_beginning .ge. 10.0) then
            write(plot_name(i)(nc_plot_name+1:nc_plot_name+4),'(f4.1)') 
     :             taper_beginning
          else
            plot_name(i)(nc_plot_name+1:nc_plot_name+1) = '_'
            write(plot_name(i)(nc_plot_name+2:nc_plot_name+4),'(f3.1)') 
     :             taper_beginning
          end if
        
          plot_name(i)(nc_plot_name+5:nc_plot_name+5) = '_'
        
          call trim_c(plot_name(i), nc_plot_name)
          if (taper_end .ge. 10.0) then
            write(plot_name(i)(nc_plot_name+1:nc_plot_name+4),'(f4.1)') 
     :             taper_end
          else
            plot_name(i)(nc_plot_name+1:nc_plot_name+1) = '_'
            write(plot_name(i)(nc_plot_name+2:nc_plot_name+4),'(f3.1)') 
     :             taper_end
          end if
        
          call trim_c(plot_name(i), nc_plot_name)

        end do

        plot_name(1) = plot_name(1)(1: nc_plot_name)//'.v.ps'
        plot_name(2) = plot_name(2)(1: nc_plot_name)//'.d.ps'
        do i = 1, 2
          call trim_c(plot_name(i), nc_plot_name)
        end do

        ntraces = nfilt + 2
        ndimen1 = npts_4alloc
        ndimen2 = nfilt+2

        if (iplot_v_d_both == 1) then
          call tsplot_sub(ts2plot(1,1,1), ndimen1, ndimen2, 
     :          sps_ts, npts_ts, ntraces, 
     :          t_strt_in, t_plot_in, pp_cm,
     :          plot_name(1), ts_name(1,1), type_c(1,1))
        else if (iplot_v_d_both == 2) then
          call tsplot_sub(ts2plot(1,1,2), ndimen1, ndimen2, 
     :          sps_ts, npts_ts, ntraces, 
     :          t_strt_in, t_plot_in, pp_cm,
     :          plot_name(2), ts_name(1,2), type_c(1,2))
        else
          call tsplot_sub(ts2plot(1,1,1), ndimen1, ndimen2, 
     :          sps_ts, npts_ts, ntraces, 
     :          t_strt_in, t_plot_in, pp_cm,
     :          plot_name(1), ts_name(1,1), type_c(1,1))
          call tsplot_sub(ts2plot(1,1,2), ndimen1, ndimen2, 
     :          sps_ts, npts_ts, ntraces, 
     :          t_strt_in, t_plot_in, pp_cm,
     :          plot_name(2), ts_name(1,2), type_c(1,2))
        end if   


        deallocate(a_in)
        deallocate (ts2plot)

* Open output file, set up and write column labels

        IF (nperiods /= 0) THEN
        
* Create rs file name and open the file
          f_rs = ' '
          f_rs = f_in(1:nc_f_in)//'.taper'
        
          call trim_c(f_rs, nc_f_rs)
          if (taper_beginning .ge. 10.0) then
            write(f_rs(nc_f_rs+1:nc_f_rs+4),'(f4.1)') 
     :             taper_beginning
          else
            f_rs(nc_f_rs+1:nc_f_rs+1) = '_'
            write(f_rs(nc_f_rs+2:nc_f_rs+4),'(f3.1)') 
     :             taper_beginning
          end if
        
          f_rs(nc_f_rs+5:nc_f_rs+5) = '_'
        
          call trim_c(f_rs, nc_f_rs)
          if (taper_end .ge. 10.0) then
            write(f_rs(nc_f_rs+1:nc_f_rs+4),'(f4.1)') 
     :             taper_end
          else
            f_rs(nc_f_rs+1:nc_f_rs+1) = '_'
            write(f_rs(nc_f_rs+2:nc_f_rs+4),'(f3.1)') 
     :             taper_end
          end if
        
          call trim_c(f_rs, nc_f_rs)
          f_rs = f_rs(1: nc_f_rs)//'.filtered.rs.out'
          call trim_c(f_rs, nc_f_rs)

          call get_lun(nu_rs)
          open(unit=nu_rs,file=f_rs(1:nc_f_rs),status='unknown')

          write(nu_rs,'(5x,a, 4x,a, 50(1x,a15))') 'per', 'freq',
     :                          (rsheader(i),i=1,2*nfilt)
     
          do iper = 1, nperiods
            omega = twopi/period(iper)
            omega2 = omega**2
            write(nu_rs,'(2(1x,f7.3),1p, 50(4x,e12.5))')
     :      period(iper), 1.0/period(iper), 
     :      (rd(iper,ifilt), omega2*rd(iper,ifilt), 
     :                    ifilt = 1, nfilt) 
          end do ! loop over nperiods
          close(nu_rs)

          deallocate (rd, rv, aa)
        
        END IF
          
      goto 1000  ! loop over input time series

9999  continue

      deallocate(period)
      
      close(nu_ctl)
      close(nu_sum)

      stop
      end

      include '\forprogs\tsplot_sub.for'
 
      include '\forprogs\filt_plot_util_subs.for'
      
!      include '\forprogs\rdrvaa.for'
!      
!      include '\forprogs\get_lun.for'
!      include '\forprogs\upstr.for'
!      include '\forprogs\trim_c.for'
!      include '\forprogs\skip.for'
!      include '\forprogs\mnmax.for'
!      include '\forprogs\smc_nsps.for'
!      include '\forprogs\smcread.for'
!      include '\forprogs\datetime.for'
!      include '\forprogs\skipcmnt.for'
!      include '\forprogs\get_path_from_system_call.for'
!       
!      include '\forprogs\filter.for'
!      include '\forprogs\smcpadf.for'
!      include '\forprogs\construct_filename_extension.for'
!      include '\forprogs\acc2vd.for'
!      include '\forprogs\rmvtrend.for'
!      include '\forprogs\smc_npts.for'
!      
!      include '\forprogs\band.for'
!      include '\forprogs\hicut.for'
!      include '\forprogs\locut.for'                                   
!      include '\forprogs\mnmaxidx.for'
!      include '\forprogs\get_avg.for'
 