* ---------------------------------------------------------------- SMC2GMN
      Program SMC2GMN

* Reads an SMC filename from a control file, loops over a series of rotation 
* angles, for each computing the geometric mean of the two horizontal
* components.  Print out minimu, maximu, and fractile values of the geometric
* mean for specified periods.
 
* Structure of control file:
* Line 1 follows:
*!Control file for program SMC2GMN
*!Summary file name:
* smc2gmn.sum
*!nazm, azm_range
*  91 90.0
*!nper for gmean fractiles (max 20):
* 4
*! periods for gmean fractiles:
* 0.1 0.3 1.0 3.0
*!Damping value (damp):
* 0.05 
*! fractions to use for fractiles:
*  0.25 0.5 0.75
*!t4mean (removes a mean from the whole record, where the mean is
*!    determined from t = 0 to t = t4mean.  If t4mean = 0.0,
*!    the mean of the whole record is used, and if t4mean < 0, no mean is
*!    removed.  Note that if working with a processed record, it is
*!    inadvisible to remove the mean as this may produce unintended
*!    distortions at long periods --- so in most cases set t4mean < 0.0):
* -10.0  
*! Skip if npts differ by more than:
*  2
*! output file name
* smc2gmn.asc
*! list of file names (assumes pairs of files)  
*   0516a.smc
*   0516c.smc
*   0526a.smc
*   0526c.smc
* StoP

 
* Dates: 01/19/05 - Written by D. Boore, based on gmeanrot.  This is a 
*                   somewhat stripped-down version of the same program 
*                   except for the output
*        01/20/05 - Add test to skip only if npts differ by more than 
*                   a specified amount.
*        01/20/05 - Print ratios
*        01/21/05 - Add deallocates is failed similarity tests
*        01/21/05 - Different output (gm50 as denominator)
*        02/23/05 - Add a file with pga for components without rotation
*                   and pgarot50
*        10/17/07 - Transparent change to user needed because smcread can
*                   read two input formats

  
      character char_head(11,2)*80, comments(200,2)*80
      character cmnts2skip(20)*80

      integer int_head(48,2), nc_f_in(2),  nc_f_out, nc_rc(2)
      integer npts(2)

      real real_head(50,2), work(200), azm_added(200)
      real alat_sta(2), along_sta(2)
      real a_in(:,:), a_out(:,:)
      allocatable :: a_in, a_out
      
      real period(20), psa(2), gmean(20,200),
     :     gmean_min(20),dazm_gmnmin(20),
     :     gmean_max(20),dazm_gmnmax(20),
     :     psa_min(20),dazm_psamin(20),
     :     psa_max(20),dazm_psamax(20),
     :     psagmn1(20), psagmn2(20), psagmn3(20)
      real pgarot(200), pgagmn1, pgagmn2, pgagmn3
      
      real omega, omega2
          

      real rnull(2), azm_in(2), azm_out(2), sps(2)
      character buf_c*80, buf4_c*4, f_ctl*40, f_sum*40,
     :          buf_out*79
      character  f_in(2)*60, f_out*60, f_outpga*60,
     :           c_az*7, rc(2)*12, f_out_work*60, f_az_psa*60,
     :           f_az_sd(60)
      character per_c*5, colhead(120)*8, frmt_c*80
                         
      
      logical f_ctl_exist

      pi = 4.0*atan(1.0)
      twopi = 2.0*pi
      d2r = pi/180.0

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr = smc2gmn.ctl): '
        read(*, '(a)') f_ctl
        if (f_ctl .eq. ' ') f_ctl = 'smc2gmn.ctl'
        call trim_c(f_ctl,nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

      call skipcmnt(nu_ctl,cmnts2skip, nc_comments)
      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call get_lun(nu_sum)
      call trim_c(f_sum,nc_f_sum)
      open(unit=nu_sum,file=f_sum(1:nc_f_sum),status='unknown')    
      write(nu_sum,'(a)') ' Summary file for program GMeanRot'
      write(nu_sum,'(2a)') '   Using control file: ', f_ctl
      do i = 1, nc_comments
        write(nu_sum,'(a)') cmnts2skip(i)
      end do
      write(nu_sum,'(a)') f_sum

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) nazm, azm_range
      write(nu_sum,*) nazm, azm_range
      dazm = azm_range/float(nazm-1)
      write(nu_sum,'(a,1x,f7.3)') ' dazm = ', dazm      
     
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) nperiods
      write(nu_sum,*) nperiods
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) (period(i),i=1,nperiods)
      write(nu_sum,*) (period(i),i=1,nperiods)      
       
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) damp
      write(nu_sum,*) damp
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) f1, f2, f3
      write(nu_sum,*) f1, f2, f3
      
      n1 = int(f1 * float(nazm))
      n2 = int(f2 * float(nazm))
      n3 = int(f3 * float(nazm))
 
      write(*,'(a)') ' n1, n2, n3 ='
      write(*,*)       n1, n2, n3
      
      write(nu_sum,'(a)') ' n1, n2, n3 ='
      write(nu_sum,*)       n1, n2, n3
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) t4mean_in
      write(nu_sum,*) t4mean_in
         
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) npts_skip
      write(nu_sum,*) npts_skip
         
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      f_out = ' '
      read(nu_ctl,'(a)') f_out
      call trim_c(f_out,nc_f_out)
      write(nu_sum,'(a)') f_out(1:nc_f_out)
      call get_lun(nu_out)
        open(unit=nu_out,file=f_out(1:nc_f_out),
     :                                status='unknown')
      f_outpga = ' '
      f_outpga = f_out(1:nc_f_out)//'.pga'
      call trim_c(f_outpga,nc_f_outpga)
      call get_lun(nu_outpga)
        open(unit=nu_outpga,file=f_outpga(1:nc_f_outpga),
     :                                status='unknown')
      
     
* Write column headers:     
         
* Set up and write column headers of output file:

*      do i = 1, 6*nperiods
*        colhead(i) = ' '
*      end do
*       
*      do i = 1, nperiods
*         per_c = ' '
*         write(per_c,'(f5.2)') period(i)
*         call trim_c(per_c,nc_per_c)
*         colhead(6*(i-1)+1) = 'recxxxxx'
*         colhead(6*(i-1)+1)(4:8) = per_c(1:nc_per_c)
*         colhead(6*(i-1)+2) = 'minxxxxx'
*         colhead(6*(i-1)+2)(4:8) = per_c(1:nc_per_c)
*         colhead(6*(i-1)+3) = 'lowxxxxx'
*         colhead(6*(i-1)+3)(4:8) = per_c(1:nc_per_c)
*         colhead(6*(i-1)+4) = 'midxxxxx'
*         colhead(6*(i-1)+4)(4:8) = per_c(1:nc_per_c)
*         colhead(6*(i-1)+5) = 'hihxxxxx'
*         colhead(6*(i-1)+5)(4:8) = per_c(1:nc_per_c)
*         colhead(6*(i-1)+6) = 'maxxxxxx'
*         colhead(6*(i-1)+6)(4:8) = per_c(1:nc_per_c)
*      end do
*        
*      frmt_c = ' '
*      frmt_c = '(2(26x,a5), 1x, xxx(5x,a8))' 
*      write(frmt_c(17:19), '(i3)') 6*nperiods
*      call trim_c(frmt_c,nc_frmt_c)
*
*      write(nu_out,frmt_c(1:nc_frmt_c)) 'file1', 'file2',
*     :  (colhead(i), i=1,6*nperiods)

 
      do i = 1, 6*nperiods
        colhead(i) = ' '
      end do
       
        
      do i = 1, nperiods
         per_c = ' '
         write(per_c,'(f5.2)') period(i)
         call trim_c(per_c,nc_per_c)
         colhead(6*(i-1)+1) = 'recxxxxx'
         colhead(6*(i-1)+1)(4:8) = per_c(1:nc_per_c)
         colhead(6*(i-1)+2) = 'g50xxxxx'
         colhead(6*(i-1)+2)(4:8) = per_c(1:nc_per_c)
         colhead(6*(i-1)+3) = 'rg5xxxxx'
         colhead(6*(i-1)+3)(4:8) = per_c(1:nc_per_c)
         colhead(6*(i-1)+4) = 'g5rxxxxx'
         colhead(6*(i-1)+4)(4:8) = per_c(1:nc_per_c)
         colhead(6*(i-1)+5) = 'minxxxxx'
         colhead(6*(i-1)+5)(4:8) = per_c(1:nc_per_c)
         colhead(6*(i-1)+6) = 'maxxxxxx'
         colhead(6*(i-1)+6)(4:8) = per_c(1:nc_per_c)
      end do
        
      frmt_c = ' '
      frmt_c = '(2(26x,a5), 1x, xxx(5x,a8))' 
      write(frmt_c(17:19), '(i3)') 6*nperiods
      call trim_c(frmt_c,nc_frmt_c)

      write(nu_out,frmt_c(1:nc_frmt_c)) 'file1', 'file2',
     :  (colhead(i), i=1,6*nperiods)

      write(nu_outpga, '(26x,a5, 26x,a5, 4(5x,a8))')    
     :     'file1', 'file2', 
     :     'pgarot00', 'pgarotn1', 'pgarotn2', 'pgarotn3'
 
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

      write(nu_sum, '(a)') ' Just before 1000 continue '

1000  continue ! Loop over files

        write(nu_sum,*)

 
* First get max dimension and allocate arrays:
        do j = 1, 2
          buf_c = ' '
          read(nu_ctl,'(a)') buf_c
          call trim_c(buf_c,nc_buf)
          buf4_c = ' '
          buf4_c = buf_c(1:4)
          call upstr(buf4_c)

          if (buf4_c(1:4) .eq. 'STOP') then
            write(*, '(a)') 
     :           ' buf_c = '//buf_c(1:nc_buf)//'; QUITTING'
            close(unit=nu_sum)
            close(nu_out)
            close(nu_outpga)
            close(nu_ctl)
            stop
          end if

          f_in(j) = ' '
          f_in(j) = buf_c(1:nc_buf)
          call trim_c(f_in(j),nc_f_in(j))
        
          write(nu_sum,'(a,i1,a)') 
     :                 ' f_in(',j,') = '//f_in(j)(1:nc_f_in(j))
          write(*,'(a,i1,a)') 
     :                 ' f_in(',j,') = '//f_in(j)(1:nc_f_in(j))
          call smc_npts(f_in(j)(1:nc_f_in(j)), npts(j), sps(j))
        end do

        max_dimen = max0(npts(1), npts(2))
        allocate( a_in(max_dimen,2), a_out(max_dimen,2))

* Second, loop over input files, reading data:
        do j = 1, 2
          call SMCRead(f_in(j)(1:nc_f_in(j)), 0.0, 100000.0,
     :                 a_in(1,j), char_head(1,j), 
     :                 int_head(1,j), real_head(1,j), 
     :                 comments(1,j))

          npts(j)  = int_head(17,j)  ! should check equality of both time series
          rnull(j) = real_head(1,j)
          azm_in(j) = real(int_head(14,j))
          azm_out(j) = azm_in(j)  ! needed because Rotate overwrites azm_out
                                  ! in argument list

          do i = 1, npts(j)
            a_out(i,j) = a_in(i,j)
          end do
        end do

* Check for equality of sps:
        if ( sps(1) .ne. sps(2)) then
            write(nu_sum,'(a,1p, 2(1x,e12.5))') 
     :      ' SKIPPING: sps1 .ne. sps2; sps1, sps2 = ', 
     :        sps(1), sps(2)
          deallocate(a_in)
          deallocate(a_out)
          go to 1000
        end if
* Check for equality of npts:
        if ( iabs(npts(1) - npts(2)) .gt. npts_skip) then
            write(nu_sum,'(a,3(1x,i6))') 
     :      ' SKIPPING: iabs(npts1 - npts2) > '//
     :      'npts_skip; npts1, npts2, npts_skip = ', 
     :        npts(1), npts(2), npts_skip
          deallocate(a_in)
          deallocate(a_out)
          go to 1000
        end if
* Check for equality of rnull:
        if ( rnull(1) .ne. rnull(2)) then
          write(nu_sum,'(a,1p2(1x,e12.5))') 
     :      ' SKIPPING: rnull1 .ne. rnull2; rnull1, rnull2 = ', 
     :        rnull(1), rnull(2)
          deallocate(a_in)
          deallocate(a_out)
          go to 1000
        end if
* Check for azimuths being 90 degrees apart (compute the dot product):
        dot = cos(d2r*azm_in(1))*cos(d2r*azm_in(2))+
     :        sin(d2r*azm_in(1))*sin(d2r*azm_in(2))
        eps = 0.001
        if (abs(dot) .gt. eps) then
          write(*,'(a)') '  Dot product not 0.0; see SUM file'//
     :                   ' for details; SKIPPING!!!'
          write(nu_sum,'(a)') '  azm_in(1,2), dot = '
          write(nu_sum,*) azm_in(1), azm_in(2), dot
          deallocate(a_in)
          deallocate(a_out)
          go to 1000
        end if

* Obtain coordinates of stations from files and check for equality:
        do j = 1, 2
          alat_sta(j) = real_head(11,j)          
          along_sta(j) = real_head(12,j)          
        end do
        if (alat_sta(1) .ne. alat_sta(2) .or. 
     :      along_sta(1) .ne. along_sta(2) ) then
          write(*,'(a)') '  Station coordinates not equal, '//
     :                   ' see SUM file; SKIPPING!!!'
          write(nu_sum,'(a)') ' alat_sta(1), alat_sta(2) = '
          write(nu_sum,*)       alat_sta(1), alat_sta(2)
          write(nu_sum,'(a)') ' along_sta(1), along_sta(2) = '
          write(nu_sum,*)       along_sta(1), along_sta(2)
          deallocate(a_in)
          deallocate(a_out)
          go to 1000
        end if
        
* Passed all equality tests.

* Initialize summary variables:

        do iper = 1, nperiods
          gmean_min(iper) = 1.0e08
          gmean_max(iper) = 0.0
          dazm_gmnmin(iper)   = -999.0
          dazm_gmnmax(iper)   = -999.0
          psa_min(iper) = 1.0e08
          psa_max(iper) = 0.0
          dazm_psamin(iper)   = -999.0
          dazm_psamax(iper)   = -999.0
        end do
        
      
* Step: Remove a mean from the whole record

*  Determine the appropriate mean:
        do j = 1, 2
          t4mean = t4mean_in
          if(t4mean .lt. 0.0) then  ! do not remove a mean
            avg = 0.0
          else
            tend = float(npts(j)-1)/sps(j)
            if (t4mean .eq. 0.0 .or. t4mean .gt. tend) then   
              t4mean = tend  ! use whole record
            end if
          end if
          n_t4mean = sps(j) * t4mean + 1.1
          cum_avg = 0.0
          do i = 1, n_t4mean
            cum_avg = cum_avg + dble(a_in(i,j))
          end do
          avg = cum_avg/float(n_t4mean)
*          avg = sngl(cum_avg/n_t4mean)
*  Remove the mean from the whole record:
          do i = 1, npts(j)
            a_in(i,j) = a_in(i,j) - avg
          end do
        end do
 
* Step: Loop over azimuths:

        do iz = 1, nazm
        
          azm_added(iz) = float(iz-1)*dazm
          az_to_rot_into = azm_in(1) + azm_added(iz)
        
          do j = 1, 2
            azm_out(j) = azm_in(j)  ! needed because Rotate overwrites azm_out
                                    ! in argument list
            do i = 1, npts(j)
              a_out(i,j) = a_in(i,j)
            end do
          end do
 
          call Rotate(a_out(1,1),a_out(1,2),rnull(2),max_dimen,
     :      azm_out(1),azm_out(2),az_to_rot_into,npts(2))
     
* Compute pga:

* Get peak accelerations
	        
          call mnmaxidx(a_out(1,1), 1, npts(1), 1,
     :             ymin, ymax, indx_min, indx_max)
	  pga1 = abs(ymax)
	  if (abs(ymin) .gt. abs(ymax)) pga1 = abs(ymin)
	  
          call mnmaxidx(a_out(1,2), 1, npts(2), 1,
     :             ymin, ymax, indx_min, indx_max)
	  pga2 = abs(ymax)
	  if (abs(ymin) .gt. abs(ymax)) pga2 = abs(ymin)
	  
	  pgarot(iz) = sqrt(pga1*pga2)
	  

     
* Compute response spectra:

          do iper = 1, nperiods
            buf_out = ' '
            buf_out = '+ Patience! Computing rd for azm = '
            write(buf_out(36:40),'(f5.1)') azm_in(1)
            buf_out(41:43) = ' + '
            write(buf_out(44:48),'(f5.1)') azm_added(iz)
            buf_out = buf_out(1:48)//' and period '
            write(buf_out(61:68),'(f8.3)') period(iper) 
            write(*,'(a)') buf_out
            omega = twopi/period(iper)
            omega2 = omega**2
            do j = 1, 2
              dt = 1.0/sps(j)
              d0 = 0.0
              v0 = 0.0
              call rdrvaa(a_out(1,j), npts(j), omega, 
     :                  damp, dt, rd, rv, aa, d0, v0)
              psa(j) = omega2 * rd 
            end do
            gmean(iper, iz) = sqrt(psa(1) * psa(2))
            if (gmean(iper, iz) .lt. gmean_min(iper)) then
              gmean_min(iper) = gmean(iper, iz)
              dazm_gmnmin(iper) = az_to_rot_into - azm_in(1)
            end if
            if (gmean(iper, iz) .gt. gmean_max(iper)) then
              gmean_max(iper) = gmean(iper, iz)
              dazm_gmnmax(iper) = az_to_rot_into - azm_in(1)
            end if          

            psamin12 = amin1(psa(1),psa(2))
            if (psamin12 .lt. psa_min(iper)) then
              psa_min(iper) = psamin12
              dazm_psamin(iper) = az_to_rot_into - azm_in(1)
            end if
            psamax12 = amax1(psa(1),psa(2))
            if (psamax12 .gt. psa_max(iper)) then
              psa_max(iper) = psamax12
              dazm_psamax(iper) = az_to_rot_into - azm_in(1)
            end if          
          end do ! loop over nperiods
        
        end do  ! end loop over azimuths
      
* Compute and print various things: 

* For pga:

        do iz = 1, nazm
          work(iz) =  pgarot(iz)
        end do
       
! find the n1, n2, and n3 largest values in work

        pgagmn1 = select(n1, nazm, work)  
        pgagmn2 = select(n2, nazm, work)
        pgagmn3 = select(n3, nazm, work)  
        
        frmt_c = ' '
        frmt_c = '(2(1x,a30), 1p, 4(1x,e12.5))' 
        call trim_c(frmt_c,nc_frmt_c)
 
        write(nu_outpga,frmt_c(1:nc_frmt_c) )
     :                           (f_in(i)(1:nc_f_in(i)), i = 1,2),
     :                           pgarot(1), pgagmn1, pgagmn2, pgagmn3
 

* For psa:   

      
        do iper = 1, nperiods
      
          do iz = 1, nazm
            work(iz) =  gmean(iper, iz)
          end do
      
! find the n1, n2, and n3 largest values in work

          psagmn1(iper) = select(n1, nazm, work)  
          psagmn2(iper) = select(n2, nazm, work)
          psagmn3(iper) = select(n3, nazm, work)       
         
        end do
        
*        frmt_c = ' '
*        frmt_c = '(2(1x,a30), 1p, xxx(1x,e12.5))' 
*        write(frmt_c(17:19), '(i3.3)') 6*nperiods
*        call trim_c(frmt_c,nc_frmt_c)
 
*        write(nu_out,frmt_c(1:nc_frmt_c))
*     :                           (f_in(i)(1:nc_f_in(i)), i = 1,2),
*     :                           (gmean(j,1), gmean_min(j)/gmean(j,1),
*     :                            psagmn1(j)/gmean(j,1), 
*     :                            psagmn2(j)/gmean(j,1), 
*     :                            psagmn3(j)/gmean(j,1),
*     :                            gmean_max(j)/gmean(j,1), 
*     :                            j = 1, nperiods)

        frmt_c = ' '
        frmt_c = '(2(1x,a30), 1p, xxx(1x,e12.5))' 
        write(frmt_c(17:19), '(i3.3)') 6*nperiods
        call trim_c(frmt_c,nc_frmt_c)

        write(nu_out,frmt_c(1:nc_frmt_c))
     :                           (f_in(i)(1:nc_f_in(i)), i = 1,2),
     :                           (gmean(j,1), 
     :                            psagmn2(j),
     :                            gmean(j,1)/psagmn2(j), 
     :                            psagmn2(j)/gmean(j,1), 
     :                            gmean_min(j)/psagmn2(j), 
     :                            gmean_max(j)/psagmn2(j), 
     :                            j = 1, nperiods)
      
        deallocate(a_in)
        deallocate(a_out)
        
      go to 1000    ! loop back for another pair of components

      end
* ---------------------------------------------------------------- SMC_Rot

      include '\forprogs\rotate.for'     
      include '\forprogs\get_lun.for'     
      include '\forprogs\skip.for'     
      include '\forprogs\smcread.for'     
      include '\forprogs\smcwrite.for'     
      include '\forprogs\mnmaxidx.for'     
      include '\forprogs\trim_c.for'     
      include '\forprogs\upstr.for'     
      include '\forprogs\smc_npts.for'     
      include '\forprogs\distaz.for'     
      include '\forprogs\rc_subs.for'     
      include '\forprogs\skipcmnt.for'     
      include '\forprogs\rdrvaa.for'
      include '\forprogs\moment.for'
      include '\forprogs\select.for'
  

*      subroutine RCF (KPARM,CSTR_IN,NCSTR_IN,F,ibgn,iend, ISTAT)

 
