      Program SMC2RS_TS

* Read a smc file, compute the time series of an oscillator, and store the
* oscillator time series as an smc file.

* Sample control file:

*!Control file for SMC2RS_TS      ! first line
*! Revision of program involving a change in the control file on this date:
*   06/13/11
*!Name of summary file:
* smc2rs_ts.sum
*!
*!PROCESSING PARAMETERS AND FILES, IN GROUPS
*!
*pp: new set of parameters
*!For each group, three lines with:
*!  rd or rv time series (RD/RV)
*!  osc T, damp, V, v0, d0, tskip, length of ts_rd
*!  stem name of output file ("stop"=quit): 
*  RD
*   6.0 0.05 1.0 0.0 0.0 0.0 100.0
* .rdts_t06.smc
*!Names of files to be processed with the parameters above:
*   ts0.smc
*   ts1.smc
*pp: new set of parameters
*!For each group, three lines with:
*!  rd or rv time series (RD/RV)
*!  osc T, damp, V, v0, d0, tskip, length of ts_rd
*!  stem name of output file ("stop"=quit): 
*  RD
*   2.0 0.05 1.0 0.0 0.0 0.0 100.0
* .rdts_t02.smc
*!Names of files to be processed with the parameters above:
*   ts0.smc
*   ts1.smc
*STOP
  
* Dates: 04/20/00 - Written by D. Boore
*        02/16/01 - Added scale factor V to input, use allocatable arrays,
*                   and print out information in comment block
*        03/14/01 - Added initial velocities to rdcalcts
*        04/10/02 - Corrected name of program written to summary file
*        07/29/03 - Suggest name of control file
*        10/17/07 - Transparent change to user needed because smcread can
*                   read two input formats
*        02/07/08 - Various changes (NOTE: in a future version specify
*                   a length of input, so can snip out a section of record)
!        12/19/10 - Allow longer file names
!        12/21/10 - Modified from smc2rdts.  Renamed and include relative 
!                   velocity time series as output.
!        12/22/10 - Changed control file, added oscillator initial conditions
!        06/10/11 - Need to allocate y_in as its original length, not the length after 
!                   skipping into the record (because smcread reads the whole thing)
!        06/12/11 - Use max npts for allocation of arrays
!        06/13/11 - Made a major change in the control file, so that I can process a list of
!                   time series (such as might be made using \smsim\a_ts_drvr) with the same
!                   osc T, damp, V, v0, d0, tskip, length of ts_rd parameters.  This makes
!                   it much easier to set up the control file.
!        12/31/13 - Increase length of input file name
!        02/07/18 - Call subroutine rdrvaa_rd_rv_ts rather than rscalc_ts;  rdrvaa_rd_rv_ts is an updated
!                   version of rscalc_ts.  

      character precision*8


      real ts_rd(:), ts_rv(:), ts_in(:)
      allocatable :: ts_rd, ts_rv, ts_in

      character f_ctl*120, f_sum*120, f_in*200, f_stem*200,
     :          f_out*200, rd_or_rv*10
      character char_head(11)*80, comments(500)*80
      character buf*200, buf3*3, buf4*4
      character date_c*10, time_c*11
      integer int_head(48)
      real real_head(50)

      character  ctl_cmmnts(100)*79,
     :           date_ctl_correct*8, date_ctl_in*30
      integer read_status
      
      logical f_ctl_exist

      twopi = 2.0 * 4.0 * atan(1.0)

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file ("ENTER"=smc2rs_ts.ctl; '//
     :    'ctl-brk to quit): '
        read(*, '(a)') f_ctl
        call trim_c(f_ctl,nc_f_ctl)
        if (f_ctl(1:4) == '    ') f_ctl = 'smc2rs_ts.ctl'
        call trim_c(f_ctl,nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do
      call trim_c(f_ctl,nc_f_ctl)
      call get_lun(nu_ctl)
      open(unit=nu_ctl, file=f_ctl(1:nc_f_ctl), status='unknown')

      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      date_ctl_in = ' '
      read(nu_ctl,'(a)') date_ctl_in
      call trim_c(date_ctl_in,nc_date_ctl_in)
      
      date_ctl_correct = ' '
      date_ctl_correct = '06/13/11'
      call trim_c(date_ctl_correct,nc_date_ctl_correct)

      if (date_ctl_correct(1:nc_date_ctl_correct) /= 
     :    date_ctl_in(1:nc_date_ctl_in)) then
        write(*,'(a)') 
     :     ' The control file has the wrong date; update your '//
     :       'control file and rerun the program!'
        write(*,'(a)') 
     :     ' The date for the control file you used is '
     :       //date_ctl_in(1:nc_date_ctl_in)
        write(*,'(a)') 
     :     ' The correct date is '
     :       //date_ctl_correct(1:nc_date_ctl_correct)
        close(nu_ctl)
        stop
      end if
      
      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call trim_c(f_sum,nc_f_sum)
      call get_lun(nu_sum)
      open(unit=nu_sum, file=f_sum(1:nc_f_sum), status='unknown')
      write(nu_sum,'(a)') '  Summary program for program SMC2RS_TS'      

      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)

      loop over input lines: DO

        write(nu_sum,*)  
        
        buf = ' '
        read(nu_ctl,'(a)',iostat = read_status) buf
        if (read_status /= 0) then
          EXIT loop over input lines
        end if
        call trim_c(buf,nc_buf)
        write(nu_sum,'(a)') ' buf = '//buf(1:nc_buf)

        buf3 = ' '
        buf3 = buf(1:3)
        call upstr(buf3)
        buf4 = ' '
        buf4 = buf(1:4)
        call upstr(buf4)
        
        IF (buf4 == 'STOP') EXIT loop over input lines
        
        IF (buf3 == 'PP:') THEN
        
! New processing parameters

          call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
          rd_or_rv = ' '
          read(nu_ctl,'(a)') rd_or_rv
          call trim_c(rd_or_rv, nc_rd_or_rv)
          call upstr(rd_or_rv)
          write(nu_sum,'(a)') ' rd_or_rv = '//rd_or_rv(1:2)

          read(nu_ctl,*) tosc, dosc, V, v0, d0, tskip, tlength_out
          write(nu_sum,'(a,5(1x,f7.2))') 
     :     '    tosc, dosc, V, v0, d0, tskip, tlength_out =', 
     :          tosc, dosc, V, v0, d0, tskip, tlength_out
          
          f_stem = ' '
          read(nu_ctl,'(a)') f_stem
          call trim_c(f_stem, nc_f_stem)
          write(nu_sum,'(a)') ' f_stem = '//f_stem(1:nc_f_stem)

          call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)

          CYCLE loop over input lines
          
        END IF  ! end loop over "PP:"

* Read the input file:

        f_in = ' '
        f_in = buf
        call trim_c(f_in, nc_f_in)
        write(nu_sum,'(2a)') 
     :      '  Processing input file: ', f_in(1:nc_f_in)
  
   
        call smc_nsps(f_in(1:nc_f_in), npts_orig, sps)
        dt = 1.0/sps
        tlength_in = float(npts_orig-1)*dt - tskip 
        npts_in1 = int(tlength_in/dt + 0.1) + 1
        if ( tlength_in .lt. tlength_out) then
           nxtend = int((tlength_out - tlength_in)/dt + 0.1)
           npts_out = npts_in1 + nxtend
        else
           npts_out = npts_in1
        end if
        
! DEBUG
        write(*,*) ' f_in = '//f_in(1:nc_f_in)
        write(*,*) ' npts_orig, npts_out = ', npts_orig, npts_out
! DEBUG
        npts4alloc = max0(npts_orig, npts_out)
        
        allocate (ts_in(npts4alloc), ts_rd(npts4alloc), 
     :            ts_rv(npts4alloc))                                   ! added 06/12/11
!        allocate (ts_in(npts_out), ts_rd(npts_out), ts_rv(npts_out))  ! commented out 0n 06/10/11

        call smcread(f_in(1:nc_f_in), tskip, tlength_in, ts_in, 
     :             char_head, int_head, real_head, comments)

        precision = ' '
        if (int_head(47) .eq. 8) then
          precision = 'HIGH'
        else
          precision = 'STANDARD'
        end if
        npts_in = int_head(17)
  
        do i = npts_in+1, npts_out
          ts_in(i) = 0.0
        end do

        int_head(17) = npts_out

        write(nu_sum, '(a,4(1x,i6))') 
     :      ' npts_orig, npts_in1, npts_in, npts_out, npts4alloc = ', 
     :        npts_orig, npts_in1, npts_in, npts_out, npts4alloc
        write(nu_sum, '(a,3(1x,f7.2))') 
     :      ' sps, tlength_in, tlength_out = ', 
     :        sps, tlength_in, tlength_out

!        rnull = real_head(1)
!        v0 = real_head(38)
!        if (abs(v0) == abs(rnull)) then
!          v0 = 0.0
!        end if
!        d0 = real_head(39)
!        if (abs(d0) == abs(rnull)) then
!          d0 = 0.0
!        end if

*DEBUG
      print *,' npts_out before call rdcalc_ts: ', npts_out
*DEBUG

        omega = twopi/tosc
        call rdrvaa_rd_rv_ts(ts_in, npts_out, omega, dosc, dt, 
     :              rd, rv, aa, d0, v0, ts_rd, ts_rv)
 
*DEBUG
      print *,' npts_out after call rdcalcts: ', npts_out
*DEBUG

* Apply scale factor:
        do i = 1, npts_out
          ts_rd(i) = V * ts_rd(i)
          ts_rv(i) = V * ts_rv(i)
        end do
  
        write(nu_sum,'(a,1x, es10.3)') '    rd = ', rd
        write(nu_sum,'(a,1x, es10.3)') '    rv = ', rv
  
        f_out = ' '
        f_out = f_in(1:nc_f_in)//f_stem(1:nc_f_stem)
        
        call trim_c(f_out, nc_f_out)
        write(nu_sum,'(2a)') 
     :   '  Writing output file: ', f_out(1:nc_f_out)

 
        call get_date(date_c)
        call get_time(time_c)
  
        ncomments = int_head(16)
  
        ncomments = ncomments + 1
        comments(ncomments) = '|'
        ncomments = ncomments + 1
        comments(ncomments) = 
     :  '| Use program SMC2RS_TS to process the input file:'
        ncomments = ncomments + 1
        comments(ncomments) = 
     :  '| '//f_in(1:nc_f_in)
        ncomments = ncomments + 1
        comments(ncomments) = 
     :  '|   on '//date_c//' at time '//time_c
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,f7.3)') 
     :      '|   - tskip = ', tskip
        ncomments = ncomments + 1
        write(comments(ncomments),
     :      '(a)') 
     :      '|   - Oscillator period, damping, V, v0, d0 = '
        ncomments = ncomments + 1
        write(comments(ncomments),
     :      '(a,1x,f7.3,1x,f5.3,1x,f7.3, 2(1x,es12.5))') 
     :      '|  ', tosc, dosc, V, v0, d0
        ncomments = ncomments + 1
        comments(ncomments) = '|'

        if (rd_or_rv(1:2) == 'RD') then

          ncomments = ncomments + 1
          comments(ncomments) = '| relative displacement time series'
          ncomments = ncomments + 1
          comments(ncomments) = '|'

          int_head(16) = ncomments

          itype = 4 ! displacement
          call smcwrite(f_out(1:nc_f_out), precision,
     :                ts_rd, char_head, itype, 
     :              int_head, real_head, comments)
     
        else
     
          ncomments = ncomments + 1
          comments(ncomments) = '| relative velocity time series'
          ncomments = ncomments + 1
          comments(ncomments) = '|'

          int_head(16) = ncomments

          itype = 4 ! displacement
          call smcwrite(f_out(1:nc_f_out), precision,
     :                ts_rv, char_head, itype, 
     :              int_head, real_head, comments)
     
        end if
     
        deallocate(ts_in, ts_rd, ts_rv)

      END DO loop over input lines

      close(nu_ctl)
      close(nu_sum)

      end

      include '\forprogs\skip.for'
      include '\forprogs\skipcmnt.for'
      include '\forprogs\rdrvaa_rd_rv_ts.for'
      include '\forprogs\smcread.for'
      include '\forprogs\smcwrite.for'
      include '\forprogs\smc_nsps.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\get_date.for'
      include '\forprogs\get_time.for'
      include '\forprogs\upstr.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\mnmaxidx.for'
      include '\forprogs\distaz.for'
      
