* ------------------------------------------------------------------- SMC2VD
      Program SMC2VD


* Produces files of velocity and displacement from acceleration file

* Input from a control file; here is an example:
*!Control file for program SMC2VD
*! As many comment lines as desired, each starting with "!"
*! The string "pp:" indicates a new set of processing parameters
*! to be applied to the following smc files.  The parameters are given on the
*! following lines.
*! Revision of program involving a change in the control file on this date:
*   11/26/10
*!Name of summary file:
* smc2vd.sum
*!mean info:
*! type of mean (0=no correction, 1=standard average; -1=specify mean)
*! t4mean_strt, t4mean_stop, mean (use if type = -1)
*! NOTE: need entries even if the parameter is not used (because read
*! info use list-directed [unformatted] input) 
*! Note that if working with a processed record that includes the 
*! complete time series produced by the processing (e.g., the padded time series
*! for acausal filtering or the post-processing baseline corrected time series
*! produced from pad-stripped or pass-through data in the first NGA project), 
*! it is inadvisible to remove the mean as this may 
*! produce unintended distortions at long periods; in this case set t4mean < 0.0.
*! If the record is pad-stripped, however, as provided by most
*! data centers, without post-processing to remove drifts in displacement,
*! a mean determined from the overall record probably should be 
*! removed.  But in this case the displacement will probably be inaccurate
*! (compared to the displacement derived from the padded, filtered acceleration
*! time series), it may be that a better approach is to specify the initial
*! velocity and displacement of the record---I am working on that (11/26/10):
* 0  1.0 20.0  -999.
*!Use v0, d0 in determining velocity and displacement (this is used if dealing
*! with pad-stripped, filtered data)?
* N
*!Obtain v0, d0 from smc file headers (a letter is required even if "N" above)?
* Y
*!v0, d0 (entries are needed even if "N" is specified two parameters above 
*! or "Y" is specified above, but in those cases the entries are not used).
* 0.0 0.0
*! time for d_last, written into the summary file:
* 160.0
*!String to append to input file name when constructing the output filename.
* .xrm
*!List of files 
* t0781pea.smc
* t0781ped.smc
*Stop
 
* Dates: 11/06/01 - Written by D.M. Boore, patterned after SMC2FS2
*        12/19/01 - Write v_last, d_last to summary file
*        04/19/02 - Added more comments in control file discussing choice of
*                   t4mean and added option to ignore v0, d0
*        05/16/02 - Change in naming (see comments in control file)
*        09/23/02 - Add more parameters to control the mean to be removed, 
*                   including the option to specify the mean
*        10/10/02 - Don't write t4mean for each file in sum file
*        10/10/02 - Write a with mean removed
*        10/18/02 - Changed all occurrences of npts_smc to npts and 
*                   added time for d_last, written to summary file.
*        10/21/02 - Added stuff to comment headers
*        07/17/03 - Add sps to smc_npts parameter list
*        11/03/03 - Write acceleration file only if avg .ne. 0.0.  This is a
*                   temporary solution to prevent overwriting the input file
*                   if it happens that the input file name has "a.smc".   I
*                   should take advantage of the longer file names allowed
*                   by F90 and redo the naming.
*        04/13/04 - Construct output filenames by appending ".v", ".d" to the
*                   input file name.
*        06/28/04 - Slight change in output (to keep from losing information).
*        10/15/07 - Allow for a higher resolution file format. Use the same 
*                   precision for output as for input.
*        02/06/09 - Correct error in t4mean_strt and t4mean_stop being reset from their input
*                   values when processing a list of files.  The problem is that if a file with
*                   a shorter record length is processed before one with a longer record
*                   length, a mean supposedly determined for the whole record was determined
*                   only for the length of the shorter preceeding record.
*        05/28/09 - Increase length of file names
!        11/26/10 - Add revision date of control file and a string to be appended to the filename
!                   before adding ".v" and ".d."
!                 - Read in v0, d0 (for possible future use).
!        12/04/10 - I increased the length of the comments string so that the complete
!                   name of the file being processed would be written in the smc file
!                   header, but this produced an error in smcwrite (comments must be
!                   declared with the same string length in the main program and in the 
!                   subroutine).  Changing the declaration in smcwrite would then mean
!                   that I would have to change it in every routine that uses smcwrite,
!                   and in addition, the smc format would no longer be the "official"
!                   format.  For these reasons I reverted back to the previous version
!        03/06/12 - Replace "n_t4mean" by "dble(real(n_t4mean)" in 
!                   "avg = sngl(cum_avg/dble(real(n_t4mean))"
!        11/07/16 - Do not include the path of the input file in the output file name.


c
c Dimension and declaration statements:

      real a(:), v(:), d(:)
      allocatable :: a, v, d

      integer npts, nc_f_a, nc_f_v, nc_f_d
      real tend
      real*8 cum_avg
      
      integer status

      character date_c*8, time_c*11, extension_c*30

      character date_ctl_correct*8, date_ctl_in*30

      character char_head(11)*80, cmnts2skip(50)*80
      character comments(500)*80 
!      character comments(500)*200 

      real real_head(50)
      integer int_head(48)

      character buf*300, buf_upper*4, 
     :        f_sum*300, f_a*300, f_v*300, f_d*300, f_out*300,
     :        path*300
     

      character f_ctl*200, precision*20
      logical f_exist, f_a_exist, rmv_trnd, 
     :  use_v0_d0, v0_d0_from_smc_header

      f_exist = .false.
      do while (.not. f_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr=SMC2VD.CTL;'//
     :    ' ctl-brk to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl(1:4) .eq. '    ') f_ctl = 'smc2vd.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_exist)
        if (.not. f_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

      call skipcmnt(nu_ctl, cmnts2skip,nc_cmnts2skip)
      date_ctl_in = ' '
      read(nu_ctl,'(a)') date_ctl_in
      call trim_c(date_ctl_in,nc_date_ctl_in)
      
      date_ctl_correct = ' '
      date_ctl_correct = '11/26/10'
      call trim_c(date_ctl_correct,nc_date_ctl_correct)

      if (date_ctl_correct(1:nc_date_ctl_correct) .ne. 
     :    date_ctl_in(1:nc_date_ctl_in)) then
        write(*,'(a)') 
     :     ' The control file has the wrong date; update your '//
     :       'control file and rerun the program!'
        close(nu_ctl)
        stop
      end if
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call trim_c(f_sum,nc_f_sum)
      call get_lun(nu_sum)
      open(unit=nu_sum,file=f_sum(1:nc_f_sum),status='unknown')

      write(nu_sum,'(a)') ' Output of program SMC2VD:'
      write(nu_sum,'(2a)') '   Control file = ', f_ctl

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) 
     :   itype_mean, t4mean_strt_in, t4mean_stop_in, spec_mean
      write(nu_sum,'(a)') 
     :  ' itype_mean, t4mean_strt_in, t4mean_stop_in, spec_mean:'
      write(nu_sum,*) 
     :    itype_mean, t4mean_strt_in, t4mean_stop_in, spec_mean

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      write(nu_sum,'(a)') buf(1:nc_buf)
      if (buf(1:nc_buf) .eq. 'Y') then
        use_v0_d0 = .true.
      else
        use_v0_d0 = .false.
      end if
      write(nu_sum,'(a,1x,l1)') 
     :  ' use_v0_d0 = ', use_v0_d0
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      write(nu_sum,'(a)') buf(1:nc_buf)
      if (buf(1:nc_buf) .eq. 'Y') then
        v0_d0_from_smc_header = .true.
      else
        v0_d0_from_smc_header = .false.
      end if
      write(nu_sum,'(a,1x,l1)') 
     :  ' v0_d0_from_smc_header = ', v0_d0_from_smc_header
      
      

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) v0_in, d0_in
      write(nu_sum,'(a, 2(1x,es10.3))') 
     :  ' v0_in, d0_in =', v0_in, d0_in

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) t_d_last
      write(nu_sum,'(a)') 
     :  ' t_d_last:'
      write(nu_sum,*) 
     :    t_d_last

      write(nu_sum,'(1x,a3, t57,a, t67,a, t78,a)') 
     :        'num', 'Files', 'v_last', 'd_last'

      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      extension_c = ' '
      read(nu_ctl,'(a)') extension_c
      call trim_c(extension_c, nc_extension_c)
      write(nu_sum,'(a)') 
     :  ' extension_c ='//extension_c(1:nc_extension_c)
 
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

      num = 0
      
      loop over files: DO
      buf = ' '
      read(nu_ctl,'(a)',iostat=status) buf
      if(status /= 0) then
        EXIT loop over files
      end if
      call trim_c(buf,nc_buf)
      buf_upper = ' '
      buf_upper = buf(1:4)
      call upstr(buf_upper)
      if(nc_buf == 0 .or. buf(1:1) == '!') then
        CYCLE loop over files
      end if
      if(buf_upper(1:4) == 'STOP') then
        EXIT loop over files
      end if
      
      f_a = ' '
      f_a = buf(1:nc_buf)
      nc_f_a = nc_buf 
      
      f_a_exist = .false.
      inquire(file=f_a(1:nc_f_a), exist=f_a_exist)
*      write(*,'(a)') ' f_a: ', f_a(1:nc_f_a)
*      write(*,'(a, 1x, l1)') ' f_a_exist = ', f_a_exist

      if (.not. f_a_exist) then
          write(*,'(a)') ' ****** FILE '//f_a(1:nc_f_a)// 
     :          ' DOES NOT EXIST; SKIPPING!!! ******* '
          CYCLE loop over files
      end if

* Passed tests; proceed

      num = num + 1

* Get number of points on this loop:

      call smc_npts(f_a(1:nc_f_a), npts, sps)

*      write(nu_sum,*)
*      write(nu_sum, '(a, 2(1x,i5))') ' npts = ', npts

      allocate( a(npts+5), v(npts+5), d(npts+5) )

* Now read the data from the files:

      write(*,'(1x,a)') 
     :   'Processing file:', f_a(1:nc_f_a)

      tskip = 0.0
      tlength = -1.0

      call SMCRead(f_a(1:nc_f_a),  
     :               tskip, tlength,
     :               a,  
     :               char_head, int_head, real_head, comments)
     
      precision = ' '
      if (int_head(47) == 8) then
        precision = 'HIGH'
      else
        precision = 'STANDARD'
      end if
      
      call trim_c(precision,nc_precision)
     
 
      rnull = real_head(1)
      if (use_v0_d0) then
        if (v0_d0_from_smc_header) then
          v0 = real_head(38)
          if (abs(v0) == abs(rnull)) then
            v0 = 0.0
          end if
          d0 = real_head(39)
          if (abs(d0) == abs(rnull)) then
            d0 = 0.0
          end if
        else
          v0 = v0_in
          d0 = d0_in
        end if
      else
        v0 = 0.0
        d0 = 0.0
      end if
      
      write(nu_sum,'(1x,a,2(1x,es10.3))') 'v0, d0 = ', v0, d0

      sps = real_head(2)
      ncomments_a = int_head(16)
      npts = int_head(17)
      dt = 1.0/sps
      tend = float(npts-1)*dt

      if (t_d_last >= tend) then
         n_d_last = npts
      else
         n_d_last = t_d_last/dt + 1
      end if


*      write(nu_sum,'(3x,a,f8.6,a,i5,a,f8.2)') 
*     :     'signal_in: dt=', dt, 
*     :     ' npts=', npts,' tend=', tend

* Step 1: Remove a mean from the whole record

      t4mean_strt = t4mean_strt_in
      t4mean_stop = t4mean_stop_in
        
*  Determine the appropriate mean:
      if (itype_mean == 0) then
        n_t4mean = 0
        avg = 0.0
      else if (itype_mean == -1) then   ! use specified mean
        avg = spec_mean
      else if (itype_mean == 1) then
        if (t4mean_strt < 0.0) t4mean_strt = 0.0
        if (t4mean_stop > tend) t4mean_stop = tend
        n_t4mean_strt = nint(sps * t4mean_strt + 1.1)
        n_t4mean_stop = nint(sps * t4mean_stop + 1.1)
        n_t4mean = n_t4mean_stop - n_t4mean_strt + 1
        t4mean = float(n_t4mean -1)/sps
*        write(nu_sum,'(a,1x,f6.2)') ' t4mean = ', t4mean
        cum_avg = 0.0d0
        do i = n_t4mean_strt, n_t4mean_stop
          cum_avg = cum_avg + dble(a(i))
        end do
        avg = sngl( cum_avg/dble( real(n_t4mean) ) )
      else            ! shouldn't be here!
        write(nu_sum, '(a,1x,i3,1x,a)') '  Should not be here, '//'
     :       improper value of itype_mean.  itype_mean = ', 
     :       itype_mean, ' QUITTING!!'
        write(*) '  Should not be here, '//'
     :       improper value of itype_mean.  itype_mean = ', 
     :       itype_mean, ' QUITTING!!'
        deallocate (a, v, d)
        EXIT loop over files
      end if

*  Remove the mean from the whole record:
      do i = 1, npts
        a(i) = a(i) - avg
      end do

* Step2: Integrate to velocity and displacement

      rmv_trnd = .false.
      call acc2vd(a, npts, dt, rmv_trnd, v0, d0, v, d)

* Step 3: Write output:

* Write acceleration if avg .ne. 0.0:
      if (avg /= 0.0) then
        indp = index(f_a,'.')
        f_out = ' '
!        f_out = f_a(1:nc_f_a)
!        if (indp .lt. 9 .and. n4vd .eq. 0) then
!          f_out = f_out(1:indp-1)//'a'//f_out(indp:nc_f_a)
!          nc_f_out = nc_f_a + 1
!        else
!          f_out(indp+n4vd:indp+n4vd) = 'a'
!          nc_f_out = nc_f_a
!        end if

       path = ' '
       call get_path_from_file_name(f_a,nc_f_a,path,nc_path)
  
       if (nc_extension_c > 0) then
         f_out = f_a(nc_path+1:nc_f_a)//
     :             extension_c(1:nc_extension_c)//'.a'
       else
         f_out = f_a(nc_path+1:nc_f_a)//'.a'
       end if
       call trim_c(f_out, nc_f_out)

        write(*,'(1x,a)') 
     :   'Writing acceleration output to file:', f_out(1:nc_f_out)

        iflag = 0 ! not used by current PC version of datetime; Chris
                ! Stephens would modify datetime.for to include Unix routines
        call datetime(date_c, time_c, iflag)

        ncomments = ncomments_a
        ncomments = ncomments + 1
        comments(ncomments) = '|'

        ncomments = ncomments + 1
        comments(ncomments) = 
     :      '| Acceleration, from processing acc file '//f_a(1:nc_f_a)
        ncomments = ncomments + 1
        comments(ncomments) = 
     :      '| using program SMC2VD'
        ncomments = ncomments + 1
        comments(ncomments) = 
     :      '|   on '//date_c//' at time '//time_c

        if (itype_mean == 0) then
          ncomments = ncomments + 1
          write(comments(ncomments),'(a)') 
     :      '|   - no mean subtracted from record'
        else if (itype_mean == 1) then
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,1x,1pe12.5,a)') 
     :      '|   - subtract mean = ', avg, ' from whole record'
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,1x,f7.3,a,1x,f7.3)') 
     :      '|     mean determined from t = ', t4mean_strt, 
     :      ' to t = ', t4mean_stop
        else if (itype_mean == -1) then
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,1x,1pe12.5,a)') 
     :      '|   - subtract mean = ', avg, ' from whole record'
          ncomments = ncomments + 1
          write(comments(ncomments),'(a)') 
     :      '|     mean was specified.' 
        else
          write(*,'(a)') ' Not supposed to be here.'
        end if

        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,f7.3)') 
     :      '|   Initial velocity = ', v0

        ncomments = ncomments + 1
        comments(ncomments) = '|   resulting in output file '//
     :      f_out(1:nc_f_out)

        ncomments = ncomments + 1
        comments(ncomments) = '|'

        int_head(16) = ncomments 
        itype = 2


        call SMCWrite(f_out, precision,
     :                  a, char_head, itype, 
     :                    int_head, real_head, comments)
     
 
      end if

* Write velocity:

      path = ' '
      call get_path_from_file_name(f_a,nc_f_a,path,nc_path)

!      indp = index(f_a,'.')  ! Probably should find last occurrence of period
      f_v = ' '
      if (nc_extension_c > 0) then
        f_v = f_a(nc_path+1:nc_f_a)//extension_c(1:nc_extension_c)//'.v'
      else
        f_v = f_a(nc_path+1:nc_f_a)//'.v'
      end if
      call trim_c(f_v, nc_f_v)

!      f_v = f_a(1:nc_f_a)
!      if (indp .lt. 9 .and. n4vd .eq. 0) then
!        f_v = f_v(1:indp-1)//'v'//f_v(indp:nc_f_a)
!        nc_f_v = nc_f_a + 1
!      else
!        f_v(indp+n4vd:indp+n4vd) = 'v'
!        nc_f_v = nc_f_a
!      end if
      write(*,'(1x,a)') 
     :   'Writing velocity output to file:', f_v(1:nc_f_v)

      iflag = 0 ! not used by current PC version of datetime; Chris
                ! Stephens would modify datetime.for to include Unix routines
      call datetime(date_c, time_c, iflag)
      ncomments = ncomments_a
      ncomments = ncomments + 1
      comments(ncomments) = '|'

      ncomments = ncomments + 1
      comments(ncomments) = 
     :      '| Velocity, from processing acc file '//f_a(1:nc_f_a)
      ncomments = ncomments + 1
      comments(ncomments) = 
     :      '| using program SMC2VD'

      ncomments = ncomments + 1
      comments(ncomments) = 
     :      '|   on '//date_c//' at time '//time_c

      if (itype_mean == 0) then
        ncomments = ncomments + 1
        write(comments(ncomments),'(a)') 
     :      '|   - no mean subtracted from record'
      else if (itype_mean == 1) then
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,1pe12.5,a)') 
     :      '|   - subtract mean = ', avg, ' from whole record'
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,f7.3,a,1x,f7.3)') 
     :      '|     mean determined from t = ', t4mean_strt, 
     :      ' to t = ', t4mean_stop
      else if (itype_mean == -1) then
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,1pe12.5,a)') 
     :      '|   - subtract mean = ', avg, ' from whole record'
        ncomments = ncomments + 1
        write(comments(ncomments),'(a)') 
     :      '|     mean was specified.' 
      else
        write(*,'(a)') ' Not supposed to be here.'
      end if

        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,f7.3)') 
     :      '|   Initial velocity = ', v0

      ncomments = ncomments + 1
      comments(ncomments) = '|   resulting in output file '//
     :      f_v(1:nc_f_v)

      ncomments = ncomments + 1
      comments(ncomments) = '|'

      int_head(16) = ncomments 
      itype = 3

      call SMCWrite(f_v, precision,
     :                v, char_head, itype, 
     :                    int_head, real_head, comments)
 
      path = ' '
      call get_path_from_file_name(f_a,nc_f_a,path,nc_path)

      f_d = ' '
      if (nc_extension_c > 0) then
        f_d = f_a(nc_path+1:nc_f_a)//extension_c(1:nc_extension_c)//'.d'
      else
        f_d = f_a(nc_path+1:nc_f_a)//'.d'
      end if
      call trim_c(f_d, nc_f_d)
!      f_d = f_a(1:nc_f_a)
!      if (indp .lt. 9 .and. n4vd .eq. 0) then
!        f_d = f_d(1:indp-1)//'d'//f_d(indp:nc_f_a)
!        nc_f_d = nc_f_a + 1
!      else
!        f_d(indp+n4vd:indp+n4vd) = 'd'
!        nc_f_d = nc_f_a
!      end if

* Write displacement:

      write(*,'(1x,a)') 
     :   'Writing displacement output to file:', f_d(1:nc_f_d)

      iflag = 0 ! not used by current PC version of datetime; Chris
                ! Stephens would modify datetime.for to include Unix routines
      call datetime(date_c, time_c, iflag)
      ncomments = ncomments_a
      ncomments = ncomments + 1
      comments(ncomments) = '|'

      ncomments = ncomments + 1
      comments(ncomments) = 
     :      '| Displacement, from processing acc file '//f_a(1:nc_f_a)
      ncomments = ncomments + 1
      comments(ncomments) = 
     :      '| using program SMC2VD'

      ncomments = ncomments + 1
      comments(ncomments) = 
     :      '|   on '//date_c//' at time '//time_c

      if (itype_mean == 0) then
        ncomments = ncomments + 1
        write(comments(ncomments),'(a)') 
     :      '|   - no mean subtracted from record'
      else if (itype_mean == 1) then
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,1pe12.5,a)') 
     :      '|   - subtract mean = ', avg, ' from whole record'
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,f7.3,a,1x,f7.3)') 
     :      '|     mean determined from t = ', t4mean_strt, 
     :      ' to t = ', t4mean_stop
      else if (itype_mean == -1) then
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,1pe12.5,a)') 
     :      '|   - subtract mean = ', avg, ' from whole record'
        ncomments = ncomments + 1
        write(comments(ncomments),'(a)') 
     :      '|     mean was specified.' 
      else
        write(*,'(a)') ' Not supposed to be here.'
      end if

      ncomments = ncomments + 1
      write(comments(ncomments),'(a,1x,f7.3)') 
     :      '|   Initial displacement = ', d0

      ncomments = ncomments + 1
      comments(ncomments) = '|   resulting in output file '//
     :      f_d(1:nc_f_d)

      ncomments = ncomments + 1
      comments(ncomments) = '|'

      int_head(16) = ncomments 
      itype = 4

      call SMCWrite(f_d, precision,
     :                d, char_head, itype, 
     :                    int_head, real_head, comments)
     
 
      write(nu_sum,'(1x,i3, a, t35,a12, t49,a12, t61,a, 
     :       1p,2(1x,e10.3))') 
     :    num, ' "v, d files, v_last, d_last = ',
     :    f_v(1:nc_f_v), f_d(1:nc_f_d), '"', v(n_d_last), d(n_d_last)

      deallocate (a, v, d)

      END DO loop over files

* Close control and output files:

      close(nu_ctl)
      close(nu_sum)

      stop
      end
* ------------------------------------------------------------------- SMC2VD


      include '\forprogs\skip.for'
      include '\forprogs\skipcmnt.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\upstr.for'
      include '\forprogs\smc_npts.for'
      include '\forprogs\smcread.for'
      include '\forprogs\smcwrite.for'
      include '\forprogs\distaz.for'
      include '\forprogs\acc2vd.for'
      include '\forprogs\Trapezoidal_Integration.for'
      include '\forprogs\imnmax.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\rc_subs.for'
      include '\forprogs\datetime.for'
      include '\forprogs\rmvtrend.for'
      include '\forprogs\mnmaxidx.for'
      include '\forprogs\get_path_from_file_name.for'
