* -------------------------------------------------- smc_acc2psagm_vs_r
      Program smc_acc2psa_vs_r

* Reads 2 time series, filters, computes geometric-mean PSA, and writes into
* a file for plotting vs. r

* The program determines if the component is horizontal, so
* all files in a folder can be included in the list.

* The program computes 3d distances, using station lat, long
* from the smc file

* NOTE:  The file looks at char_head(1)(1:1) to see if the file is
* an acceleration, velocity, or response spectral file, and acts 
* appropriately.  I was tempted to simply compute the velocity and
* response spectrum on the fly, but if the data are from USGS or CGS,
* then the pads have been stripped from the data and I might have
* distorted results.

* If response spectra only, then do not ask for periods -1 and 0 (the
* program checks for this, however).

* Here is a sample control file:
*!Control file for program SMC_Acc2PSAGM_Vs_R
*!Name of summary file:
* smc_acc2psagm_vs_r.sum
*!Eqnum
* 999
*!Mag of quake
* 6.0
*!Number of Fault Planes:
*1
*!Fault orientation (ref lat,long,elev(m),depth, strike, dip, s1, s2, w1, w2):
* 36.03952 -120.60191 0.0 0.0 136.8 83.0 0.0 38.0 0.0 13.0
*!Name of output file:
* park04_bdsn_psa_vs_r.asc
*!rjb min for plot
* 0.1
*! flc, nslope_lc, fhc (0 = no filter), nslope_hc, icaus(1=causal) (nslope >= 2)
*!  (nslope >= 2,4,6, etc for causal and 4,8,12, etc for acausal; this is
*!   enforced in the program)
* 0.04 8  0.0  4  0
*!Time for taper at beginning and end of time series, before adding zeros
*! (Not used if no filter or a causal filter, but still need this parameter
*! (in the input file):
* 0.0 0.0
*!Damping value (need as placeholder even if only computing acceleration and/or velocity):
* 0.05
*!Number of periods:
*  6
*!Periods (<0 for pgv, 0 for pga):
* -1.0
* 0.0
* 0.1
* 0.2
* 1.0
* 3.0
*!t4mean (removes a mean from the whole record, where the mean is
*!    determined from t = 0 to t = t4mean.  If t4mean = 0.0,
*!    the mean of the whole record is used, and if t4mean < 0, no mean is
*!    removed.  Note that if working with a processed record, it is
*!    inadvisible to remove the mean as this may produce unintended
*!    distortions at long periods --- so in most cases set t4mean < 0.0):
* 0.0 
*!Sensor type (e.g., "acc" or "vel"):
* acc
*!ADC (e.g., "16", or "-" for not specified; treated as a character):
* 16
*! List of files to process:
*C:\parkfield_04\ms\bdsn\14094992.BK.PKD.HLE.sac.smc.a0p1.a.pad
*C:\parkfield_04\ms\bdsn\14094992.BK.PKD.HLN.sac.smc.a0p1.a.pad
*stop
  
* Dates: 07/06/05 - Written by D. Boore, patterned after  boat_psa_vs_r.for
*                   and smc2asc.for
*        07/11/05 - Add eqnum, amag, nper, etc to output
*        10/19/05 - Obtain stacode starting at column 1 of text header 3.
*        10/17/07 - Transparent change to user needed because smcread can
*                   read two input formats
*        05/10/08 - Write filename, component information
*        05/11/08 - Obtain v30rec from real_head(40)
*        05/13/08 - Simplify program, assuming input file is acceleration.
*                   Also, get rid of arrays for response variables and print
*                   within the loop of files.  Allow choice of skipping the
*                   files if a vertical component.  
*        05/13/08 - Allow for filtering of record before computing things
*        09/05/08 - Increase allowed number of output periods.
*        02/06/09 - Correct error in t4mean_strt and t4mean_stop being reset from their input
*                   values when processing a list of files.  The problem is that if a file with
*                   a shorter record length is processed before one with a longer record
*                   length, a mean supposedly determined for the whole record was determined
*                   only for the length of the shorter preceeding record.

 
      parameter(nper_max = 300)
      real per_smc(nper_max)
      real rd_smc(nper_max,5), rv_smc(nper_max,5), 
     :     psv_smc(nper_max,5), aa_smc(nper_max,5), damp_smc(5)
      real tprd(nper_max,5), tprv(nper_max,5), tpaa(nper_max,5)

      real alat_ref(10), along_ref(10), elev_ref(10), h_ref(10), 
     :     strike_f(10), dip_f(10), s1(10), s2(10), w1(10), w2(10),
     :     d_jb_segs(10), d_cd2f_segs(10), d_c_segs(10)
     
      real v30rec, d_jb4plot, d_jb, 
     :     d_cd2f4plot, d_cd2f
     

      character f_stainfo*80, stacode*8, buf*120, buf4*4, per_c*5
      character f_smc*80, f_sum*80, f_out*80, colhead(50)*8, frmt_c*600
      character buf_out*80, fname*80, skip_vert*10
      
      character sensor_c*10, adc_c*10
      character sensor_r*10, adc_r*10
      
      integer iclass, nc_stacode
      
      character char_head(11)*80, comments(500)*80, cmnts2skip(200)*80
      integer int_head(48)
      integer icomp_v, icomp_h, nc_f_smc_out
      real real_head(50)

      real a(:), y(:)
      real a_out(:), v_out(:), d_out(:) 
      allocatable :: a, y
      allocatable :: a_out, v_out, d_out
      real sd(50), psv(50), psa(50), 
     :     psa_out(50,2), gm_out(50),
     :     damp_ctl, per_out(50) 
      real stalat_in(2), stalong_in(2)
      real omega, omega2
 
      character f_ctl*40
      logical f_ctl_exist, f_smc_exist
      logical rmv_trnd

      logical zcross_l, zcross_t, use_first, use_last
      logical remove_mean_before, remove_mean_after
      logical apply_pads, save_padded_time_series, 
     :        replace_discarded_data_with_zeros
      
      twopi = 2.0*4.0*atan(1.)
      
        apply_pads = .true.
        replace_discarded_data_with_zeros = .true.
        zcross_l = .true.
        use_first = .true.
        zcross_t = .true.
        use_last = .true.
        remove_mean_before = .false.
        remove_mean_after = .false.
        save_padded_time_series = .false.
      

      call get_lun(nu_debug)
      open (unit=nu_debug, file='debug.out', 
     :      status='unknown')

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file '//
     :    '(cr=smc_acc2psagm_vs_r.ctl;'//
     :    ' ctl-brk to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl(1:4) .eq. '    ') 
     :      f_ctl = 'smc_acc2psagm_vs_r.ctl'
        call trim_c(f_ctl,nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')


*  Get the name of summary file:

      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      write(*,'(2a)') ' f_sum = ', f_sum
      call trim_c(f_sum, nc_f_sum)
      call get_lun(nu_sum)
      open (unit=nu_sum, file=f_sum(1:nc_f_sum), 
     :      status='unknown')
  
      
*  Get the eqnum (usually from NGA flatfile):
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl,*) ieqnum
       
*  Get the mag of quake (usually from NGA flatfile):
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl,*) amag_ctl
       
*  Get fault segment information:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl,*) n_f
      write(nu_sum,'(1x,a,1x,i3)') 'n_f', n_f
 
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      write(nu_sum,'(a)') 'alat_ref, along_ref, elev_ref, etc'
      do i = 1, n_f
        read(nu_ctl,*) alat_ref(i), along_ref(i), elev_ref(i), h_ref(i),
     :                 strike_f(i), dip_f(i), 
     :                 s1(i), s2(i), w1(i), w2(i)
        write(nu_sum,*) alat_ref(i),along_ref(i),elev_ref(i),h_ref(i),
     :                 strike_f(i), dip_f(i), 
     :                 s1(i), s2(i), w1(i), w2(i)
      end do

*  Get the name of output file:

      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      f_out = ' '
      read(nu_ctl,'(a)') f_out
      call trim_c(f_out, nc_f_out)
      write(*,'(2a)') ' f_out = ', f_out(1:nc_f_out)
      write(nu_sum,'(2a)') ' f_out = ', f_out(1:nc_f_out)
      call get_lun(nu_out)
      open (unit=nu_out, file=f_out(1:nc_f_out), 
     :      status='unknown')
     
*  Get minimum d_jb for plot:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl,*) d_jbmin
      write(nu_sum,'(1x,a,1x,f5.2)') ' d_jbmin = ', d_jbmin
      
* PARAMETERS FOR FILTERING:

      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
          do ihc = 1, nc_cmnts2skip
            write(nu_sum,'(1x,a)') cmnts2skip(ihc)
          end do
          read(nu_ctl,*) flc, nslope_lc, fhc, nslope_hc, icaus
          write(nu_sum,'(a)') ' Filter parameters upon input:'
          write(nu_sum,*) flc, nslope_lc, fhc, nslope_hc, icaus
          
* Force power of two and four for causal and acausal filters, respectively:

          if (icaus .eq. 0) then   ! acausal
            if (nslope_lc .lt. 4) nslope_lc = 4
            if (nslope_hc .lt. 4) nslope_hc = 4
            nslope_lc = 4 * int(float(nslope_lc)/4.0)
            nslope_hc = 4 * int(float(nslope_hc)/4.0)
            nroll_lc = nslope_lc/4
            nroll_hc = nslope_hc/4
          else                     ! causal
            if (nslope_lc .lt. 2) nslope_lc = 2
            if (nslope_hc .lt. 2) nslope_hc = 2
            nslope_lc = 2 * int(float(nslope_lc)/2.0)
            nslope_hc = 2 * int(float(nslope_hc)/2.0)
            nroll_lc = nslope_lc/2
            nroll_hc = nslope_hc/2
          end if
          
          write(nu_sum,'(a)') ' Filter parameters after enforcing'//
     :       ' power of two and four for causal and acausal filters:'
          write(nu_sum,'(a)') ' flc, nslope_lc, fhc, nslope_hc, icaus'
          write(nu_sum,*) flc, nslope_lc, fhc, nslope_hc, icaus
          write(nu_sum,'(a)') ' and nroll_lc, nroll_hc'
          write(nu_sum,*) nroll_lc, nroll_hc

* OPTION TO TAPER END OF TIME SERIES BEFORE ADDING LEADING AND
* TRAILING ZERO PADS

      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
          do ihc = 1, nc_cmnts2skip
            write(nu_sum,'(1x,a)') cmnts2skip(ihc)
          end do
          read(nu_ctl,*) taper_beginning, taper_end
          write(nu_sum,'(a)') 
     :                 ' taper_beginning, taper_end upon input:'
          write(nu_sum,*) taper_beginning, taper_end
 
 
* Get damping:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl,*) damp_ctl
      
* Get number of periods (<= 10):
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl,*) nper_out

* Get periods:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      do i = 1, nper_out
        read(nu_ctl,*) per_out(i)
      end do
      
* Get t4mean:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl,*) t4mean_in
       
* Get sensor type:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      sensor_c = ' '
      read(nu_ctl,'(a)') sensor_c
      call trim_c(sensor_c, nc_sensor_c)
        
* Get adc type:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      adc_c = ' '
      read(nu_ctl,'(a)') adc_c
      call trim_c(adc_c, nc_adc_c)
        
 
      write(nu_out,'(a)') '!Output of program smc_acc2psa_vs_r.for'
      write(nu_out,'(a)') '!using control file: '//f_ctl(1:nc_f_ctl)
      write(nu_out,'(a)') '!and summary file: '//f_sum(1:nc_f_sum)
      write(nu_out,'(a)') '!nper, periods:'
      write(nu_out, '(2x,i5)') nper_out
      write(nu_out, '(1x,50(1x,f7.3))') (per_out(i),i=1,nper_out)
      


* Set up and write column headers of output file:

      do i = 1, nper_out
        per_c = ' '
        write(per_c,'(f5.2)') per_out(i)
        call trim_c(per_c,nc_per_c)
        colhead(i) = ' '
        write(colhead(i)(1:8),'(a8)') 'PSA00000'
        colhead(i)(8-nc_per_c+1:8) = per_c(1:nc_per_c)
        write(nu_debug,'(a,i2,a)') ' colhead(', i, ') = '//colhead(i)
      end do
      
      frmt_c = ' '
      frmt_c = 
     : '(76x,a,5x,a,1x,a,'//
     : '1x,a,1x,a,1x,a,2x,a,2x,a,1x,a,1x,a,1x,a,'//
     : '3x,a,2x,a,'//                                    !67 characters 
     : 'xx(3x,a,5x,a))'                                  !xx are characters 68 and 69
      call trim_c(frmt_c,nc_frmt_c)
      write(frmt_c(68:69), '(i2)') nper_out
! If insert new variables, be careful about specifying column numbers
! corresponding to xx


      call trim_c(frmt_c,nc_frmt_c)
      
      write(nu_sum,'(a)') ' frmt_c = '//frmt_c(1:nc_frmt_c)


      write(nu_out,frmt_c(1:nc_frmt_c)) 
     :   'f_smc', 
     :   'sensor', 'adc', 
     :   'icompv', 'icomph',
     :   'eqnum', 'mag', 'stacode', 'iclass', 
     :    'v30used', 
     :    'f_low-cut',
     :    'd_jb', 'dcd2f',
     :    (colhead(i), 'logPSA', i = 1, nper_out)
     
* Loop over file names:

      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      
      nfiles = 0
      
1000  continue

      do i2 = 1, 2  ! loop over components
      
      buf = ' '
      read(nu_ctl,'(a)',end = 9999) buf
      call trim_c(buf,nc_buf)
      buf4 = ' '
      buf4 = buf(1:4)
      call upstr(buf4)
      if(buf4(1:4) .eq. 'STOP') go to 9999
      
      f_smc = ' '
      f_smc = buf(1:nc_buf)
      nc_f_smc = nc_buf     
      inquire(file=f_smc(1:nc_f_smc), exist=f_smc_exist)
 
      if (.not. f_smc_exist) then
          write(nu_sum,'(a)') ' ****** FILE '//f_smc(1:nc_f_smc)// 
     :          ' DOES NOT EXIST; SKIPPING!!! ******* '
          go to 1000
      end if

* Get various information from headers:

      write(nu_sum, '(a)') ' Processing file '//f_smc(1:nc_f_smc) 

      call ReadHdrs(f_smc(1:nc_f_smc), 
     :                char_head, int_head, real_head, comments)

      inull = int_head(1)
      rnull = real_head(1)
      
* If requested, skip vertical component:
      
      skip_vert = 'Y'
      if (skip_vert .eq. 'Y') then
        if (int_head(13) .ne. inull .and. 
     :    int_head(13) .ne. 90) then     ! must be vertical
        write(nu_sum, '(3x,a,1x,i5)') 'int_head(13) = ', int_head(13)
        write(nu_sum, '(3x,a)') 
     :        ', not a horizontal component; SKIPPING to next record'
        go to 1000
        end if
      end if

* Skip if station coordinates are missing:

      stalat_in(i2) = real_head(11)
      stalong_in(i2) = real_head(12)
      if (stalat_in(i2)  .eq. rnull .or.
     :    stalong_in(i2) .eq. rnull) then
        write(nu_sum, '(3x,a,1p,2(1x,e10.3))') 
     :    'stalat, stalong = ', stalat_in(i2), stalong_in(i2)
        write(nu_sum, '(3x,a)') 
     :        ', a null value; SKIPPING to next record'
        go to 1000
      end if

      nfiles = nfiles + 1

* Get npts and allocate the array:

        call smc_npts(f_smc(1:nc_f_smc), npts, sps)
        write(nu_sum,'(a,1x,i6)') '   After smc_npts, npts = ', npts
      
 
        allocate (a(npts))

* Read the input file:

        do i = 1, npts
          a(i) = 0.0
        end do

        tskip = 0.0
        tlength = -1.0
      
        call SMCRead(f_smc(1:nc_f_smc), tskip, tlength,
     :               a, char_head, int_head, real_head, comments)
     
        icomp_v = int_head(13)
        icomp_h = int_head(14)
     
      
        stacode = ' '
        stacode = char_head(3)(1:8)
!        stacode  = char_head(3)(2:7)
        call trim_c(stacode ,nc_stacode )
        if (nc_stacode .eq. 0) then
          stacode = 'null'
          call trim_c(stacode,nc_stacode)
        end if
        
        v30rec = real_head(40)
          
      
        sps = real_head(2)
        dt = 1.0/sps
        ncomments = int_head(16)
        tend = float(npts-1)/sps
        
        write(nu_sum,'(a)') '  sps, dt, ncomments, tend, t4mean_in = '
        write(nu_sum, * )      sps, dt, ncomments, tend, t4mean_in
        
        ru = -999.0
      
* Need to allocate a_out, v_out, d_out differently because
* more than npts_max might be used in acausal filter routines (those
* routines pad the array beyond npts).  Must do this even if choose not to
* add pads when do an acausal filter.

        if (flc .eq. 0.0 .and. fhc .eq. 0.0) then   ! no filtering
          npad = 0      
        else                                        ! filtering
          if (icaus .eq. 1) then
            npad = 0                                ! causal, no pads
          else
            if (flc .eq. 0.0) then                  ! hicut implied
              npad = 3.0*float(nroll_hc)/(fhc*dt)
            else if (fhc .eq. 0.0) then             ! locut implied
              npad = 3.0*float(nroll_lc)/(flc*dt)
            else                                    ! band implied
              nroll_bp = max0(nroll_lc, nroll_hc)
              nslope_bp = max0(nslope_lc, nslope_hc)
              npad1=3.0*float(nroll_lc)/(flc*dt)
              npad2=6.0*float(nroll_bp)/((fhc-flc)*dt)
              npad = max0(npad1, npad2)
            end if
          end if
        end if

        nlz = npad/2
        ntz = npad/2
        izcl = nlz
        izct = ntz
        npts4alloc = npts + nlz + ntz + npad + 2

* extra trailing pad of npad because the filter subroutines add their own 
* zero pad equal to npad to the end of the time series.  If no integration was
* being done, then that would be OK, but I want to add npad/2 to the front so 
* that the integration accounts properly for the filter transient. 
* I first added only an extra npad/2, but this led to a hard-to-find run time
* error.

        allocate (a_out(npts4alloc))
        allocate (v_out(npts4alloc))
        allocate (d_out(npts4alloc))

        do i = 1, npts4alloc
          a_out(i) = 0.0
          v_out(i) = 0.0
          d_out(i) = 0.0
        end do
 
* Step : Remove a mean from the whole record

        t4mean = t4mean_in

*  Determine the appropriate mean:
        if(t4mean .lt. 0.0) then  ! do not remove a mean
          avg = 0.0
        else
          if (t4mean .eq. 0.0 .or. t4mean .gt. tend) then   
            t4mean = tend  ! use whole record
          end if
          n_t4mean = sps * t4mean + 1.1
          cum_avg = 0.0
          do i = 1, n_t4mean
            cum_avg = cum_avg + dble(a(i))
          end do
          avg = cum_avg/float(n_t4mean)
*        avg = sngl(cum_avg/n_t4mean)
*  Remove the mean from the whole record:
         write(nu_sum,'(a,1x,1p,e10.3)') ' avg = ', avg
        end if
        
        do i = 1, npts
          a_out(i) = a(i) - avg
        end do
 
        deallocate (a)
        
           
* Step : PAD TIME SERIES IF USE ACAUSAL FILTER

        if ((flc .ne. 0.0 .or. fhc .ne. 0.0) 
     :         .and. icaus .eq. 0
     :         .and. apply_pads) then  ! acausal filter, apply pads

          write(nu_sum,'(a)') 
     :          ' Inside pad time series if acausal filter:'
          write(nu_sum,'(a)') ' Working on file '//f_smc(1:nc_f_smc)
          write(nu_sum,'(a)') 
     :                 ' taper_beginning, taper_end upon input:'
          write(nu_sum,*) taper_beginning, taper_end
 
          call smcpadf(a_out, npts, sps, nlz, ntz, 
     :      replace_discarded_data_with_zeros, 
     :      taper_beginning, taper_end, 
     :      npts_out, izcl, izct,  
     :      remove_mean_before, remove_mean_after, 
     :      zcross_l, use_first, zcross_t, use_last,
     :      comments, ncomments)    

* Store new ncomments, npts:

          int_head(16) = ncomments            

        else

          nlz = 0
          ntz = 0
          izcl = 1
          izct = npts
          npts_out = npts

        end if

* Step : FILTER

        if (flc .ne. 0.0 .or. fhc .ne. 0.0) then

          call filter(a_out, npts_out, dt, flc, nroll_lc, 
     :              fhc, nroll_hc, icaus)
        end if


* Step: compute response spectra, pgv, pga
 
* Loop over periods and damping, computing and writing response spectra:
        do iper = 1, nper_out
        

          write(*,'(a,1x,i2,1x,f7.3)') ' iper, per_out(iper) = ',
     :                                    iper, per_out(iper)

          buf_out = ' '
          buf_out = '+ Patience! Computing rd for ts '//
     :      f_smc(1:nc_f_smc)//' and period '
          write(buf_out(60:67),'(f8.3)') per_out(iper) 
          write(*,'(a)') buf_out
          if (per_out(iper) .lt. 0.0) then  ! pgv
            v0 = 0.0
            d0 = 0.0
            rmv_trnd = .false.
            call acc2vd(a_out, npts_out, dt, rmv_trnd, v0, d0, 
     :                  v_out, d_out)
            call abs_mnmaxidx(v_out, 1, npts_out, 1,
     :        vmin, vmax, indxpgv_min, indxpgv_max, 
     :        psa_out(iper,i2)) 
     
            write(nu_sum, '(a,1x,1p,e10.3)') 
     :        ' pgv = ', psa_out(iper,i2)
            
          else if (per_out(iper) .eq. 0.0) then  ! pga
            call abs_mnmaxidx(a_out, 1, npts_out, 1,
     :             amin, amax, indxpga_min, indxpga_max, 
     :             psa_out(iper,i2)) 

            write(nu_sum, '(a,1x,1p,e10.3)') 
     :        ' pga = ', psa_out(iper,i2)
            
          else ! rs
            omega = twopi/per_out(iper)
            omega2 = omega**2
            d0=0.0
            v0=0.0
            call rdrvaa(a_out, npts_out, omega, 
     :                   damp_ctl, dt, 
     :                   sd(iper), rv, aa, d0, v0)
            psv(iper) = omega*sd(iper) 
            psa_out(iper,i2) = omega2*sd(iper) 
            
            write(nu_sum, '(a,1x,f7.3,1p,1x, e10.3)') ' per, psa = ', 
     :         per_out(iper), psa_out(iper,i2)
            
          end if
          
        end do ! loop over nperiods
      
        deallocate (a_out, v_out, d_out)         
       
      end do  ! end loop over components

* Check for equality of stalat, stalong:

      if (stalat_in(1) .ne. stalat_in(2) .or. 
     :    stalong_in(1) .ne. stalong_in(2)) then
         print *,' ERROR, stalat or stalong not same;'
         print *,' file 2 = ', f_smc(1:nc_f_smc)
         print *,' QUITTING!!!'
         close(nu_ctl)
         close(nu_sum)
         stop
      end if
      
* Loop over periods, computing and writing geometric mean response spectra:

      do iper = 1, nper_out        
        gm_out(iper) = sqrt(psa_out(iper,1)*psa_out(iper,2))
      end do
        
* Compute distance      
       
      staelev = 0.0
      h_min_c = 3.0
      
      stalong = stalong_in(2)
      stalat = stalat_in(2)
      
      call dist3dmf(stalat, stalong, staelev,
     : alat_ref, along_ref, elev_ref, h_ref, strike_f, dip_f,
     : w1, w2, s1, s2, n_f, h_min_c, d_jb, az_jb, 
     : d_cd2f, az_cd2f, 
     : d_c, az_c, d_jb_segs, d_cd2f_segs, d_c_segs)
     
      write(nu_sum,'(a)') ' d_jb, d_cd2f = '
      write(nu_sum, * ) d_jb, d_cd2f
     
      if (d_jb .lt. d_jbmin) then
        d_jb4plot  = d_jbmin
      else
        d_jb4plot  = d_jb 
      end if

      if (d_cd2f .lt. d_jbmin) then
        d_cd2f4plot = d_jbmin
      else
        d_cd2f4plot = d_cd2f
      end if

* Write results to output file

      iclass = 0
      
! right justify some output variables:

        fname = ' '
        fname(81-nc_f_smc:80) = 
     :          f_smc(1:nc_f_smc)
     
        sensor_r = ' '
        sensor_r(11-nc_sensor_c:10) = 
     :          sensor_c(1:nc_sensor_c)
        
        adc_r = ' '
        adc_r(4-nc_adc_c:3) = 
     :          adc_c(1:nc_adc_c)
        
        if (v30rec .eq. real_head(1)) then
           v30_out = 0.0
        else
           v30_out = v30rec
        end if
        
        write(nu_out,'(
     :        1x,a80,
     :        1x,a10, 1x,a3,
     :        1x,i6, 1x,i6,
     :        1x,i5, 1x,f4.2, 1x,a8, 6x,i1, 
     :        2x,f6.1,
     :        4x,f6.3,
     :        1x,f6.1, 1x,f6.1, 
     :        50(1p,1x,e10.3,1x,e10.3))') 
     :        fname, 
     :        sensor_r, adc_r,
     :        icomp_v, icomp_h,
     :        ieqnum, amag_ctl, stacode(1:nc_stacode), iclass,
     :        v30_out,
     :        flc,
     :        d_jb4plot, d_cd2f4plot, 
     :        ( gm_out(i), 
     :          alog10(gm_out(i) ), i = 1, nper_out )
      

* Go back for a new file:
     
      go to 1000

9999  continue
  

* Close control and output files:

      close(nu_ctl)
 
      close(nu_out)
      close(nu_sum)
      close(nu_debug)
      
      stop

      end
* -------------------------------------------------- smc_acc2psagm_vs_r

 

      include '\forprogs\acc2vd.for'
      include '\forprogs\Trapezoidal_Integration.for'
      include '\forprogs\dist3dmf.for'
      include '\forprogs\deg2km_f.for'      
      include '\forprogs\rmvtrend.for'     
      include '\forprogs\rdrvaa.for'
      include '\forprogs\abs_mnmaxidx.for'     
      include '\forprogs\mnmaxidx.for'
      include '\forprogs\skipcmnt.for'
      include '\forprogs\skip.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\smcread.for'
      include '\forprogs\smc_npts.for'
      include '\forprogs\rs2Rdhdr.for'
      include '\forprogs\rs2Read.for'
      include '\forprogs\fs2Read.for'
      include '\forprogs\readhdrs.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\upstr.for'
      include '\forprogs\yintrf.for'
      include '\forprogs\BJF94V30Amp.for'      
      include '\forprogs\lin_interp.for'      
      include '\forprogs\locate.for'      
 
      include '\forprogs\rcc.for'
      include '\forprogs\rcf.for'
      include '\forprogs\rci.for'
      include '\forprogs\tabs_rmv.for'

      include '\gm_predictions\bjf97.for'
      
      include '\forprogs\smcpadf.for'
      include '\forprogs\filter.for'
      include '\forprogs\band.for'
      include '\forprogs\hicut.for'
      include '\forprogs\locut.for'
      include '\forprogs\notch.for'
      include '\forprogs\get_avg.for'

  
